<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Repository\MongoRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;

class MigrationAclController_Cli extends AbstractController_Cli
{

    public function updateAction()
    {
        ob_end_clean();

        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        echo 'processing users ...', PHP_EOL;
        foreach (UserRepository::findAll() as $user) {
            $user->setPortalIds($user->getPortalIds());

            UserRepository::persist($user);
        }
        UserRepository::flush();

        echo 'processing portals ...', PHP_EOL;
        foreach (PortalRepository::findAll() as $portal) {
            $portal->setAdminIds($portal->getAdminIds());
            $portal->setReviewerIds($portal->getReviewerIds());

            PortalRepository::persist($portal);
        }
        PortalRepository::flush();

        echo 'processing researches ...', PHP_EOL;
        foreach (ResearchRepository::findAll() as $key => $research) {
            $research->setOwner($research->getOwner());
            $research->setAuthor($research->getAuthor());
            $research->setCoAuthors($research->getCoAuthors());
            $research->setContributors($research->getContributors());
            $research->setSupervisors($research->getSupervisors());
            $research->setEditors($research->getEditors());
            $research->setAccessLevel($research->getAccessLevel());

            if ($research instanceof Application) {
                $research->setStatus($research->getStatus());
                $research->setWorkIds($research->getWorkIds());
                $research->setExpositionIds($research->getExpositionIds());
                $research->setReviewerIds($research->getReviewerIds());
                $research->setExternalReaderIds($research->getExternalReaderIds());
            } elseif ($research instanceof Exposition) {
                $research->setStatus($research->getStatus());
                $research->setReviewerIds($research->getReviewerIds());
            }

            ResearchRepository::persist($research);
        }

        ResearchRepository::flush();

        echo 'persisting all ...', PHP_EOL;
        MongoRepository::flush();
    }

}