<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
/**
 * Created by PhpStorm.
 * User: Muehlschlegel
 * Date: 31.05.2015
 * Time: 16:31
 */
class NewsletterController_Cli extends AbstractController_Cli
{

    public function sendAction()
    {
        if (!config('app.allow.messaging')) {
            Log::debugNewsletter('skipped (messaging disabled)');
            return;
        }

        foreach (NewsletterRepository::findNotCompleted() as $newsletter) {
            if ($newsletter->getStarted()) {
                Log::debugNewsletter($newsletter->getId(), 'resumed at', substr_count($newsletter->getSucceededEmails(), ','));
            } else {
                $newsletter->setStarted(time());
                NewsletterRepository::update($newsletter);

                Log::debugNewsletter($newsletter->getId(), 'started');
            }

            foreach (explode(',', $newsletter->getTargetEmails()) as $targetEmail) {
                if (strpos($newsletter->getSucceededEmails(), $targetEmail) !== false) {
                    continue;
                }
                if (NewsletterRepository::countByEmailAndContent($targetEmail, $newsletter->getHashedContent()) > 0) {
                    continue;
                }

                try {
                    $mail = $newsletter->generateMailObject($targetEmail);
                    $mail->send();

                    $newsletter->setLastUpdated(time());
                    $newsletter->addSucceededEmail($targetEmail);
                    NewsletterRepository::update($newsletter);
                } catch (Exception $e) {
                    $newsletter->setLastUpdated(time());
                    $newsletter->addFailedEmail($targetEmail);
                    NewsletterRepository::update($newsletter);

                    Log::errNewsletter($newsletter->getId(), 'failed', $targetEmail);
                }
            }

            $newsletter->setEnded(time());
            NewsletterRepository::update($newsletter);

            $this->sendStatisticMail($newsletter);

            Log::debugNewsletter($newsletter->getId(), 'ended');
        }
    }

    protected function sendStatisticMail(Newsletter $newsletter)
    {
        $content = 'This is an automated generated message to inform you about the results of the mail delivery.<br/>';
        $content .= 'For further information please contact <a href="mailto:support@researchcatalogue.net">support@researchcatalogue.net</a>.<br/>';
        $content .= '<br/>';
        $content .= 'Title: ' . $newsletter->getSubject() . '<br/>';
        $content .= 'Type: ' . $newsletter->getTarget() . '<br/>';
        $content .= 'Target: ' . $newsletter->getScope() . '<br/>';
        $content .= 'Recipients: Members of portal "' . $newsletter->getPortal()->getName() . '"<br/>';
        $content .= '&nbsp;&nbsp;Total: ' . number_format($newsletter->getTargetEmailCount(), 0, ',', '.') . '<br/>';
        $content .= '&nbsp;&nbsp;Succeeded: ' . number_format($newsletter->getSucceededEmailCount(), 0, ',', '.') . '<br/>';
        $content .= '&nbsp;&nbsp;Skipped: ' . number_format($newsletter->getSkippedEmailCount(), 0, ',', '.') . '<br/>';
        $content .= '&nbsp;&nbsp;Failed: ' . number_format($newsletter->getFailedEmailCount(), 0, ',', '.') . '<br/>';
        $content .= 'Created: ' . date('m/d/Y - H:i:s', $newsletter->getCreated()) . '<br/>';
        $content .= 'Started: ' . date('m/d/Y - H:i:s', $newsletter->getStarted()) . '<br/>';
        $content .= 'Ended: ' . date('m/d/Y - H:i:s', $newsletter->getEnded()) . '<br/>';
        $content .= '<br/>';
        $content .= 'Creator: ' . $newsletter->getUser()->getName() . ' (' . $newsletter->getUserid() . ')<br/>';

        $mail = new Mail();
        $mail->addTo($newsletter->getUser()->getEmail());
        $mail->addBcc('rc@muehlschlegel.info');
        $mail->setFrom(config('app.email'));
        $mail->setSubject('RC: Statistics to mail delivery of "' . $newsletter->getSubject() . '"');
        $mail->setBodyHtml($content, 'UTF-8');
        $mail->buildHtml();
        $mail->send();

        Log::debugNewsletter($newsletter->getId(), 'statistics');
    }
}
