<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class CommentController extends BasicControllerAction
{

    public function addAction()
    {
        $form = $this->_getForm();
        $research = $this->_getRequestedResearch();

        if ($form->isPostAndValid()) {
            $comment = new Comment();
            $comment->setValues($form->getValues());
            $comment->setUserId(Identity::getUserId());
            $comment->setResearchId($research->getId());
            CommentRepository::add($comment);

            EntityChangeRepository::addChange($research, EntityChange::TYPE_COMMENT);

            $this->_setSuccessHeader();

            $this->forward('list');
        } else {
            $this->view->form = $form;
        }
    }

    public function removeAction()
    {
        $comment = $this->_getRequestedComment();
        $form = $this->_getForm($comment);

        if ((!Identity::getUser()->isAdmin() && $comment->isDeleted()) ||
            (!Identity::getUser()->isAdmin() && $comment->getUser()->getId() != Identity::getUserId() && $comment->getResearch()->getOwner()->getId() != Identity::getUserId())
        ) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {

            if (Identity::getUser()->isAdmin() && $form->getValue('completely')) {
                CommentRepository::remove($comment);
            } else {
                $comment->setValues($form->getValues());
                CommentRepository::update($comment);
            }

            $this->_setSuccessHeader();
            $this->forward('list', null, null, ['research' => $comment->getResearchId()]);
        } else {
            $this->view->form = $form;
        }
    }

    public function convertAction()
    {
        $comment = $this->_getRequestedComment();
        $form = $this->_getForm($comment);

        if ($comment->isDeleted() || !Identity::getUser()->isAdmin()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            // convert comment to report
            $report = unserialize('O:6:"Report"' . substr(serialize($comment), 13));
            $report->setId(null);
            $report->setPortalId($form->getValue('portalid'));

            ReportRepository::add($report);
            CommentRepository::remove($comment);

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function listAction()
    {
        if (Identity::exists()) {
            $this->view->loggedin = true;
            $this->view->isAdmin = Identity::getUser()->isAdmin();
            $this->view->currentUser = Identity::getUser();
        }
        $this->view->research = $this->_getRequestedResearch();
        $this->view->comments = CommentRepository::findByResearch($this->_getRequestedResearch(), 0);
        $this->view->showComments = $this->_getParam('showComments');
    }

}