<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class ErrorController extends Zend_Controller_Action
{

    public function errorAction()
    {
        $errors = $this->_getParam('error_handler');

        if (!$errors) {
            $this->view->message = 'You have reached the error page';
            return;
        }

        if (Identity::exists()) {
            $identity = Identity::getUserId();
        } else {
            $identity = null;
        }

        switch ($errors->type) {
            case Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_ROUTE:
            case Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_CONTROLLER:
            case Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_ACTION:

                // 404 error -- controller or action not found
                $this->getResponse()->setHttpResponseCode(404);
                $this->view->message = 'Page not found';

                Log::errPageNotFound($identity, $this->view->message, $errors->exception);
                break;
            default:
                // application error
                $this->getResponse()->setHttpResponseCode(500);
                $this->view->message = 'Application error';

                $info = [];
                if (PHP_SAPI == 'cli') {
                    $info[] = 'cli: ' . implode(' ', $_SERVER['argv']);
                } else {
                    $info[] = 'url: ' . config('app.url') . $_SERVER['REQUEST_URI'];
                    if (!empty($_SERVER['HTTP_REFERER'])) {
                        $info[] = 'referer: ' . $_SERVER['HTTP_REFERER'];
                    }
                }
                if (Identity::exists()) {
                    $info[] = 'identity: ' . Identity::getUserId();
                }
                Log::errApplicationError($identity, implode(' , ',$info) . "\t" . $this->view->message, $errors->exception);
                break;
        }

        $this->view->exception = $errors->exception;
        $this->view->request = $errors->request;
    }

    public function getLog()
    {
        $bootstrap = $this->getInvokeArg('bootstrap');
        if (!$bootstrap->hasResource('Log')) {
            return false;
        }
        $log = $bootstrap->getResource('Log');
        return $log;
    }

}