<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\SimpleMediaFactory;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\ResearchRepository;

class FileController extends BasicControllerAction
{

    public function listAction()
    {
        $work = $this->_getRequestedWork();
        $this->view->files = $work->getMediaList();
        $this->view->work = $work;
        $this->view->isOwner = $work->isOwner(Identity::getUser());
    }

    public function addAction()
    {
        $work = $this->_getRequestedWork();
        $form = $this->_getForm();

        if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form);
        } else {
            $formToUse = $form;
        }

        if ($formToUse->isPostAndValid()) {
            $simpleMedia = SimpleMediaFactory::create($formToUse->getValue('mediatype'));
            $simpleMedia->setOwner(Identity::getUser());
            $simpleMedia->setResearch($work);
            $simpleMedia->setValues($formToUse->getValues());

            SimpleMediaRepository::persist($simpleMedia);

            $work->addFile($simpleMedia);
            ResearchRepository::persist($work);

            $this->_setFormActionHeader($simpleMedia);
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function editAction()
    {
        $simpleMedia = $this->_getRequestedFile();
        $form = $this->_getForm($simpleMedia)->setDefaultsFrom($simpleMedia);

        if ($simpleMedia->getMediaType()) {
            $formToUse = $form->getTab($simpleMedia->getMediaType());
        } else if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form);
        } else {
            $formToUse = $form;
        }

        if ($formToUse->isPostAndValid()) {
            $simpleMedia->setValues($formToUse->getValues());
            SimpleMediaRepository::persist($simpleMedia);

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->file = $simpleMedia;
        }
    }

    public function removeAction()
    {
        $file = $this->_getRequestedFile();
        if (!$file->isOwner(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($file);
        if ($form->isPostAndValid()) {
            SimpleMediaRepository::remove($file);

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

}