<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use GuzzleHttp\Exception\ClientException;
use Rc\Domain\Model\User\Setting\MadekRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Service\Madek\MadekServiceFactory;
use Rc\Service\MadekService;

class MadekMediaController extends BasicControllerAction
{
    /**
     * @var MadekService
     */
    protected $madekService;

    /**
     * @throws Exception
     */
    public function init()
    {
        parent::init();

        $this->madekService = MadekServiceFactory::create();
    }

    /**
     * @throws Exception
     */
    public function authAction()
    {
        $user = Identity::getUser();

        $madekRepository = new MadekRepository();
        $madekRepository->setToken($this->getParam('token'));

        $madekService = new MadekService(['auth' => ['', $madekRepository->getToken()]]);

        try {
            $madekService->userId();

            $user->getSettings()->setRepository($madekRepository);
            UserRepository::persist($user);
        } catch (ClientException $exception) {
            $this->view->error = 'Authentication failed: the provided token was declined!';
        }
    }

    /**
     * @throws Exception
     */
    public function logoutAction()
    {
        $this->_disableView();

        $user = Identity::getUser();
        $user->getSettings()->removeRepository(MadekRepository::class);

        UserRepository::persist($user);
    }

    /**
     * @throws GatewayException
     * @throws Exception
     */
    public function listAction()
    {
        if (!$this->madekService->auth()) {
            header('X-Repository-Auth: 0');
            $this->view->files = [];
        } else {
            $this->view->files = $this->madekService->mediaEntities();
        }
    }

}

?>