<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Repository\MessageRepository;

class MessengerController extends BasicControllerAction
{

    public function init()
    {
        $this->initPortal();
    }

    public function indexAction()
    {
        $this->forward('list');
    }

    public function listAction()
    {
        $this->view->assign([
            'incomingMessages'        => MessageRepository::findByReceiver(Identity::getUser(), [
                'page'  => $this->_getParam('imp', 0),
                'limit' => 25,
            ]),
            'incomingMessagePager'    => $this->_getParam('imp', 0),
            'maxIncomingMessagePager' => ceil(MessageRepository::countByReceiver(Identity::getUser()) / 25) - 1,
            'outgoingMessages'        => MessageRepository::findBySender(Identity::getUser(), [
                'page'  => $this->_getParam('omp', 0),
                'limit' => 25,
            ]),
            'outgoingMessagePager'    => $this->_getParam('omp', 0),
            'maxOutgoingMessagePager' => ceil(MessageRepository::countBySender(Identity::getUser()) / 25) - 1,
        ]);
    }

    public function addAction()
    {
        $form = $this->_getForm()->setDefaults($this->getAllParams());

        if ($form->isPostAndValid()) {
            foreach ((array)$form->getValue('receiverids') as $receiverId) {
                $message = new \Rc\Domain\Model\Message();
                $message->setValues($form->getValues());
                $message->setReceiverId($receiverId);
                $message->setSenderId(Identity::getUserId());

                $this->_sendMessage($message);
            }

            $this->_setSuccessHeader();
            $this->forward('list');
        } else {
            $this->view->form = $form;
        }
    }

    public function readSelectedAction()
    {
        $messages = $this->_getRequestedMessages();
        $form = $this->_getForm($messages);

        if ($form->isPostAndValid()) {
            foreach ($messages as $message) {
                if ($message->getReceiverId() != Identity::getUserId() && $message->getSenderId() != Identity::getUserId()) {
                    continue;
                }
                if ($message->isRead()) {
                    continue;
                }

                $message->setRead(true);
                MessageRepository::persist($message);
            }

            $this->_setSuccessHeader();
            $this->forward('list');
        } else {
            $this->view->form = $form;
        }
    }

    public function removeSelectedAction()
    {
        $messages = $this->_getRequestedMessages();
        $form = $this->_getForm($messages);

        if ($form->isPostAndValid()) {
            foreach ($messages as $message) {
                if ($message->getReceiverId() != Identity::getUserId() && $message->getSenderId() != Identity::getUserId()) {
                    continue;
                }

                if ($message->getReceiverId() == Identity::getUserId()) {
                    MessageRepository::removeForReceiver($message);
                } else {
                    MessageRepository::removeForSender($message);
                }
            }

            $this->_setSuccessHeader();
            $this->forward('list');
        } else {
            $this->view->form = $form;
        }
    }

    public function showAction()
    {
        $message = $this->_getRequestedMessage();
        $form = $this->_getForm($message);

        if ($message->getReceiverId() != Identity::getUserId() && $message->getSenderId() != Identity::getUserId()) {
            throw new InsufficientRightsException();
        }

        if ($message->getReceiverId() == Identity::getUserId() && !$message->isRead()) {
            $message->setRead(true);
            MessageRepository::persist($message);
        }

        $this->view->form = $form;
        $this->view->message = $message;
        $this->view->currentUser = Identity::getUser();
        $this->view->systemUser = \Rc\Domain\Repository\UserRepository::findSystemAccount();
    }

    public function removeAction()
    {
        $message = $this->_getRequestedMessage();
        $form = $this->_getForm($message);

        if ($message->getReceiverId() != Identity::getUserId() && $message->getSenderId() != Identity::getUserId()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            if ($message->getReceiverId() == Identity::getUserId()) {
                MessageRepository::removeForReceiver($message);
            } else {
                MessageRepository::removeForSender($message);
            }

            $this->_setSuccessHeader();
            $this->forward('list');
        } else {
            $this->view->form = $form;
            $this->view->message = $message;
        }
    }

}