<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Metrics\Metrics;
use Rc\Utility\ResearchUtility;

class MetricsController extends BasicControllerAction
{

    public function indexAction()
    {
        try {
            $this->_disableView();

            $key = $this->getParam('k');
            $value = $this->getParam('v');
            $tags = $this->getParam('t', []);

            if (!$this->isValidRequest($key, $value, $tags)) {
                Metrics::increment('metrics.invalid_requests', 1);
                return;
            }

            switch ($this->getParam('m')) {
                case 'c':
                    Metrics::increment($key, $value, $tags);
                    break;
                case 't':
                    Metrics::timing($key, $value, $tags);
                    break;
            }
        } finally {
            // silently ignore all exceptions
            return;
        }
    }

    public function visitDurationAction()
    {
        $research = $this->_getRequestedResearch();

        $params = $this->getAllParams();
        $params['t'] = ResearchUtility::expositionMetricTags($research, Identity::getUserOrNull());

        $this->forward('index', null, null, $params);
    }

    /**
     * @param string $key
     * @param int|float $value
     * @param array $tags
     * @return bool
     */
    protected function isValidRequest($key, $value, array $tags)
    {
        if (!$this->_request->isPost()) {
            return false;
        }

        if (!preg_match('~^[A-Z\.\_]+$~i', $key)) {
            return false;
        }

        if (!is_numeric($value)) {
            return false;
        }

        foreach ($tags as $tagKey => $tagValue) {
            if (!preg_match('~^[A-Z\_]+$~i', $tagKey)) {
                return false;
            }

            if (!is_numeric($tagValue) && !preg_match('~^[A-Z\_]+$~i', $tagKey)) {
                return false;
            }
        }

        return true;
    }

}

?>