<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\SimpleMedia\ArchiveSimpleMedia;
use Rc\Domain\Repository\Application\ApplicationProgramRepository;
use Rc\Domain\Repository\Research\ApplicationRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;

class ProfileController extends BasicControllerAction
{

    public function init()
    {
        $this->initPortal();
    }

    public function indexAction()
    {
        if ($this->hasParam('person')) {
            $this->forward('show');
        } else {
            $this->forward('show-own');
        }
    }

    public function mediaAction()
    {
        $this->_disableLayout();
        $this->_disableView();

        echo $this->view->profile()->renderMedia($this->_getRequestedEntity());
    }

    public function showOwnAction()
    {
        $this->view->descriptionMaximum = 1;
        $this->view->user = Identity::getUser();
        $this->view->person = Identity::getUser();

        $ids = [];
        foreach (ProfileShortcutRepository::findByUserId(Identity::getUserId()) as $shortcut) {
            $ids[] = $shortcut->getEntityId();
        }
        $this->view->shortcuts = ResearchRepository::findByIds($ids);

        $comments = [];
        foreach (CommentRepository::findByPerson(Identity::getUser(), 0, null, ['created' => 'DESC']) as $comment) {
            $comments[$comment->getCreated()] = $comment;
        }
        foreach (ReportRepository::findByPerson(Identity::getUser(), 0, null, ['created' => 'DESC']) as $report) {
            $comments[$report->getCreated()] = $report;
        }
        krsort($comments);
        $this->view->comments = $comments;

        $this->view->applicationProgramList = ApplicationProgramRepository::findEligibleByUser(Identity::getUser());
        $this->view->readableApplications = ApplicationRepository::findByUserForReading(Identity::getUser());
    }

    public function showAction()
    {
        try {
            $user = $this->_getRequestedPerson();
        } catch (Exception $e) {
            $this->redirect('/portal');
            return;
        }

        if ($user->isHidden()) {
            $this->redirect('/portal');
            return;
        }

        if ($user->getDeleted() && (!Identity::exists() || !Identity::isSuperAdmin())) {
            $this->redirect('/portal');
            return;
        }

        $this->view->descriptionMaximum = 1;
        $this->view->loggedin = Identity::exists();
        if (Identity::exists()) {
            $this->view->currentPerson = Identity::getUser();
        }
        $this->view->person = $user;

        $comments = [];
        foreach (CommentRepository::findByPerson($user, 0, null, ['created' => 'DESC']) as $comment) {
            $comments[$comment->getCreated()] = $comment;
        }
        foreach (ReportRepository::findByPerson($user, 0, null, ['created' => 'DESC']) as $report) {
            $comments[$report->getCreated()] = $report;
        }
        krsort($comments);
        $this->view->comments = $comments;

        $this->layout->basic->seo = $this->view->seo()->user($user);
    }

    public function editAction()
    {
        $user = Identity::getUser();
        $form = $this->_getForm($user)->setDefaultsFrom($user);

        if ($form->isPostAndValid()) {
            $user->setValues($form->getValues());
            UserRepository::persist($user);

            $this->_setSuccessHeader();
            $this->forward('show-own');
        } else {
            $this->view->form = $form;
        }
    }

    public function portalMembershipEditAction()
    {
        $user = Identity::getUser();
        $form = $this->_getForm($user)->setDefaults([
            'portalids' => array_merge($user->getPortalIds(), $user->getPendingPortalIds()),
        ]);

        if ($form->isPostAndValid()) {
            $portalIds = $form->getValue('portalids');

            $this->_handlePortalMembership(Identity::getUser(), $portalIds);

            UserRepository::persist($user);

            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function portalMembershipRequestAction()
    {
        $user = $this->_getRequestedUser();
        $portal = $this->_getRequestedPortal();
        $decision = $this->_getParam('decision');

        if (!$portal->isAdmin(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        if (!$user->isPendingPortal($portal)) {
            $this->view->message = 'info.portal.membership.handeled';
            return;
        }

        if ($decision == 'accept') {
            $user->addPortal($portal);
            $user->removePendingPortal($portal);
            $this->view->message = 'info.portal.membership.accepted';
        } else {
            $user->removePendingPortal($portal);
            $this->view->message = 'info.portal.membership.declined';
        }

        UserRepository::persist($portal);
    }

    public function changePictureAction()
    {
        $user = Identity::getUser();
        $form = $this->_getForm($user)->setDefaultsFrom($user);

        if ($form->isPostAndValid()) {
            if ($form->getValue('media')) {
                $user->setMedia($form->getValue('media'));
                UserRepository::persist($user);
            } elseif ($form->getValue('delete')) {
                $user->setMedia(null);
                UserRepository::persist($user);
            }

            $this->_setSuccessHeader();
            $this->forward('show-own');
        } else {
            $this->view->form = $form;
        }
    }

    public function changeCvAction()
    {
        $user = Identity::getUser();
        $form = $this->_getForm($user)->setDefaultsFrom($user);

        if ($form->isPostAndValid()) {
            $user->setCurriculumVitae($form->getValue('media'));
            UserRepository::persist($user);

            $this->_setSuccessHeader();
            $this->forward('show-own');
        } else {
            $this->view->form = $form;
        }
    }

    public function removeCvAction()
    {
        $user = Identity::getUser();
        $form = $this->_getForm($user)->setDefaultsFrom($user);

        if ($form->isPostAndValid()) {
            $user->setCurriculumVitae(null);
            UserRepository::persist($user);

            $this->_setSuccessHeader();
            $this->forward('show-own');
        } else {
            $this->view->form = $form;
        }
    }

    public function showExpositionAction()
    {
        if ($this->hasParam('application') && $this->_getRequestedApplication()->containsBoundExpositionId($this->_getParam('exposition'))) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('exposition'));
        }

        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('exposition'));
        }

        try {
            $exposition = ExpositionRepository::getById($this->_getParam('exposition'));

            $this->view->showReports = $this->_getParam('show-reports', false);
            $this->view->showComments = $this->_getParam('show-comments', false);
            $this->view->entity = $exposition;
            $this->view->relations = ResearchRepository::findRelations($this->view->entity);
            $this->view->additionalParams = $this->hasParam('application') ? ['application' => $this->getParam('application')] : [];

            $this->layout->basic->seo = $this->view->seo()->exposition($exposition);
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function showWorkAction()
    {
        if ($this->hasParam('application') && $this->_getRequestedApplication()->containsBoundWorkId($this->_getParam('work'))) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('work'));
        }

        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('work'));
        }
        try {
            $this->view->entity = $this->_getRequestedWork();
            $this->view->relations = ResearchRepository::findRelations($this->view->entity);
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function loadArchiveUrlsAction()
    {
        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('work'));
        }

        try {
            /** @var My_View_Helper_SizeMedia $sizeMedia */
            $sizeMedia = $this->view->getHelper('sizeMedia');
            $work = $this->_getRequestedWork();
            $simpleMedia = $this->_getRequestedFile();

            $resourceUrlList = [];
            foreach ($simpleMedia->getMediaList() as $media) {
                $size = $sizeMedia->calculateContainerSize(
                    $media->getSplashScreen()->getDimension(),
                    $this->_getParam('width', 300),
                    $this->_getParam('height', 250)
                );
                $resourceUrlList[] = $sizeMedia->url($media->getSplashScreen(), $size);
            }

            echo json_encode(array_values($resourceUrlList));
            $this->_disableView();
        } catch (Exception $exception) {
            echo $exception->getMessage();
            echo json_encode([]);
            $this->_disableView();
        }
    }

    public function downloadMediaAction()
    {
        try {
            $this->_getRequestedWork(); // check rights

            $simpleMedia = $this->_getRequestedFile();
            if ($simpleMedia instanceof ArchiveSimpleMedia) {
                $file = tempnam(temp_path(), 'download-media-');
                $filename = $simpleMedia->getName() . '.zip';

                $archive = new \ZipArchive();
                $archive->open($file);
                foreach ($simpleMedia->getMediaList() as $key => $resource) {
                    $derivative = $resource->getPrimaryDerivative();
                    $archive->addFile($derivative->getUri(), $key . '.' . $derivative->getFileExtension());
                }
                $archive->close();
            } else {
                $derivative = $simpleMedia->getMediaContainer()->getMedia()->getPrimaryDerivative();

                $file = $derivative->getUri();
                $filename = $simpleMedia->getName() . '.' . $derivative->getFileExtension();
            }

            header('X-Accel-Redirect: ' . sendfile($file));
            header('Content-type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . $filename . '"');

            $this->_disableLayout();
            $this->_disableView();
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function showProjectAction()
    {
        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('project'));
        }
        try {
            $this->view->entity = $this->_getRequestedProject();
            $this->view->relations = ResearchRepository::findRelations($this->view->entity);
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function showDegreeAction()
    {
        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('degree'));
        }
        try {
            $this->view->entity = $this->_getRequestedDegree();
            $this->view->relations = ResearchRepository::findRelations($this->view->entity);
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function showApplicationAction()
    {
        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('application'));
        }

        try {
            $this->view->entity = $this->_getRequestedApplication();
            $this->view->relations = ResearchRepository::findRelations($this->view->entity);
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function showPersonAction()
    {
        $this->forward('show', null, null, ['person' => $this->_getParam('person')]);
    }

    public function addGroupAction()
    {
        $group = new ProfileGroup();
        $form = $this->_getForm(Identity::getUser());

        if ($form->isPostAndValid() && !Identity::getUser()->isLimitedUser()) {
            $group
                ->setUserId(Identity::getUserId())
                ->setValues($form->getValues());
            ProfileGroupRepository::add($group);

            $this->_setSuccessHeader();
            $this->forward('show-own');
        } else {
            $this->view->form = $form;
        }
    }

    public function editGroupAction()
    {
        $group = $this->_getRequestedProfileGroup();
        $form = $this->_getForm(Identity::getUser())->setDefault('name', $group->getName());

        if ($group->getUserId() != Identity::getUserId() || !$group->isRemovable()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            $group->setName($form->getValue('name'));
            ProfileGroupRepository::update($group);

            $this->_setSuccessHeader();
            $this->forward('show-own');
        } else {
            $this->view->form = $form;
        }
    }

    public function hideGroupAction()
    {
        $group = $this->_getRequestedProfileGroup();

        if ($group->getUserId() != Identity::getUserId()) {
            throw new InsufficientRightsException();
        }

        $group->setHide($this->_getParam('hidden'));
        ProfileGroupRepository::update($group);

        $this->_setSuccessHeader();
        $this->_disableView();
    }

    public function removeGroupAction()
    {
        $group = $this->_getRequestedProfileGroup();
        $form = $this->_getForm(Identity::getUser());

        if ($group->getUserId() != Identity::getUserId() || !$group->isRemovable()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            ProfileGroupService::assignElementsToDefaultGroup(Identity::getUserId(),
                ProfileGroupElementRepository::findByGroupId($group->getId()));
            ProfileGroupRepository::remove($group);

            $this->_setSuccessHeader();
            $this->forward('show-own');
        } else {
            $this->view->form = $form;
        }
    }

    public function collapseGroupAction()
    {
        $group = $this->_getRequestedProfileGroup();

        if ($group->getUserId() != Identity::getUserId()) {
            throw new InsufficientRightsException();
        }

        $group->setCollapsed($this->_getParam('state'));
        ProfileGroupRepository::update($group);

        $this->_disableView();
    }

    public function updateGroupOrderAction()
    {
        foreach ($this->_getParam('order') as $position => $groupId) {
            $group = ProfileGroupRepository::findById($groupId);
            if (!isset($group) || $group->getUserId() != Identity::getUserId()) {
                continue;
            }
            $group->setPosition($position);
            ProfileGroupRepository::update($group);
        }
        $this->_disableView();
    }

    public function updateGroupElementOrderAction()
    {
        $group = $this->_getRequestedProfileGroup();
        if ($group->getUserId() != Identity::getUserId()) {
            throw new InsufficientRightsException();
        }
        if ($this->hasParam('entity')) {
            $groupElement = ProfileGroupElementRepository::findByUserIdAndEntityId(Identity::getUserId(),
                $this->_getParam('entity'));
            if (isset($groupElement)) {
                $groupElement->setGroupId($group->getId());
                ProfileGroupElementRepository::update($groupElement);
            }
        }
        foreach ($this->_getParam('order', []) as $position => $entityId) {
            $groupElement = ProfileGroupElementRepository::findByUserIdAndEntityId(Identity::getUserId(), $entityId);
            if (!isset($groupElement) || $group->getId() != $groupElement->getGroupId()) {
                continue;
            }
            $groupElement->setPosition($position);
            ProfileGroupElementRepository::update($groupElement);
        }
        $this->_disableView();
    }

    public function addShortcutAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        $shortcut = ProfileShortcutRepository::findByUserIdAndEntityId(Identity::getUserId(), $entity->getId());
        if (!isset($shortcut)) {
            $shortcut = new ProfileShortcut();
            $shortcut->setUserId(Identity::getUserId());
            $shortcut->setEntityId($entity->getId());
            ProfileShortcutRepository::add($shortcut);
        }
        $this->_disableView();
    }

    public function removeShortcutAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        $shortcut = ProfileShortcutRepository::findByUserIdAndEntityId(Identity::getUserId(), $entity->getId());
        if (isset($shortcut)) {
            ProfileShortcutRepository::remove($shortcut);
        }
        $this->_disableView();
    }

}
