<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Project;
use Rc\Domain\Repository\Research\ProjectRepository;
use Rc\Domain\Repository\UserRepository;

class ProjectController extends BasicControllerAction
{

    public function addAction()
    {
        if (Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $project = new Project();
        $project->setOwner(Identity::getUser());
        $form = $this->_getForm($project)->setDefaultsFrom($project);

        if ($form->isPostAndValid()) {
            $project->setValues($form->getValues());

            $ids = $form->getTab('meta')->getValue('principalinvestigators');
            if (!empty($ids) && is_array($ids)) {
                $project->setPrincipalInvestigator(UserRepository::findById(reset($ids)));
            } else {
                $project->setPrincipalInvestigator($project->getAuthor());
            }

            ProjectRepository::persist($project);
            KeywordRepository::addKeywords($project->getKeywords());

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
            $this->render('profile-entity/add', null, true);
        }
    }

    public function editAction()
    {
        $project = $this->_getRequestedProject();
        $form = $this->_getForm($project, Identity::getUser())->setDefaultsFrom($project);

        if ($form->isPostAndValid()) {
            $oldSupervisors = $project->getSupervisors();

            $project->setValues($form->getValues());

            if ($project->isOwner(Identity::getUser())) {
                $this->_handleCollaboration($project, $project->getSupervisors(), $oldSupervisors, 'supervisor');
            }

            ProjectRepository::persist($project);
            EntityChangeRepository::addChange($project, EntityChange::TYPE_EDIT);
            KeywordRepository::addKeywords($project->getKeywords());

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
            $this->render('profile-entity/edit', null, true);
        }
    }


    public function searchAction()
    {
        $this->_disableView();

        $retVal = [];
        foreach (ProjectRepository::suggest($this->_getParam('property'), $this->_getParam('value'), ['limit' => 10]) as $suggest) {
            $retVal[] = [
                'value' => $suggest->getEntityId(),
                'label' => $suggest->getMatch(),
            ];
        }

        header('Content-Type: application/json');
        echo json_encode($retVal);
    }


}