<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class SessionController extends BasicControllerAction
{

    public function loginAction()
    {
        if (Identity::exists()) {
            return true;
        }

        if (!Identity::setIdentity($this->_getParam('username'), $this->_getParam('password'))) {
            $this->view->error = 'error.wrong.email.or.password.given';
        } else if (!Identity::exists()) {
            $this->view->error = 'error.unexpected';
            Log::errLogin(Identity::getUserId());
        } else {
            if (Identity::getUser()->getSettings()->getHideNavigationTooltip()) {
                setcookie('navigationtooltip', 1, time() + 86400 * 90, '/');
            }
            Log::infoLogin(Identity::getUserId());
        }
    }

    public function switchAction()
    {
        $user = $this->_getRequestedUser();

        if (!Identity::exists() || !Identity::getUser()->isAdmin()) {
            $this->redirect('/portal');
        }

        Identity::switchUser($user);
        $this->view->user = $user;

        if ($this->_getParam('instant')) {
            $this->redirect('/profile');
        }
    }

    public function switchBackAction()
    {
        if (!Identity::exists()) {
            $this->redirect('/portal');
        }

        Identity::switchUserBack();

        if (!empty($_SESSION['switch_back_url'])) {
            $this->redirect($_SESSION['switch_back_url']);
        } else {
            $this->redirect('/admin');
        }
    }

    public function ignoreMaintenanceAction()
    {
        setcookie(self::MAINTENANCE_COOKIE_NAME, 'ignore', time() + 86400, '/', $_SERVER['HTTP_HOST'], true, true);

        $this->redirect('/');
    }

    public function logoutAction()
    {
        if (Identity::exists()) {
            Identity::clear();
        }

        $this->redirect('/portal');
    }

}

?>