<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMediaFactory;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;

class SimpleMediaController extends BasicControllerAction
{

    public function addAction()
    {
        $exposition = $this->_getRequestedResearch();
        $form = $this->_getForm();

        if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form);
        } else {
            $formToUse = $form;
        }

        if ($formToUse->isPostAndValid()) {
            $simpleMedia = SimpleMediaFactory::create($formToUse->getValue('mediatype'));
            $simpleMedia->setOwner(Identity::getUser());
            $simpleMedia->setResearch($exposition);
            $simpleMedia->setValues($formToUse->getValues());

            SimpleMediaRepository::persist($simpleMedia);

            $this->_setFormActionHeader('/simple-media/edit?file=' . $simpleMedia->getId());
            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function showAction()
    {
        /** @var File $file */
        $file = $this->_getRequestedFile();

        $this->view->assign('file', $file);
    }

    public function editAction()
    {
        $simpleMedia = $this->_getRequestedFile();
        $form = $this->_getForm($simpleMedia)->setDefaultsFrom($simpleMedia);

        if (!$simpleMedia->isOwner(Identity::getUser())) {
            $this->view->form = $this->_createInsufficientRightsForm($form->getTitle());
            return;
        }

        if ($simpleMedia->getMediaType()) {
            $formToUse = $form->getTab($simpleMedia->getMediaType());
        } else if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form);
        } else {
            $formToUse = $form;
        }

        if ($formToUse->isPostAndValid()) {
            session_write_close();

            $simpleMedia->setValues($formToUse->getValues());
            SimpleMediaRepository::persist($simpleMedia);

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->file = $simpleMedia;
        }
    }

    public function removeAction()
    {
        $files = [];
        $usedOnWeaves = [];
        $form = $this->_getForm();

        foreach ($this->_getRequestedFiles() as $simpleMedia) {
            if (!$simpleMedia->isOwner(Identity::getUser())) {
                throw new InsufficientRightsException();
            }

            $weaves = WeaveRepository::findBySimpleMedia($simpleMedia);
            if (count($weaves) > 0) {
                $files[$simpleMedia->getId()] = $simpleMedia;
                $usedOnWeaves[$simpleMedia->getId()] = $weaves;
            }
        }

        if ($form->isPostAndValid() && empty($usedOnWeaves)) {
            foreach ($this->_getRequestedFiles() as $simpleMedia) {
                SimpleMediaRepository::remove($simpleMedia);
            }

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->files = $this->_getRequestedFiles();
            $this->view->files = $files;
            $this->view->usedOnWeaves = $usedOnWeaves;
        }
    }

    public function listAction()
    {
        $this->view->files = SimpleMediaRepository::findByResearch($this->_getRequestedResearch(),
            ['sort' => ['name' => SimpleMediaRepository::SORT_ASC]]);
    }

    public function addTemplateAction()
    {
        $file = new SimpleMedia();
        $research = $this->_getRequestedResearch();
        $form = $this->_getForm();

        if ($form->isPostAndValid()) {
            $file->setOwner(Identity::getUser())
                ->setValues($form->getValues())
                ->setResearch($research);
            SimpleMediaRepository::persist($file);

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    private function _createInsufficientRightsForm($title)
    {
        $form = new DefaultForm();
        $form->setTitle($title);

        $form->addElement('html', 'info', [
            'value' => 'You don\'t have permissions to edit this simple media!',
        ]);

        $form->addElement('button', 'okbutton', [
            'label'   => 'button.ok',
            'onclick' => '$(this).closest(\'form\').dialog(\'close\'); return false;',
        ]);

        return $form;
    }
}

?>