<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\UserRepository;

class StatisticController extends BasicControllerAction
{

    public function init()
    {
        if ($this->getRequest()->getClientIp(false) != '127.0.0.1') {
            throw new InsufficientRightsException();
        }
        $this->_disableView();

        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');
    }

    public function indexAction()
    {
        $this->_logCollaboratedResearches();
        $this->_logUserObjects();
    }

    private function _logCollaboratedResearches()
    {
        $collaboratedResearches = [];
        foreach (ExpositionRepository::findAll() as $research) {
            if (!$research->hasCollaborateurs()) {
                continue;
            }

            $collaboratedResearches[] = [
                'id' => $research->getId(),
                'author' => $research->getAuthor()->getId(),
                'coauthor' => $research->getCoAuthors(),
                'contributor' => $research->getContributors(),
            ];
        }
        Log::infoCollaboration(json_encode($collaboratedResearches));
    }

    private function _logUserObjects()
    {
        $userObjects = [];
        foreach (UserRepository::findVisible() as $user) {
            $files = [];
            foreach (SimpleMediaRepository::findByUser($user) as $file) {
                $files[] = [
                    'id' => $file->getId(),
                    'simple-media' => !$file->hasWork(),
                ];
            }

            $userObjects[] = [
                'id' => $user->getId(),
                'files' => $files,
            ];
        }
        Log::infoObjects(json_encode($userObjects));
    }
}

?>