<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Research\Application;
use Rc\Service\Ingest\IngestService;

class My_Controller_Action_Helper_Application extends Zend_Controller_Action_Helper_Abstract
{
    /**
     * @return $this
     */
    public function direct()
    {
        return $this;
    }

    /**
     * @param Application $application
     * @return MediaInterface
     */
    public function createSubmitAttachment(Application $application)
    {
        $view = clone $this->_actionController->view;
        $view->assign('application', $application);

        $title = 'Your application ' . $application->getTitle();
        $content = $view->render('helpers/application-submit-mail.phtml');
        return $this->createPdf($title, $content);
    }

    /**
     * @param Application $application
     * @return MediaInterface
     */
    public function createReviewAttachment(Application $application)
    {
        $view = clone $this->_actionController->view;
        $view->assign('application', $application);

        $title = 'Reviews for application ' . $application->getTitle();
        $content = $view->render('helpers/application-review-mail.phtml');
        return $this->createPdf($title, $content);
    }

    /**
     * @param string $title
     * @param string $content
     * @return MediaInterface
     */
    protected function createPdf($title, $content)
    {
        $pdfFile = temp_path(uniqid('application_', true) . '.pdf');

        $pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
        $pdf->SetCreator(PDF_CREATOR);
        $pdf->SetAuthor('Research Catalogue');
        $pdf->SetTitle($title);
        $pdf->SetSubject($title);
        $pdf->setHeaderFont([PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN]);
        $pdf->setFooterFont([PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA]);
        $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
        $pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
        $pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
        $pdf->SetFooterMargin(PDF_MARGIN_FOOTER);
        $pdf->SetAutoPageBreak(true, PDF_MARGIN_BOTTOM);
        $pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);
        $pdf->SetFont('dejavusans', '', 10);

        $pdf->AddPage();
        $pdf->writeHTML($content, true, false, true, false, '');

        $pdf->Output($pdfFile, 'F');

        return IngestService::masterFromFile($pdfFile);
    }
}
