<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\ObjectShareAwareInterface;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Application\ApplicationFormRepository;
use Rc\Domain\Repository\Application\ApplicationProgramRepository;
use Rc\Domain\Repository\Portal\IssueRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\Research\WorkRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Utility\TokenGenerator;

class BasicFormGenerator
{

    static private $userList = null;

    public static function getMainFieldsOfStudy()
    {
        return [
            'classical-cello'               => 'Music: Baroque/Classical Cello',
            'classical-bassoon'             => 'Music: Baroque/Classical Bassoon',
            'classical-flute'               => 'Music: Baroque/Classical Flute',
            'classical-oboe'                => 'Music: Baroque/Classical Oboe',
            'classical-timpani'             => 'Music: Baroque/Classical Timpani',
            'classical-viola'               => 'Music: Baroque/Classical Viola',
            'classical-violin'              => 'Music: Baroque/Classical Violin',
            'bass-guitar'                   => 'Music: Bass Guitar',
            'basso-continuo'                => 'Music: Basso Continuo',
            'bassoon'                       => 'Music: Bassoon',
            'cello'                         => 'Music: Cello',
            'accordion'                     => 'Music: Accordion',
            'choral-conducting'             => 'Music: Choral Conducting',
            'clarinet'                      => 'Music: Clarinet',
            'classical-clarinet'            => 'Music: Classical Clarinet',
            'composition'                   => 'Music: Composition',
            'composition-and-performance'   => 'Music: Composition and Performance',
            'digital-arts'                  => 'Music: Digital Arts',
            'double-bass'                   => 'Music: Double Bass',
            'reconstruction'                => 'Music: Editing/Reconstruction',
            'ethnology'                     => 'Music: Ethnology',
            'flute'                         => 'Music: Flute',
            'fortepiano'                    => 'Music: Fortepiano',
            'guitar'                        => 'Music: Guitar',
            'harp'                          => 'Music: Harp',
            'harpsichord'                   => 'Music: Harpsichord',
            'historical-keyboard'           => 'Music: Historical Keyboards',
            'horn'                          => 'Music: Horn',
            'instrument'                    => 'Music: Instrument',
            'jazz-bass'                     => 'Music: Jazz Bass',
            'jazz-clarinet'                 => 'Music: Jazz Clarinet',
            'jazz-composition'              => 'Music: Jazz Composition/Arranging',
            'jazz-directing'                => 'Music: Jazz Directing',
            'jazz-drumkit'                  => 'Music: Jazz Drumkit/Percussion',
            'jazz-flute'                    => 'Music: Jazz Flute',
            'jazz-guitar'                   => 'Music: Jazz Guitar/Electric Guitar',
            'jazz-harp'                     => 'Music: Jazz Harp',
            'jazz-organ'                    => 'Music: Jazz Organ',
            'jazz-piano'                    => 'Music: Jazz Piano/Keyboards/Synthesiser',
            'jazz-saxophone'                => 'Music: Jazz Saxophone',
            'jazz-trombone'                 => 'Music: Jazz Trombone',
            'jazz-trumpet'                  => 'Music: Jazz Trumpet',
            'jazz-marimba'                  => 'Music: Jazz Vibraphone/Marimba',
            'jazz-violin'                   => 'Music: Jazz Violin',
            'jazz-vocals'                   => 'Music: Jazz Vocals',
            'keyboard'                      => 'Music: Keyboard Continuo',
            'lute'                          => 'Music: Lute/Theorbo/Vihuela',
            'marimba'                       => 'Music: Marimba',
            'medieval-harp'                 => 'Music: Medieval/Renaissance Harp',
            'medieval-wind'                 => 'Music: Medieval/Renaissance Wind',
            'pedagogy'                      => 'Music: Music Pedagogy',
            'psychology'                    => 'Music: Music Psychology',
            'technology'                    => 'Music: Music Technology',
            'musicology'                    => 'Music: Musicology',
            'natural-horn'                  => 'Music: Natural Horn',
            'natural-trumpet'               => 'Music: Natural Trumpet',
            'ney'                           => 'Music: Ney',
            'oboe'                          => 'Music: Oboe',
            'ophicleide'                    => 'Music: Ophicleide',
            'orchestral-conducting'         => 'Music: Orchestral Conducting',
            'organ'                         => 'Music: Organ',
            'percussion'                    => 'Music: Percussion',
            'performance'                   => 'Music: Performance',
            'performance-musicology'        => 'Music: Performance and Musicology',
            'period-percussion-instruments' => 'Music: Period Percussion Instruments',
            'piano'                         => 'Music: Piano',
            'piano-accompaniment'           => 'Music: Piano Accompaniment/Repetiteur',
            'pop-bass-guitar'               => 'Music: Pop Bass Guitar',
            'pop-drumkit'                   => 'Music: Pop Drumkit/Percussion',
            'pop-electric-guitar'           => 'Music: Pop Electric Guitar',
            'pop-keyboard'                  => 'Music: Pop Keyboards/Synthesiser',
            'pop-recording'                 => 'Music: Pop Recording/Production',
            'pop-songwriting'               => 'Music: Pop Songwriting/Arranging',
            'pop-vocals'                    => 'Music: Pop Vocals',
            'recorder'                      => 'Music: Recorder',
            'sackbut'                       => 'Music: Sackbut',
            'saxophone'                     => 'Music: Saxophone',
            'social-science'                => 'Music: Social Science',
            'timpani'                       => 'Music: Timpani',
            'trombone'                      => 'Music: Trombone',
            'trumpet'                       => 'Music: Trumpet',
            'tuba'                          => 'Music: Tuba',
            'ud'                            => 'Music: Ud',
            'viola'                         => 'Music: Viola',
            'viola-da-gambaion'             => 'Music: Viola da Gamba',
            'violin'                        => 'Music: Violin',
            'viols'                         => 'Music: Viols',
            'voice'                         => 'Music: Voice',
            'baritone'                      => 'Music: Voice: Baritone',
            'bass'                          => 'Music: Voice: Bass',
            'contralto'                     => 'Music: Voice: Contralto',
            'counter-tenor'                 => 'Music: Voice: Counter Tenor',
            'mezzo-soprano'                 => 'Music: Voice: Mezzo Soprano',
            'soprano'                       => 'Music: Voice: Soprano',
            'tenor'                         => 'Music: Voice: Tenor',
            'other'                         => 'main-field-of-study.other',
        ];
    }

    public static function getRemove()
    {
        $form = new DefaultForm();

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.entry.remove'),
        ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    /**
     * @param Research|null $entity
     * @return DefaultForm
     */
    public static function getShare(Research $entity = null)
    {
        $form = new DefaultForm();

        if (Identity::getUser()->isMemberOfNonDefaultPortal()) {
            $multiOptions = [
                Work::ACCESS_LEVEL_PRIVATE => 'field.access.private',
                Work::ACCESS_LEVEL_PORTAL  => 'field.access.portal',
                Work::ACCESS_LEVEL_LIMITED => 'field.access.limited',
                Work::ACCESS_LEVEL_PUBLIC  => 'field.access.public',
            ];
        } else {
            $multiOptions = [
                Work::ACCESS_LEVEL_PRIVATE => 'field.access.private',
                Work::ACCESS_LEVEL_LIMITED => 'field.access.limited',
                Work::ACCESS_LEVEL_PUBLIC  => 'field.access.public',
            ];
        }

        $form->addElement('radio', 'accesslevel', [
            'required'     => true,
            'multioptions' => $multiOptions,
            'labelStyle'   => 'width:95%',
        ]);

        if (isset($multiOptions[Work::ACCESS_LEVEL_PORTAL])) {
            $form->addElement('multiselect', 'accessLevelPortalsIds', [
                'multiOptions' => static::_getPortalList(),
                'style'        => 'margin-left: 100px',
            ]);
        }

        if ($entity instanceof ObjectShareAwareInterface) {
            $url = config('app.url') . '/shared/' . TokenGenerator::generateForModel($entity);
            $form->addElement('checkbox', 'objectShare', [
                'description' => 'When enabled, the object is accessible to anybody that knows this link. This includes readers without an rc account.<br/><a href="' . $url . '">' . $url . '</a>',
            ]);
        }

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    protected static function _getCountryList()
    {
        $countryList = MyLocal::getTranslationList('territory', null, 2);
        unset($countryList['SU'], $countryList['ZZ'], $countryList['VD'], $countryList['DD']);
        asort($countryList);
        return $countryList;
    }

    /**
     * @param \DefaultForm $form
     * @param Research $entity
     * @return \Zend_Form
     */
    protected static function _addMediaTab(DefaultForm $form, Research $entity)
    {
        $form->addTab('files');
        return $form;
    }

    /**
     * @param \DefaultForm $form
     * @param Rc\Domain\Model\Research $entity
     * @return \Zend_Form
     */
    protected static function _addCollaborationTab(DefaultForm $form, Rc\Domain\Model\Research $entity)
    {
        $coauthors = static::_getLazyUserList(static::_getParam('collaboration.coauthors', $entity->getCoAuthors()),
            false);
        foreach ($coauthors as $id => $coauthor) {
            if ($entity->isInvitePending($id, 'coauthor')) {
                $coauthors[$id] = '[' . $coauthor . ']';
            }
        }

        $contributors = static::_getLazyUserList(static::_getParam('collaboration.contributors',
            $entity->getContributors()), false);
        foreach ($contributors as $id => $contributor) {
            if ($entity->isInvitePending($id, 'contributor')) {
                $contributors[$id] = '[' . $contributor . ']';
            }
        }

        $supervisors = static::_getLazyUserList(static::_getParam('collaboration.supervisors',
            $entity->getSupervisors()), false);
        foreach ($supervisors as $id => $supervisor) {
            if ($entity->isInvitePending($id, 'supervisor')) {
                $supervisors[$id] = '[' . $supervisor . ']';
            } elseif ($entity->isDisapprovedCollaborator($id, 'supervisor')) {
                $supervisors[$id] = '{' . $supervisor . '}';
            }
        }

        $tab = $form->addTab('collaboration');
        if ($entity instanceof Exposition) {
            $tab->addElement('select', 'authorid', [
                'value'        => $entity->getAuthor()->getId(),
                'label'        => 'author',
                'multiOptions' => self::_getLazyUserList(array_merge([$entity->getAuthor()->getId()],
                    $entity->getCoAuthors()), false, true),
                'description'  => '<span class="form-description-line">if you change author you will be added as co-author</span>',
            ]);
        }

        $tab
            ->addElement('multiselect', 'coauthors', [
                'label'                     => 'field.co-author',
                'multiOptions'              => $coauthors,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'            => '/user/search',
                    'placeholder'    => 'Add user...',
                    'formatNewValue' => '[%s]',
                ]),
            ])
            ->addElement('multiselect', 'contributors', [
                'label'                     => 'field.contributor',
                'multiOptions'              => $contributors,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'            => '/user/search',
                    'placeholder'    => 'Add user...',
                    'formatNewValue' => '[%s]',
                ]),
            ])
            ->addElement('multiselect', 'supervisors', [
                'label'                     => 'field.supervisor',
                'multiOptions'              => $supervisors,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'            => '/user/search',
                    'placeholder'    => 'Add user...',
                    'formatNewValue' => '[%s]',
                    'formatOldValue' => '{%s}',
                ]),
            ]);

        if ($entity instanceof Exposition && Identity::isSwitchedUserAdmin()) {
            $tab->addElement('select', 'ownerid', [
                'value'        => $entity->getOwner()->getId(),
                'label'        => 'owner',
                'multiOptions' => static::_getLazyUserList(array_merge([$entity->getOwner()->getId()], [$entity->getAuthor()->getId()], $entity->getCoAuthors()), false, true),
                'description'  => '<span class="form-description-line" style="color:brown;font-weight:bold">If you change the owner you will loose ALL WRITE ACCESS to the exposition! You will be added as co-author.</span>',
            ]);
        }

        return $tab;
    }

    protected static function _getWorkGenreList()
    {
        return [
            static::_getTranslate()->translate('genre.publication') => [
                'publication' => 'genre.publication',
                'paper'       => 'genre.publication.paper',
                'catalogue'   => 'genre.publication.catalogue',
                'article'     => 'genre.publication.article',
                'book'        => 'genre.publication.book',
                'broadcast'   => 'genre.publication.broadcast',
                'cd'          => 'genre.publication.cd',
                'dvd'         => 'genre.publication.dvd',
            ],
            static::_getTranslate()->translate('genre.event')       => [
                'event'        => 'genre.event',
                'exhibition'   => 'genre.event.exhibition',
                'screening'    => 'genre.event.screening',
                'concert'      => 'genre.event.concert',
                'performance'  => 'genre.event.performance',
                'festival'     => 'genre.event.festival',
                'seminar'      => 'genre.event.seminar',
                'conference'   => 'genre.event.conference',
                'presentation' => 'genre.event.presentation',
                'workshop'     => 'genre.event.workshop',
            ],
            static::_getTranslate()->translate('genre.art')         => [
                'art'            => 'genre.art',
                'installation'   => 'genre.art.installation',
                'scenery'        => 'genre.art.scenery',
                'piece'          => 'genre.art.piece',
                'design'         => 'genre.art.design',
                'screenplay'     => 'genre.art.screenplay',
                'sound'          => 'genre.art.sound',
                'photograph'     => 'genre.art.photograph',
                'painting'       => 'genre.art.painting',
                'scalemodel'     => 'genre.art.scalemodel',
                'digitalartwork' => 'genre.art.digitalartwork',
                'visualisation'  => 'genre.art.visualisation',
                'illustration'   => 'genre.art.illustration',
                'ceramic'        => 'genre.art.ceramic',
                'print'          => 'genre.art.print',
                'construction'   => 'genre.art.construction',
                'drawing'        => 'genre.art.drawing',
                'video'          => 'genre.art.video',
                'composition'    => 'genre.art.composition',
                'movie'          => 'genre.art.movie',
            ],
        ];
    }

    protected static function _getLanguageList()
    {
        return MyLocal::getTranslationList('language');
    }

    protected static function _getDatabaseAdapter()
    {
        return Zend_Registry::get('Zend_Db_Adapter');
    }

    /**
     * @return Zend_Translate|Zend_Translate_Adapter
     * @throws \Zend_Exception
     */
    protected static function _getTranslate()
    {
        return Zend_Registry::get('Zend_Translate');
    }

    protected static function translate($messageId, $locale = null)
    {
        return self::_getTranslate()->translate($messageId, $locale);
    }

    protected static function _getLocale()
    {
        return Zend_Registry::get('Zend_Locale');
    }

    protected static function _getRoleList()
    {
        #TODO: available roles have to be restricted for portal admins
        return [
            User::ROLE_REVIEWER     => 'role.reviewer',
            User::ROLE_PORTAL_ADMIN => 'role.portaladmin',
            User::ROLE_SUPER_ADMIN  => 'role.superadmin',
        ];
    }

    protected static function _getDefaultPortal()
    {
        $defaultPortal = PortalRepository::getDefault();
        return [
            $defaultPortal->getId() => $defaultPortal->getName(),
        ];
    }

    /**
     * @param bool $ignoreDefaultPortal
     * @param string $feature
     * @return Portal[]
     */
    protected static function _getPortalList($ignoreDefaultPortal = true, $feature = null)
    {
        $retVal = [];
        foreach (PortalRepository::findAll(['sort' => ['name' => 1]]) as $portal) {
            if ($ignoreDefaultPortal && $portal->isDefault()) {
                continue;
            }
            if (isset($feature) && !$portal->getFeature($feature)) {
                continue;
            }
            $retVal[$portal->getId()] = $portal->getName();
        }

        return $retVal;
    }

    protected static function _getApplicationProgramList()
    {
        $retVal = [];
        foreach (ApplicationProgramRepository::findEligibleByUser(Identity::getUser()) as $program) {
            /** @var Portal[] $applicantPortals */
            $applicantPortals = array_intersect($program->getApplicantPortals(), Identity::getUser()->getPortals());
            if (!empty($applicantPortals)) {
                foreach ($applicantPortals as $portal) {
                    $retVal[$program->getId() . '-' . $portal->getId()] = $program->getTitle() .
                        ' (' . $portal->getName() . ')';
                }
            } else {
                $retVal[$program->getId() . '-' . $program->getPortal()->getId()] = $program->getTitle() .
                    ' (' . $program->getPortal()->getName() . ')';
            }
        }

        return $retVal;
    }

    protected static function _getPortalsWhichAllowsPublishing($type, $excludeDefaultPortal = true, $excludePortalWithoutExternalContent = false)
    {
        $retVal = [];
        $method = 'findByAllow' . $type . 'Publishing';
        /** @var Portal $portal */
        foreach (PortalRepository::$method() as $portal) {
            if ($excludePortalWithoutExternalContent && !$portal->getAllowExternalContent()) {
                continue;
            }

            $retVal[$portal->getId()] = $portal->getName();
        }

        if ($excludeDefaultPortal) {
            unset($retVal[PortalRepository::getDefault()->getId()]);
        }

        return $retVal;
    }

    protected static function _getPortalsWhichAllowsLimitedPublishing($type, $excludeDefaultPortal = true, $excludePortalWithoutExternalContent = false)
    {
        $retVal = [];
        $method = 'findByAllowLimited' . $type . 'Publishing';
        /** @var Portal $portal */
        foreach (PortalRepository::$method() as $portal) {
            if ($excludePortalWithoutExternalContent && !$portal->getAllowExternalContent()) {
                continue;
            }

            $retVal[$portal->getId()] = $portal->getName();
        }

        if ($excludeDefaultPortal) {
            unset($retVal[PortalRepository::getDefault()->getId()]);
        }

        return $retVal;
    }

    protected static function _getPortalsWhichAllowsConnecting($type)
    {
        $retVal = [];
        $method = 'findByAllow' . $type . 'Connecting';
        /** @var Portal $portal */
        foreach (PortalRepository::$method() as $portal) {
            $retVal[$portal->getId()] = $portal->getName();
        }
        return $retVal;
    }

    /**
     * @param Tool $tool
     * @param Exposition $exposition
     * @return SimpleMediaInterface[]
     */
    protected static function _getSupportedFiles(Tool $tool, Exposition $exposition)
    {
        $retVal = [];
        foreach (SimpleMediaRepository::findByTool($tool, $exposition) as $file) {
            $retVal[$file->getId()] = $file->getName();
        }
        return $retVal;
    }

    /**
     * @param bool $exclude_current_user
     * @return array
     */
    protected static function _getUserList($exclude_current_user = true)
    {
        if (!isset(self::$userList)) {
            self::$userList = [];
            foreach (UserRepository::findVisible() as $user) {
                self::$userList[$user->getId()] = $user->getName();
            }
        }

        $retVal = self::$userList;
        if ($exclude_current_user) {
            unset($retVal[Identity::getUserId()]);
        }

        return $retVal;
    }

    /**
     * @param Portal $portal
     * @return array
     */
    protected static function _getIssueList(Portal $portal)
    {
        $issueList = [];
        foreach (IssueRepository::findByPortal($portal) as $issue) {
            $issueList[$issue->getId()] = $issue->getNumber() . '. ' . $issue->getTitle();
        }
        return $issueList;
    }

    protected static function _getLazyUserList($ids, $exclude_current_user = true, $includeSetInvisibleUsers = false)
    {
        if (!isset($ids) || !is_array($ids)) {
            return [];
        }

        $retVal = [];
        foreach (UserRepository::findVisibleByIds($ids) as $user) {
            $retVal[$user->getId()] = $user->getName();
        }

        if ($includeSetInvisibleUsers) {
            $invisibleUsers = [];
            foreach ($ids as $id) {
                if (!isset($retVal[$id])) {
                    $invisibleUsers[] = $id;
                }
            }

            if (!empty($invisibleUsers)) {
                foreach (UserRepository::findByIds($invisibleUsers) as $user) {
                    $retVal[$user->getId()] = $user->getName();
                }
            }
        }

        if ($exclude_current_user) {
            unset($retVal[Identity::getUserId()]);
        }

        return $retVal;
    }

    /**
     * @param Portal $portal
     * @param string $genre
     * @return array
     */
    protected static function _getApplicationFormList(Portal $portal, $genre)
    {
        $retVal = [];
        foreach (ApplicationFormRepository::findByPortalAndGenre($portal, $genre) as $applicationForm) {
            $retVal[$applicationForm->getId()] = $applicationForm->getTitle();
        }
        return $retVal;
    }

    /**
     * @param array $ids
     * @param int $excludedId
     * @return array
     */
    protected static function _getLazyRelationList(array $ids, $excludedId)
    {
        $retVal = [];
        foreach (ResearchRepository::findByIds($ids) as $research) {
            if ($research->getId() == $excludedId) {
                continue;
            }
            $retVal[$research->getId()] = $research->getTitle();
        }
        return $retVal;
    }

    /**
     * @param array $ids
     * @param int $excludedId
     * @return array
     */
    protected static function _getLazyExpositionList(array $ids, $excludedId = null)
    {
        $retVal = [];
        foreach (ExpositionRepository::findByIds($ids) as $exposition) {
            if ($exposition->getId() == $excludedId) {
                continue;
            }
            $retVal[$exposition->getId()] = $exposition->getTitle();
        }
        return $retVal;
    }

    /**
     * @param array $ids
     * @param int $excludedId
     * @return array
     */
    protected static function _getLazyWorkList(array $ids, $excludedId = null)
    {
        $retVal = [];
        foreach (WorkRepository::findByIds($ids) as $work) {
            if ($work->getId() == $excludedId) {
                continue;
            }
            $retVal[$work->getId()] = $work->getTitle();
        }
        return $retVal;
    }

    protected static function _getReviewerList(Portal $portal = null)
    {
        if (!isset($portal)) {
            $users = UserRepository::findByRole(User::ROLE_REVIEWER);
        } else {
            $users = $portal->getReviewers();
        }
        $retVal = [];
        foreach ($users as $user) {
            $retVal[$user->getId()] = $user->getName();
        }
        return $retVal;
    }

    /**
     * @param User $user
     * @return string[]
     */
    protected static function _getResearchListByPerson(User $user)
    {
        $retVal = [];
        foreach (ExpositionRepository::findOwnAndCollaboratedByPerson($user) as $exposition) {
            $retVal[$exposition->getId()] = $exposition->getTitle();
        }
        return $retVal;
    }

    protected static function _getParam($path, $default = null)
    {
        $request = Zend_Controller_Front::getInstance()->getRequest();

        $parts = explode('.', $path);
        $param = $request->getParam($parts[0]);

        for ($i = 1; $i < count($parts); $i++) {
            $param = &$param[$parts[$i]];
        }

        if (!$param) {
            return $default;
        }

        return $param;
    }

    /**
     * @return null|Zend_Controller_Request_Abstract|Zend_Controller_Request_Http
     */
    protected static function _getRequest()
    {
        return Zend_Controller_Front::getInstance()->getRequest();
    }

    /**
     * @param \DefaultForm $form
     * @param Research $research
     * @return \Zend_Form
     */
    protected static function _addRelationTab(DefaultForm $form, Research $research)
    {
        $relations = static::_getLazyRelationList(static::_getParam('relation.relations', $research->getRelations()),
            $research->getId());
        return $form->addTab('relation')
            ->addElement('multiselect', 'relations', [
                'label'                     => 'field.relations',
                'multiOptions'              => $relations,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/profile-entity/suggest-relations?excluded_id=' . $research->getId(),
                    'placeholder' => 'Add item...',
                ]),
            ]);
    }
}

?>
