<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Degree;

class DegreeFormGenerator extends BasicFormGenerator
{
    public static function getAdd(Degree $degree)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.degree.add', $degree);

        $tab = $form->addTab('meta');
        $tab->addElement('languageBar', 'language-bar')
            ->addElement('text', 'title', [
                'required' => true,
                'label'    => 'field.name',
            ])
            ->addElement('select', 'level', [
                'required'     => true,
                'label'        => 'field.level',
                'multiOptions' => self::getLevels(),
            ])
            ->addElement('datePicker', 'startdate', [
                'label' => 'field.start-date',
            ])
            ->addElement('datePicker', 'conferreddate', [
                'label' => 'field.date-conferred',
            ])
            ->addElement('awardingInstitutions', 'awardinginstitutions', [
                'required'  => true,
                'label'     => 'field.awarding-institution',
                'countries' => ['' => ''] + static::_getCountryList(),
            ])
            ->addElement('text', 'keywords', [
                'required'    => true,
                'label'       => 'field.keywords',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ])
            ->addElement('text', 'referenceurl', [
                'label' => 'field.reference-url',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'label'      => 'field.description',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            'max' => 2500,
                        ],
                    ],
                ],
            ]);

        self::_addMediaTab($form, $degree);
        self::_addCollaborationTab($form, $degree);
        self::_addRelationTab($form, $degree);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getEdit(Degree $degree)
    {
        $form = static::getAdd($degree);
        $form->setTitle('dialog.degree.edit', $degree);

        return $form;
    }

    public static function getPublish(Degree $degree)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.degree.publish', $degree);

        $form
            ->addElement('select', 'portal', [
                'label'        => 'field.portal',
                'multioptions' => ['' => ''] + static::_getPortalsWhichAllowsPublishing('degree', false),
                'onchange'     => '$("#' . $form->getElementsPrefix() . 'portalid").val($(this).val())',
            ])
            ->addElement('hidden', 'portalid', [
                'required' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getLevels()
    {
        return [
            'associate' => 'degree.level.associate',
            'bachelor'  => 'degree.level.bachelor',
            'master'    => 'degree.level.master',
            'doctorate' => 'degree.level.doctorate',
        ];
    }

}