<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;

class EditorFormGenerator extends BasicFormGenerator
{

    public static function getCollaborationMessage(Exposition $research)
    {
        $form = new DefaultForm();
        $form->setTitle('editor.toolbox.collaboration.message');

        $allowedReceivers = [
            $research->getAuthor()->getId() => $research->getAuthor()->getName(),
        ];
        foreach ($research->getCollaborateurList() as $collaborator) {
            $allowedReceivers[$collaborator->getId()] = $collaborator->getName();
        }

        $previewUrl = '/view/' . $research->getId() . '/' . $research->getDefaultWeaveId();
        $editorUrl = '/editor?research=' . $research->getId() . '&weave=' . $research->getDefaultWeaveId();

        $form
            ->addElement('multiselect', 'receiverid', [
                'label'            => 'field.collaborator',
                'required'         => true,
                'multiOptions'     => is_array($allowedReceivers) ? $allowedReceivers : static::_getUserList(),
                'data-multiselect' => true,
                'data-multiselect-options' => json_encode([
                    'addAll'         => true,
                ]),
                'style'            => 'height:160px',
            ])
            ->addElement('text', 'subject', [
                'label'    => 'field.subject',
                'required' => true,
            ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'required'    => true,
                'data-editor' => true,
                'data-scope'  => 'portal',
                'value'       => '<p>
							research: ' . $research->getTitle() . ' (
								<a href="' . $editorUrl . '" target="_blank">edit</a>,
								<a href="' . $previewUrl . '" target="_blank">preview</a>
							)<br/>
							------------------------------------------------------------------------------------
						<p/>',
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

}

?>