<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class FeedsFormGenerator extends BasicFormGenerator
{

    public static function getSettings()
    {
        $entityUrl = new My_View_Helper_Url();

        $form = new DefaultForm();
        $form->setTitle('dialog.feeds.settings');

        $entityFeeds = EntityFeedRepository::findByUser(Identity::getUser());
        if (!empty($entityFeeds)) {
            $tab = $form->addTab('entities');
            foreach ($entityFeeds as $entityFeed) {
                $link = '<a href="' . $entityUrl->entityUrl($entityFeed->getTarget()) . '" target="_blank">' .
                    $entityFeed->getTarget()->getTitle() . '</a>';

                $fieldset = $tab->addFieldset($entityFeed->getTargetId());
                $fieldset->addElement('html', 'entity', [
                    'value' => self::_getTranslate()->translate('entity.' . $entityFeed->getTarget()->getName()) . ' ' . $link . ':',
                    'style' => 'overflow: hidden; text-overflow: ellipsis; white-space: nowrap; margin-bottom: -15px',
                    'tag'   => 'p',
                ]);
                $fieldset->addInlineElement(
                    [
                        $fieldset->createElement('checkbox', 'receiveShared', [
                            'description' => 'change.type.share',
                            'value'       => $entityFeed->getReceiveShared(),
                            'style'       => 'margin-left: 25px',
                        ]),
                        $fieldset->createElement('checkbox', 'receiveEdited', [
                            'description' => 'change.type.edit',
                            'value'       => $entityFeed->getReceiveEdited(),
                        ]),
                        $fieldset->createElement('checkbox', 'receiveCommented', [
                            'description' => 'change.type.comment',
                            'value'       => $entityFeed->getReceiveCommented(),
                        ]),
                        $fieldset->createElement('checkbox', 'receivePublished', [
                            'description' => 'change.type.publish',
                            'value'       => $entityFeed->getReceivePublished(),
                        ]),
                    ]
                );
            }
        }

        $userFeeds = UserFeedRepository::findByUser(Identity::getUser());
        if (!empty($userFeeds)) {
            $tab = $form->addTab('persons');
            foreach ($userFeeds as $userFeed) {
                $link = '<a href="/profile/?person=' . $userFeed->getTargetid() . '" target="_blank">' .
                    $userFeed->getTarget()->getName() . '</a>';

                $fieldset = $tab->addFieldset($userFeed->getTargetid());
                $fieldset->addInlineElement(
                    [
                        $fieldset->createElement('html', 'person', [
                            'value' => '<span style="display: inline-block; width: 175px">' . $link . ': </span>',
                            'tag'   => 'span',
                            'class' => 'inline',
                        ]),
                        $fieldset->createElement('checkbox', 'receiveShared', [
                            'description' => 'change.type.share',
                            'value'       => $userFeed->getReceiveShared(),
                        ]),
                        $fieldset->createElement('checkbox', 'receiveEdited', [
                            'description' => 'change.type.edit',
                            'value'       => $userFeed->getReceiveEdited(),
                        ]),
                        $fieldset->createElement('checkbox', 'receiveCommented', [
                            'description' => 'change.type.comment',
                            'value'       => $userFeed->getReceiveCommented(),
                        ]),
                        $fieldset->createElement('checkbox', 'receivePublished', [
                            'description' => 'change.type.publish',
                            'value'       => $userFeed->getReceivePublished(),
                        ]),
                    ]
                );
            }
        }

        $portalFeeds = PortalFeedRepository::findByUser(Identity::getUser());
        if (!empty($portalFeeds)) {
            $tab = $form->addTab('portals');
            foreach ($portalFeeds as $portalFeed) {
                $link = '<a href="/portals/?portal=' . $portalFeed->getTargetid() . '" target="_blank">' .
                    $portalFeed->getTarget()->getShortName() . '</a>';

                $fieldset = $tab->addFieldset($portalFeed->getTargetid());
                $fieldset->addInlineElement(
                    [
                        $fieldset->createElement('html', 'person', [
                            'value' => '<span style="display: inline-block; width: 175px">' . $link . ': </span>',
                            'tag'   => 'span',
                            'class' => 'inline',
                        ]),
                        $fieldset->createElement('checkbox', 'receiveShared', [
                            'description' => 'change.type.share',
                            'value'       => $portalFeed->getReceiveShared(),
                        ]),
                        $fieldset->createElement('checkbox', 'receiveEdited', [
                            'description' => 'change.type.edit',
                            'value'       => $portalFeed->getReceiveEdited(),
                        ]),
                        $fieldset->createElement('checkbox', 'receiveCommented', [
                            'description' => 'change.type.comment',
                            'value'       => $portalFeed->getReceiveCommented(),
                        ]),
                        $fieldset->createElement('checkbox', 'receivePublished', [
                            'description' => 'change.type.publish',
                            'value'       => $portalFeed->getReceivePublished(),
                        ]),
                    ]
                );
            }
        }

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }
}