<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class MessengerFormGenerator extends BasicFormGenerator
{

    public static function getAdd()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.message.add');

        $form->addElement('multiselect', 'receiverids', [
            'label'                     => 'field.target',
            'required'                  => true,
            'multiOptions'              => static::_getLazyUserList((array)static::_getParam('receiverids')),
            'data-autocomplete'         => true,
            'data-autocomplete-options' => json_encode([
                'url'                => '/user/search',
                'placeholder'        => 'Add user...',
                'maxcount'           => 10,
                'excludeCurrentUser' => true,
            ]),
        ])
            ->addElement('text', 'subject', [
                'label'    => 'field.subject',
                'required' => true,
            ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'required'    => true,
                'data-editor' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getShow(\Rc\Domain\Model\Message $message)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.message.show', $message)
            ->setAction('/messenger/add');

        $form->addElement('text', 'receiverids', [
            'style' => 'visibility:hidden;width:0px',
        ])
            ->addElement('text', 'subject', [
                'label' => 'field.subject',
                'style' => 'visibility:hidden;width:0px',
            ])
            ->addElement('text', 'date', [
                'label' => 'field.date',
                'style' => 'visibility:hidden;width:0px',
            ]);

        $files = [];
        foreach ($message->getAttachments() as $name => $attachment) {
            $files[] = '<a href="' . $attachment->getMaster()->getExternalUri() . '" target="_blank">' . $name . '</a>';
        }
        if (!empty($files)) {
            $form->addElement('html', 'attachment', [
                'label' => 'field.attachment',
                'value' => implode(', ', $files),
                'tag'   => 'span',
            ]);
        }

        $form->addElement('html', 'content', []);

        $form->addCancelButton('message.close')->addElement('button', 'answer', [
            'type'  => 'submit',
            'label' => 'button.message.answer',
        ]);

        return $form;
    }

    /**
     * @param \Rc\Domain\Model\Message[] $messages
     * @return DefaultForm
     */
    public static function getReadSelected($messages)
    {
        $messageIds = [];
        foreach ($messages as $message) {
            $messageIds[$message->getId()] = $message->getId();
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.messages.read')
            ->addElement('html', 'info', [
                'value' => static::_getTranslate()->translate('info.entries.read'),
            ])
            ->addElement('multiselect', 'message', [
                'value'        => $messageIds,
                'multiOptions' => $messageIds,
                'style'        => 'display:none',
            ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        foreach ($form->getElement('message')->getDecorators() as $key => $dec) {
            if ($key !== 'My_Form_Decorator_DefaultElement') {
                $form->getElement('message')->removeDecorator($key);
            }
        }

        $form->addCancelButton('no')->addSubmitButton('yes');
        return $form;
    }

    /**
     * @param \Rc\Domain\Model\Message[] $messages
     * @return DefaultForm
     */
    public static function getRemoveSelected($messages)
    {
        $messageIds = [];
        foreach ($messages as $message) {
            $messageIds[$message->getId()] = $message->getId();
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.messages.remove')
            ->addElement('html', 'info', [
                'value' => static::_getTranslate()->translate('info.entries.remove'),
            ])
            ->addElement('multiselect', 'message', [
                'value'        => $messageIds,
                'multiOptions' => $messageIds,
                'style'        => 'display:none',
            ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        foreach ($form->getElement('message')->getDecorators() as $key => $dec) {
            if ($key !== 'My_Form_Decorator_DefaultElement') {
                $form->getElement('message')->removeDecorator($key);
            }
        }

        $form->addCancelButton('no')->addSubmitButton('yes');
        return $form;
    }

    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.message.remove');

        return $form;
    }

}

?>