<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Popover;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\PictureTool;
use Rc\Domain\Repository\Research\Exposition\PopoverRepository;

class PopoverFormGenerator extends BasicFormGenerator
{

    public static function getEdit(Popover $popover)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.popover.edit', $popover);
        $form->addTab('meta')
            ->addElement('text', 'title', [
                'required' => true,
                'label'    => 'field.title',
            ]);

        $tab = $form->addTab('style');
        $tab->addAccordion('layout', [
            $tab->createElement('number', 'width', [
                'label'    => 'field.width',
                'required' => true,
                'min'      => 0,
            ]),
            $tab->createElement('number', 'height', [
                'label'    => 'field.height',
                'required' => true,
                'min'      => 0,
            ]),
        ]);
        $tab->addAccordion('margin', [
            $tab->createElement('number', 'margin-left', [
                'label' => 'field.left',
            ]),
            $tab->createElement('number', 'margin-top', [
                'label' => 'field.top',
            ]),
        ]);
        $tab->addAccordion('padding', [
            $tab->createElement('number', 'padding-left', [
                'label' => 'field.left',
            ]),
            $tab->createElement('number', 'padding-top', [
                'label' => 'field.top',
            ]),
            $tab->createElement('number', 'padding-right', [
                'label' => 'field.right',
            ]),
            $tab->createElement('number', 'padding-bottom', [
                'label' => 'field.bottom',
            ]),
        ]);
        $tab->addAccordion('border', [
            $tab->createElement('select', 'border-style', [
                'label'        => 'field.style',
                'multioptions' => Tool::getBorderStyles(),
            ]),
            $tab->createElement('number', 'border-width', [
                'label' => 'field.strength',
                'min'   => 0,
            ]),
            $tab->createElement('color', 'border-color', [
                'label' => 'field.color',
            ]),
            $tab->createElement('number', 'border-radius', [
                'label' => 'field.radius',
            ]),
        ]);
        $tab->addAccordion('background', [
            $tab->createElement('color', 'background-color', [
                'label' => 'field.color',
            ]),
            $tab->createElement('select', 'background-image-file-id', [
                'label'        => 'field.image',
                'multiOptions' => ['' => ''] + self::_getSupportedFiles(new PictureTool(), $popover->getExposition()),
            ]),
            $tab->createInlineElement([
                $tab->createElement('select', 'background-image-style', [
                    'label'        => 'field.image.properties',
                    'style'        => 'width:90px',
                    'multioptions' => Tool::getBackgroundImageStyles(),
                ]),
                $tab->createElement('select', 'background-image-position', [
                    'style'        => 'width:112px',
                    'multioptions' => Tool::getBackgroundImagePositions(),
                ]),
                $tab->createElement('select', 'background-image-size', [
                    'style'        => 'width:90px',
                    'multioptions' => Tool::getBackgroundImageSizes(),
                ]),
            ]),
        ]);
        $tab->addAccordion('shadow', [
            $tab->createElement('number', 'shadow-margin-left', [
                'label' => 'field.margin.left',
            ]),
            $tab->createElement('number', 'shadow-margin-top', [
                'label' => 'field.margin.top',
            ]),
            $tab->createElement('number', 'shadow-unschaerfe', [
                'label' => 'field.unschaerfe',
            ]),
            $tab->createElement('color', 'shadow-color', [
                'label' => 'field.color',
            ]),
        ]);
        $tab->addAccordion('hyperlink', [
            $tab->createElement('color', 'hyperlink-color', [
                'label' => 'field.hyperlink.color',
            ]),
            $tab->createElement('color', 'hyperlink-visited-color', [
                'label' => 'field.hyperlink.visited.color',
            ]),
            $tab->createElement('color', 'hyperlink-hover-color', [
                'label' => 'field.hyperlink.hover.color',
            ]),
        ]);
        if (Identity::isAdmin()) {
            $tab->addAccordion('enhancedsettings', [
                $tab->createElement('textarea', 'raw-css', [
                    'label' => 'field.raw-css',
                ]),
            ]);
        }

        $form->addTab('options')
            ->addElement('select', 'viewon', [
                'label'        => 'field.view_on',
                'required'     => true,
                'multiOptions' => Popover::getViewOns(),
            ])
            ->addElement('select', 'position', [
                'label'        => 'field.position',
                'required'     => true,
                'multiOptions' => Popover::getPositions(),
            ])
            ->addElement('select', 'background', [
                'label'        => 'field.background',
                'required'     => true,
                'multiOptions' => Popover::getBackgrounds(),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getAdd(Weave $weave)
    {
        $creationTypes = [
            'new'       => 'creation_type.new',
            'duplicate' => 'creation_type.duplicate',
            'text'      => 'creation_type.text',
        ];
        $existingPopovers = [];
        foreach (PopoverRepository::findByWeave($weave) as $popover) {
            $existingPopovers[$popover->getId()] = $popover->getTitle();
        }

        if (empty($existingPopovers)) {
            unset($creationTypes['duplicate']);
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.popover.add');

        $form->addTab('meta')
            ->addElement('text', 'title', [
                'required' => true,
                'label'    => 'field.title',
            ])
            ->addElement('select', 'creation_type', [
                'label'        => 'field.creation_type',
                'required'     => true,
                'multiOptions' => $creationTypes,
            ])
            ->addElement('select', 'popover', [
                'label'        => ' ',
                'multiOptions' => $existingPopovers,
            ])
            ->addElement('textarea', 'text', [
                'label' => ' ',
            ]);

        $form->addTab('options')
            ->addElement('select', 'viewon', [
                'label'        => 'field.view_on',
                'required'     => true,
                'multiOptions' => Popover::getViewOns(),
            ])
            ->addElement('select', 'position', [
                'label'        => 'field.position',
                'required'     => true,
                'multiOptions' => Popover::getPositions(),
            ])
            ->addElement('select', 'background', [
                'label'        => 'field.background',
                'required'     => true,
                'multiOptions' => Popover::getBackgrounds(),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.popover.remove');

        return $form;
    }

    public static function getRestore()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.item.restore');

        $form->addElement(
            'html',
            'info',
            [
                'value' => static::_getTranslate()->translate('info.entry.restore'),
            ]
        )
            ->addElement(
                'hidden',
                'confirmation',
                [
                    'value'    => 'confirmation',
                    'required' => true,
                ]
            );

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    public static function getRestoreDeleted(Weave $weave)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.footnote.restore.deleted');

        $history = [];
        foreach (PopoverRepository::findDeletedByWeave($weave, ['limit' => 25]) as $popover) {
            $history[] = '<tr>
		        <td>' . date('d/m/y - H:i:s', $popover->getLastModified()) . '
		        <td>' . $popover->getTitle() . '
		        <td><button  class="ui-button ui-widget ui-state-default ui-corner-all ui-button-text-icon-primary" style="margin:0"
		        onclick="Dialog.restoreFootnote(' . $popover->getId() . '); return false;">Restore</button>';
        }

        if ($history) {
            $form->addElement(
                'html',
                'history',
                [
                    'value' => '<table style="width:569px"><tr><th>deleted at<th>title<th>' . implode(
                            null,
                            $history
                        ) . '</table>',
                ]
            );
        } else {
            $form->addElement(
                'html',
                'history',
                [
                    'value' => 'No deleted footnotes found.',
                ]
            );
        }

        $form->addCancelButton('ok');

        return $form;
    }

}