<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Repository\UserRepository;

class PortalFormGenerator extends BasicFormGenerator
{

    public static function getAdvancedSearch()
    {
        $form = new DefaultForm();
        $form->setAttrib('action', '/portal/search-result')
            ->setMethod('get');

        $form->addElement('hidden', 'resulttype', [
            'value' => 'research',
        ]);
        $form->addElement('select', 'format', [
            'label'        => 'field.format',
            'multiOptions' => ['json' => 'json', 'html' => 'html'],
            'value'        => 'html',
        ]);

        $form->addTab('research', [], ['elementsBelongTo' => null])
            ->addElement('text', 'fulltext', [
                'label'    => 'field.full.text',
                'onchange' => '$("[name=fulltext]").val($(this).val())',
            ])
            ->addElement('text', 'title', [
                'label'                     => 'field.title',
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'      => '/research/search',
                    'property' => 'title',
                ]),
            ])
            ->addElement('select', 'author', [
                'label'                     => 'field.author',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('author', []), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search',
                    'placeholder' => 'Add author...',
                    'maxcount'    => 1,
                ]),
            ])
            ->addElement('text', 'keyword', [
                'label'       => 'field.keyword',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ])
            ->addElement('select', 'portal', [
                'label'        => 'field.portal',
                'multiOptions' => ['' => ''] + static::_getPortalList(false),
            ])
            ->addInlineElement([
                $form->createElement('checkbox', 'status-progress', [
                    'label'       => 'field.status',
                    'description' => 'research.status.progress',
                ]),
                $form->createElement('checkbox', 'status-published', [
                    'description' => 'research.status.published',
                ]),
            ])
            ->addElement('hidden', 'type_research', [
                'value' => 'research',
            ]);

        $form->addTab('user', [], ['elementsBelongTo' => null])
            ->addElement('text', 'fulltext', [
                'label'    => 'field.full.text',
                'onchange' => '$("[name=fulltext]").val($(this).val())',
            ])
            ->addElement('text', 'name', [
                'label'                     => 'field.name',
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'      => '/user/search',
                    'property' => 'name',
                ]),
            ])
            ->addElement('select', 'country', [
                'label'        => 'field.country',
                'multiOptions' => ['' => ''] + static::_getCountryList(),
            ])
            ->addElement('hidden', 'type_user', [
                'value' => 'user',
            ]);

        $form->addTab('work', [], ['elementsBelongTo' => null])
            ->addElement('text', 'fulltext', [
                'label'    => 'field.full.text',
                'onchange' => '$("[name=fulltext]").val($(this).val())',
            ])
            ->addElement('text', 'title', [
                'label'                     => 'field.title',
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'      => '/work/search',
                    'property' => 'title',
                ]),
            ])
            ->addElement('select', 'author', [
                'label'                     => 'field.author',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('author', []), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search',
                    'placeholder' => 'Add author...',
                    'maxcount'    => 1,
                ]),
            ])
            ->addElement('select', 'genre', [
                'label'        => 'field.type',
                'multiOptions' => ['' => ''] + static::_getWorkGenreList(),
            ])
            ->addElement('text', 'keyword', [
                'label'       => 'field.keyword',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ])
            ->addElement('hidden', 'type_work', [
                'value' => 'work',
            ]);

        $form->addTab('project', [], ['elementsBelongTo' => null])
            ->addElement('text', 'fulltext', [
                'label'    => 'field.full.text',
                'onchange' => '$("[name=fulltext]").val($(this).val())',
            ])
            ->addElement('text', 'title', [
                'label'                     => 'field.title',
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'      => '/project/search',
                    'property' => 'title',
                ]),
            ])
            ->addElement('select', 'study', [
                'label'        => 'field.study',
                'multiOptions' => ['' => ''] + static::getMainFieldsOfStudy(),
            ])
            ->addElement('text', 'keyword', [
                'label'       => 'field.keyword',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ])
            ->addElement('text', 'genre', [
                'label' => 'field.genre',
            ])
            ->addElement('text', 'area', [
                'label' => 'field.area',
            ])
            ->addElement('text', 'investigator', [
                'label' => 'field.investigator',
            ])
            ->addElement('select', 'contributor', [
                'label'                     => 'field.contributor',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('contributor', []), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search',
                    'placeholder' => 'Add user...',
                    'maxcount'    => 1,
                ]),
            ])
            ->addElement('select', 'supervisor', [
                'label'                     => 'field.supervisor',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('supervisor', []), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search',
                    'placeholder' => 'Add user...',
                    'maxcount'    => 1,
                ]),
            ])
            ->addElement('hidden', 'type_project', [
                'value' => 'project',
            ]);

        $form->addTab('degree', [], ['elementsBelongTo' => null])
            ->addElement('text', 'fulltext', [
                'label'    => 'field.full.text',
                'onchange' => '$("[name=fulltext]").val($(this).val())',
            ])
            ->addElement('text', 'title', [
                'label'                     => 'field.name',
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'      => '/degree/search',
                    'property' => 'title',
                ]),
            ])
            ->addElement('select', 'level', [
                'label'        => 'field.level',
                'multiOptions' => ['' => ''] + DegreeFormGenerator::getLevels(),
            ])
            ->addElement('text', 'keyword', [
                'label'                     => 'field.keyword',
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'      => '/degree/search',
                    'property' => 'keywords',
                ]),
            ])
            ->addElement('select', 'portal', [
                'label'        => 'field.portal',
                'multiOptions' => ['' => ''] + static::_getPortalList(false),
            ])
            ->addElement('hidden', 'type_degree', [
                'value' => 'degree',
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getRegister()
    {
        $form = new DefaultForm();
        $form->setMethod('post');

        $form->addElement('text', 'lastname', [
            'required' => true,
            'label'    => 'field.lastname',
        ])
            ->addElement('text', 'firstname', [
                'required' => true,
                'label'    => 'field.firstname',
                'info'     => 'Please add initials',
            ])
            ->addElement('html', 'firstname-info', [
                'required' => true,
                'value'    => 'If you intend to use an alias or a pseudonym please contact the <a href="' . My_View_Helper_MaskEmail::mask('support@researchcatalogue.net',
                        'Usage of alias or pseudonym') . '">user support</a> later and use your real name here.',
            ])
            ->addElement('text', 'email', [
                'required'   => true,
                'label'      => 'field.email',
                'validators' => [
                    'EmailAddress',
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new UserRepository(),
                            'field'   => 'email',
                            'exclude' => [
                                'field' => 'id',
                                'value' => isset($user) ? $user->getId() : '',
                            ],
                        ],
                    ],
                ],
            ])
            ->addElement('password', 'password', [
                'required' => true,
                'label'    => 'field.password',
            ])
            ->addElement('checkbox', 'announcements', [
                'label'       => 'field.announcements',
                'description' => 'description.announcements',
            ])
            ->addElement('checkbox', 'digest', [
                'label'       => 'field.digest',
                'description' => 'description.digest',
            ])
            ->addElement('checkbox', 'messageNotification', [
                'label'       => 'field.message.notification',
                'description' => 'description.message.notification',
            ])
            ->addElement('checkbox', 'hideProfile', [
                'label'       => 'field.hideProfile',
                'description' => 'description.hideProfile',
            ]);

        $form->addSubmitButton('fullaccount');
        $form->addSubmitButton('basicaccount');

        return $form;
    }

    public static function getUpgrade()
    {
        $form = new DefaultForm();
        $form->setMethod('post');

        $form
            ->addElement('select', 'country', [
                'required'     => true,
                'label'        => 'field.country',
                'multiOptions' => ['' => ''] + array_combine(static::_getCountryList(), static::_getCountryList()),
            ])
            ->addElement('text', 'city', [
                'required' => true,
                'label'    => 'field.city',
            ])
            ->addElement('text', 'postcode', [
                'required' => true,
                'label'    => 'field.postcode',
            ])
            ->addElement('text', 'address1', [
                'required' => true,
                'label'    => 'field.address1',
            ])
            ->addElement('text', 'address2', [
                'label' => 'field.address2',
            ])
            ->addElement('text', 'address3', [
                'label' => 'field.address3',
            ])
            ->addElement('text', 'telephone', [
                'required' => true,
                'label'    => 'field.telephone',
                'info'     => 'SAR may call you under this number during normal business hours to verify your identity. Please include the country dialing code.',
            ])
            ->addElement('hidden', 'email', [
                'required' => true,
            ])
            ->addElement('hidden', 'name', [
                'required' => true,
            ]);

        $form->addSubmitButton('letteragreement');

        return $form;
    }

    public static function getForgottenPassword()
    {
        $form = new DefaultForm();
        $form->setMethod('post');

        $form->addElement('text', 'email', [
            'required'   => true,
            'label'      => 'field.email',
            'validators' => [
                'EmailAddress',
                [
                    'RecordExists',
                    false,
                    [
                        'adapter' => new UserRepository(),
                        'field'   => 'email',
                    ],
                ],
            ],
        ]);

        $form->addSubmitButton('password.change');

        return $form;
    }

    public static function getUnsubscribe()
    {
        $form = new DefaultForm();
        $form
            ->addElement('checkbox', 'announcements', [
                'label'       => 'field.announcements',
                'description' => 'description.announcements',
            ])
            ->addElement('checkbox', 'digest', [
                'label'       => 'field.digest',
                'description' => 'description.digest',
            ])
            ->addSubmitButton();

        return $form;
    }

    public static function getChangePassword()
    {
        return SettingsFormGenerator::getChangePassword();
    }

}

?>
