<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Project;

class ProjectFormGenerator extends BasicFormGenerator
{
    public static function getAdd(Project $project)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.project.add');

        $principalInvestigators = static::_getLazyUserList(static::_getParam('meta.principalinvestigators', [$project->getPrincipalInvestigator()->getId()]), false);

        $form->addTab('meta')
            ->addElement('languageBar', 'language-bar')
            ->addMultiLanguageElement('text', 'title', [
                'required'   => true,
                'label'      => 'field.title',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Exposition::VAL_TITLE_LENGTH_MIN,
                            Exposition::VAL_TITLE_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('select', 'mainfieldofstudytype', [
                'required'     => true,
                'label'        => 'field.main-field-of-study',
                'multiOptions' => ['' => ''] + self::getMainFieldsOfStudy(),
            ])
            ->addElement('text', 'mainfieldofstudy', [
                'required' => true,
                'label'    => ' ',
            ])
            ->addElement('text', 'keywords', [
                'label'       => 'field.keywords',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'required'   => true,
                'label'      => 'field.abstract',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Project::VAL_DESCRIPTION_LENGTH_MIN,
                            Project::VAL_DESCRIPTION_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('datePicker', 'startdate', [
                'required' => true,
                'label'    => 'field.startdate',
            ])
            ->addElement('datePicker', 'enddate', [
                'label' => 'field.enddate',
            ])
            ->addElement('text', 'genre', [
                'label'      => 'field.project-genre',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Project::VAL_GENRE_LENGTH_MIN,
                            Project::VAL_GENRE_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('text', 'area', [
                'label'      => 'field.area',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Project::VAL_AREA_LENGTH_MIN,
                            Project::VAL_AREA_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('number', 'ects', [
                'label' => 'field.ects',
            ])
            ->addElement('text', 'referenceurl', [
                'label' => 'field.reference-url',
            ])
            ->addElement('multiselect', 'principalinvestigators', [
                'label'                     => 'field.principal-investigator',
                'multiOptions'              => $principalInvestigators,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search',
                    'placeholder' => 'Add user...',
                    'maxcount'    => 1,
                ]),
            ]);

        self::_addMediaTab($form, $project);
        self::_addCollaborationTab($form, $project)
            ->getElement('coauthors')
            ->setOptions(['label' => 'field.researcher']);
        self::_addRelationTab($form, $project);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getEdit(Project $project)
    {
        $form = static::getAdd($project);
        $form->setTitle('dialog.project.edit', $project);

        return $form;
    }
}

?>