<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;

class SimpleMediaFormGenerator extends BasicFormGenerator
{

    public static function getAdd()
    {
        $jsOnChangeEvent = '$("input[name*=name]",$(this).closest("form")).val($(this).fileinput("getValue").replace(/\.\w{3,4}$/i,"")).select();';

        $form = new DefaultForm();
        $form->setTitle('dialog.file.add');

        $form->addTab('image')
            ->addElement('file', 'media', [
                'label'      => 'field.image',
                'onchange'   => $jsOnChangeEvent,
                'validators' => [
                    ['MimeType', false, 'image'],
                ],
            ])
            ->addElement('text', 'name', [
                'required'   => true,
                'label'      => 'field.name',
                'validators' => [
                    ['StringLength', false, [SimpleMedia::VAL_TITLE_LENGTH_MIN, SimpleMedia::VAL_TITLE_LENGTH_MAX]],
                ],
            ])
            ->addElement('text', 'copyrightholder', [
                'required' => true,
                'label'    => 'field.copyright',
            ])
            ->addElement('textarea', 'description', [
                'label' => 'field.description',
            ])
            ->addElement('hidden', 'mediatype', [
                'required' => true,
                'readonly' => true,
                'value'    => MediaType::IMAGE,
            ])
            ->addCancelButton()
            ->addSubmitButton();

        $form->addTab('video')
            ->addElement('file', 'thumb', [
                'label'      => 'field.preview',
                'validators' => [
                    ['MimeType', false, 'image'],
                ],
            ])
            ->addElement('file', 'media', [
                'label'      => 'field.video',
                'onchange'   => $jsOnChangeEvent,
                'validators' => [
                    ['MimeType', false, ['video', 'application/octet-stream']],
                ],
            ])
            ->addElement('text', 'name', [
                'required'   => true,
                'label'      => 'field.name',
                'validators' => [
                    ['StringLength', false, [SimpleMedia::VAL_TITLE_LENGTH_MIN, SimpleMedia::VAL_TITLE_LENGTH_MAX]],
                ],
            ])
            ->addElement('text', 'copyrightholder', [
                'required' => true,
                'label'    => 'field.copyright',
            ])
            ->addElement('textarea', 'description', [
                'label' => 'field.description',
            ])
            ->addElement('hidden', 'mediatype', [
                'required' => true,
                'readonly' => true,
                'value'    => MediaType::VIDEO,
            ])
            ->addCancelButton()
            ->addSubmitButton();

        $form->addTab('audio')
            ->addElement('file', 'media', [
                'label'      => 'field.audio',
                'onchange'   => $jsOnChangeEvent,
                'validators' => [
                    ['MimeType', false, ['audio', 'application/octet-stream']],
                ],
            ])
            ->addElement('text', 'name', [
                'required'   => true,
                'label'      => 'field.name',
                'validators' => [
                    ['StringLength', false, [SimpleMedia::VAL_TITLE_LENGTH_MIN, SimpleMedia::VAL_TITLE_LENGTH_MAX]],
                ],
            ])
            ->addElement('text', 'copyrightholder', [
                'required' => true,
                'label'    => 'field.copyright',
            ])
            ->addElement('textarea', 'description', [
                'label' => 'field.description',
            ])
            ->addElement('hidden', 'mediatype', [
                'required' => true,
                'readonly' => true,
                'value'    => MediaType::AUDIO,
            ])
            ->addCancelButton()
            ->addSubmitButton();

        $form->addTab('pdf')
            ->addElement('file', 'thumb', [
                'label'      => 'field.preview',
                'validators' => [
                    ['MimeType', false, 'image'],
                ],
            ])
            ->addElement('file', 'media', [
                'label'      => 'field.pdf',
                'onchange'   => $jsOnChangeEvent,
                'validators' => [
                    ['MimeType', false, ['application/pdf', 'application/x-pdf']],
                ],
            ])
            ->addElement('text', 'name', [
                'required'   => true,
                'label'      => 'field.name',
                'validators' => [
                    ['StringLength', false, [SimpleMedia::VAL_TITLE_LENGTH_MIN, SimpleMedia::VAL_TITLE_LENGTH_MAX]],
                ],
            ])
            ->addElement('text', 'copyrightholder', [
                'required' => true,
                'label'    => 'field.copyright',
            ])
            ->addElement('textarea', 'description', [
                'label' => 'field.description',
            ])
            ->addElement('hidden', 'mediatype', [
                'required' => true,
                'readonly' => true,
                'value'    => MediaType::PDF,
            ])
            ->addCancelButton()
            ->addSubmitButton();

        $form->addTab('text')
            ->addElement('text', 'name', [
                'required'   => true,
                'label'      => 'field.name',
                'validators' => [
                    ['StringLength', false, [SimpleMedia::VAL_TITLE_LENGTH_MIN, SimpleMedia::VAL_TITLE_LENGTH_MAX]],
                ],
            ])
            ->addElement('text', 'copyrightholder', [
                'required' => true,
                'label'    => 'field.copyright',
            ])
            ->addElement('textarea', 'textcontent', [
                'data-editor' => true,
                'label'       => 'field.content',
            ])
            ->addElement('textarea', 'description', [
                'label' => 'field.description',
            ])
            ->addElement('hidden', 'mediatype', [
                'required' => true,
                'readonly' => true,
                'value'    => MediaType::HTML,
            ])
            ->addCancelButton()
            ->addSubmitButton();

        return $form;
    }

    public static function getEdit(SimpleMedia $simpleMedia)
    {
        $form = self::getAdd();
        $form->setTitle('dialog.file.edit', $simpleMedia);

        return $form;
    }

    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.file.remove');

        return $form;
    }

    public static function getAddTemplate()
    {
        $form = self::getAdd();
        $form->setTitle('dialog.save.as.new.template');

        foreach ($form->getTabs() as $tab) {
            $tabName = substr($tab->getName(), 3); #skip 'tab'
            if ($tabName != 'text') {
                $form->removeTab($tabName);
            }
        }

        return $form;
    }
}

?>