<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class Mail extends Zend_Mail
{

    /**
     * @var array
     */
    static protected $fileCache;

    /**
     * @param string $email
     * @param string $name
     * @return $this
     */
    public function setFrom($email, $name = 'Research Catalogue')
    {
        return parent::setFrom($email, $name);
    }

    public function buildHtml()
    {
        // Important, without this line the example don't work!
        // The images will be attached to the email but these will be not
        // showed inline
        $this->setType(Zend_Mime::MULTIPART_RELATED);

        $matches = [];
        preg_match_all("~<img.*?src=['\"]((https?://.*)|(file://.*))['\"]~iU", $this->getBodyHtml()->getRawContent(), $matches);

        if (count($matches[1]) === 0) {
            return true;
        }

        $retVal = true;
        foreach (array_unique($matches[1]) as $filename) {
            if (strpos($filename, 'http://') !== false || strpos($filename, 'https://') !== false) {
                $retVal = $retVal && $this->replaceRemoteFile($filename);
            } else if (strpos($filename, 'file://') !== false) {
                $retVal = $retVal && $this->replaceLocalFile($filename);
            } else {
                $retVal = false;
            }
        }

        return $retVal;
    }

    protected function replaceRemoteFile($filename)
    {
        if (isset(self::$fileCache[$filename])) {
            return $this->replaceFile($filename);
        }

        if (!is_array(getimagesize($filename))) {
            return false;
        }

        $ch = curl_init($filename);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_exec($ch);

        self::$fileCache[$filename] = [
            'content'  => file_get_contents($filename),
            'mimetype' => curl_getinfo($ch, CURLINFO_CONTENT_TYPE),
        ];

        return $this->replaceFile($filename);
    }

    protected function replaceLocalFile($filename)
    {
        if (isset(self::$fileCache[$filename])) {
            return $this->replaceFile($filename);
        }

        if (!file_exists($filename) || !is_readable($filename)) {
            return false;
        }

        self::$fileCache[$filename] = [
            'content'  => file_get_contents($filename),
            'mimetype' => $this->mimeByExtension($filename),
        ];

        return $this->replaceFile($filename);
    }

    protected function replaceFile($filename)
    {

        $at = $this->createAttachment(self::$fileCache[$filename]['content']);
        $at->type = self::$fileCache[$filename]['mimetype'];
        $at->disposition = Zend_Mime::DISPOSITION_INLINE;
        $at->encoding = Zend_Mime::ENCODING_BASE64;
        $at->id = 'cid_' . md5_file($filename);
        if (strpos($filename, 'file://') === 0) {
            $at->filename = pathinfo(substr($filename, 8), PATHINFO_FILENAME);
        } else {
            $at->filename = substr($filename, -strlen($filename) + strrpos($filename, '/') + 1);
        }
        $this->setBodyHtml(str_replace($filename, 'cid:' . $at->id, $this->getBodyHtml()->getRawContent()), 'UTF-8', Zend_Mime::ENCODING_8BIT);
        return true;
    }

    protected function mimeByExtension($filename)
    {
        $extension = pathinfo($filename, PATHINFO_EXTENSION);
        switch ($extension) {
            case 'gif':
                $type = 'image/gif';
                break;
            case 'jpg':
            case 'jpeg':
                $type = 'image/jpg';
                break;
            case 'png':
                $type = 'image/png';
                break;
            default:
                $type = 'application/octet-stream';
        }

        return $type;
    }
}

?>
