<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Application;

class My_Validate_Application extends Zend_Validate_Abstract
{

    /**
     * Defined by Zend_Validate_Interface
     * Returns true if $value is a valid date of the format YYYY-MM-DD
     * If optional $format or $locale is set the date format is checked
     * according to Zend_Date, see Zend_Date::isDate()
     *
     * @param Application $value
     * @param ApplicationDefaultForm $form
     * @return bool
     */
    public function isValid($value, ApplicationDefaultForm $form = null)
    {
        $this->_setValue($value);

        $values = $form->getValuesIncludingFiles(false);

        $statusForm = ApplicationFormGenerator::getEdit($value, true);
        if (!$statusForm->isValid($values)) {
            $messages = array_merge(['application_form' => [], 'application_file' => []], $statusForm->getMessages());
            foreach (['application_form', 'application_file'] as $tab) {
                foreach ($statusForm->getTab($tab)->getElements() as $key => $element) {
                    if ($element instanceof Zend_Form_Element_File && $element->isRequired() && !isset($messages[$tab][$key]) && !isset($values[$tab][$key])) {
                        $messages[$tab][$key]['fileUploadErrorNoFile'] = 'File was not uploaded';
                    }
                }
            }
        } else {
            $messages = [];
        }

        foreach ($messages as $key => $elements) {
            if (empty($elements)) {
                unset($messages[$key]);
            }
        }

        $this->_messages = $messages;

        return empty($messages);
    }
}
