<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\MediaType;

class ToolFactory
{
    const TOOL_PICTURE = 'picture';
    const TOOL_SLIDESHOW = 'slideshow';
    const TOOL_VIDEO = 'video';
    const TOOL_AUDIO = 'audio';
    const TOOL_TEXT = 'text';
    const TOOL_SIMPLE_TEXT = 'simpletext';
    const TOOL_PDF = 'pdf';
    const TOOL_PLAY = 'play';
    const TOOL_SHAPE = 'shape';
    const TOOL_COMMENT = 'comment';
    const TOOL_EMBED = 'embed';
    const TOOL_OBJECT_VIEWER = 'objectviewer';

    /**
     * @var array
     */
    protected static $_supportedMediaTypes = [];

    /**
     * @var array
     */
    protected static $_mediaTypeDefaultToolMapping = [
        MediaType::IMAGE   => self::TOOL_PICTURE,
        MediaType::VIDEO   => self::TOOL_VIDEO,
        MediaType::AUDIO   => self::TOOL_AUDIO,
        MediaType::HTML    => self::TOOL_TEXT,
        MediaType::TEXT    => self::TOOL_SIMPLE_TEXT,
        MediaType::PDF     => self::TOOL_PDF,
        MediaType::SHAPE   => self::TOOL_SHAPE,
        MediaType::ARCHIVE => self::TOOL_OBJECT_VIEWER,
    ];

    /**
     * @return array
     */
    public static function getTools()
    {
        return [
            self::TOOL_SIMPLE_TEXT,
            self::TOOL_TEXT,
            self::TOOL_PICTURE,
            self::TOOL_AUDIO,
            self::TOOL_VIDEO,
            self::TOOL_SLIDESHOW,
            self::TOOL_PDF,
            self::TOOL_PLAY,
            self::TOOL_SHAPE,
            self::TOOL_COMMENT,
            self::TOOL_EMBED,
            self::TOOL_OBJECT_VIEWER,
        ];
    }

    /**
     * @param string $tool
     * @return bool
     */
    public static function exists($tool)
    {
        return array_search($tool, self::getTools()) !== false;
    }

    /**
     * @param string $tool
     * @return \Rc\Domain\Model\Research\Exposition\Weave\Tool
     * @throws Exception
     */
    public static function buildByName($tool)
    {
        if (!self::exists($tool)) {
            throw new Exception('TOOL_DONT_EXISTS [' . $tool . ']');
        }
        $objectName = self::getObjectName($tool);
        return new $objectName;
    }

    /**
     * @param string $tool
     * @return string
     */
    public static function getObjectName($tool)
    {
        return 'Rc\Domain\Model\Research\Exposition\Weave\Tool\\' . ucfirst($tool) . 'Tool';
    }

    /**
     * @param string $mediaType
     * @return Tool
     * @throws Exception
     */
    public static function buildByMediaType($mediaType)
    {
        return self::buildByName(self::getToolNameByMediaType($mediaType));
    }

    /**
     * @param string $mediaType
     * @return string
     * @throws Exception
     */
    public static function getToolNameByMediaType($mediaType)
    {
        if (!self::isMediaTypeSupported($mediaType)) {
            throw new Exception('MEDIA_TYPE_IS_NOT_SUPPROTED [' . $mediaType . ']');
        }

        return self::$_mediaTypeDefaultToolMapping[$mediaType];
    }

    /**
     * @param string $mediaType
     * @return bool
     */
    public static function isMediaTypeSupported($mediaType)
    {
        return isset(self::$_mediaTypeDefaultToolMapping[$mediaType]);
    }
}
