<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

/**
 * Class NewsletterRepository
 * @method static Newsletter findById($id, $limit = null)
 */
class NewsletterRepository extends MySqlRepository
{

    /**
     * @param \Rc\Domain\Model\Portal $portal
     * @param int $limit
     * @param string $orderBy
     * @return Newsletter[]
     */
    public static function findByPortal(\Rc\Domain\Model\Portal $portal, $limit = null, $orderBy = null)
    {
        return static::findByPortalId($portal->getId(), $limit, $orderBy);
    }

    /**
     * @param int $portalid
     * @param int $limit
     * @param string $orderBy
     * @return Newsletter[]
     */
    public static function findByPortalId($portalid, $limit = null, $orderBy = null)
    {
        return static::_findBy('portalid', $portalid, 0, $limit, $orderBy);
    }

    /**
     * @param string $email
     * @param string $hashedContent
     * @return int
     */
    public static function countByEmailAndContent($email, $hashedContent)
    {
        $result = static::_query('
            SELECT
                id
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                succeededEmails LIKE "%' . static::_escape($email) . '%" AND
                hashedContent = "' . static::_escape($hashedContent) . '" AND
                testmode = 0
            LIMIT
                1
        ');

        return count(static::_fetchAndFreeResults($result));
    }

    /**
     * @param int $page
     * @param int $limit
     * @param array $orderBy
     * @return Newsletter[]
     */
    public static function findNotApproved($page = 0, $limit = null, $orderBy = null)
    {
        $result = static::_query('
            SELECT SQL_CALC_FOUND_ROWS
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                started IS NULL AND
                approved IS NULL AND
                testmode = 0
            ORDER BY
                ' . static::_getValidOrderBy($orderBy) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        return static::_fetchAndFreeResults($result);
    }

    /**
     * @param int $page
     * @param int $limit
     * @param array $orderBy
     * @return Newsletter[]
     */
    public static function findNotCompleted($page = 0, $limit = null, $orderBy = null)
    {
        $result = static::_query('
            SELECT SQL_CALC_FOUND_ROWS
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                ended IS NULL AND
                delivery <= NOW() AND
                approved = 1 AND
                (lastUpdated IS NULL OR lastUpdated < "' . date('Y-m-d H:i:s', time() - 300) . '") AND
                testmode = 0
            ORDER BY
                ' . static::_getValidOrderBy($orderBy) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        return static::_fetchAndFreeResults($result);
    }
}
