<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

/**
 * @method static ProfileGroup findById($id, $limit = null)
 */
class ProfileGroupRepository extends MySqlRepository
{

    const GROUP_NAME_EXPOSITION = 'research expositions';
    const GROUP_NAME_EXPOSITION_COLLABORATED = 'research expositions (collaborated)';
    const GROUP_NAME_EXPOSITION_SUPERVISED = 'research expositions (supervised)';
    const GROUP_NAME_WORK = 'works';
    const GROUP_NAME_WORK_COLLABORATED = 'works (collaborated)';
    const GROUP_NAME_WORK_SUPERVISED = 'works (supervised)';
    const GROUP_NAME_DEGREE_SUPERVISED = 'degrees (supervised)';
    const GROUP_NAME_ACTIVITY = 'activities';
    const GROUP_NAME_ACTIVITY_COLLABORATED = 'activities (collaborated)';
    const GROUP_NAME_ACTIVITY_SUPERVISED = 'activities (supervised)';
    const GROUP_NAME_APPLICATION = 'application';
    const GROUP_NAME_APPLICATION_COLLABORATED = 'application (collaborated)';
    const GROUP_NAME_APPLICATION_SUPERVISED = 'application (supervised)';

    /**
     * @param int $userId
     * @param int $page
     * @param int $limit
     * @param array $orderBy
     * @return ProfileGroup[]
     */
    public static function findByUserId($userId, $page = null, $limit = null, $orderBy = ['position' => 'ASC'])
    {
        $result = static::_query('
            SELECT
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . '
            ORDER BY
                ' . static::_getValidOrderBy($orderBy) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        return static::_fetchAndFreeResults($result);
    }

    public static function getMaximumPosition($userId)
    {
        $result = static::_query('
            SELECT
                MAX(position)
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . '
        ');

        return (int)reset($result->fetch_array());
    }

    public static function countByUserIdAndName($userId, $name)
    {
        $result = static::_query('
            SELECT
                COUNT(*)
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . ' AND
                name = "' . static::_escape($name) . '"
        ')->fetch_array();

        return (int)reset($result);
    }

    public static function add(MySqlEntity $object)
    {
        if ($object instanceof ProfileGroup && $object->getPosition() === null) {
            $object->setPosition(self::getMaximumPosition($object->getUserId()) + 1);
        }

        return parent::add($object);
    }

    /**
     * @param int $userId
     * @param string $name
     * @return ProfileGroup
     */
    public static function findByUserIdAndName($userId, $name)
    {
        $result = static::_query('
            SELECT
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . ' AND
                name = "' . static::_escape($name) . '"
        ');

        return static::_fetchAndFreeResult($result);
    }

}
