<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\User;

class ReportRepository extends CommentRepository
{

    /**
     * @param $page
     * @param null $limit
     * @param null $orderBy
     * @return Report[]
     */
    public static function findNotDeleted($page, $limit = null, $orderBy = null)
    {
        return static::_findBy('deleted', [null], $page, $limit, $orderBy);
    }

    public static function findByResearchId($researchid, $page, $limit = null, $orderBy = null)
    {
        return static::_findBy([
            'researchid',
            'deleted',
        ], [
            $researchid,
            null,
        ], $page, $limit, $orderBy);
    }

    public static function findByPerson(User $user, $page, $limit = null, $orderBy = null)
    {
        return static::findByPersonId($user->getId(), $page, $limit, $orderBy);
    }

    public static function findByPersonId($personId, $page, $limit = null, $orderBy = null)
    {
        return static::_findBy([
            'userid',
            'deleted',
        ], [
            $personId,
            null,
        ], $page, $limit, $orderBy);
    }

    public static function findByResearchAndPortal(Exposition $research, Portal $portal, $page, $limit = null, $orderBy = null)
    {
        return static::findByResearchIdAndPortalId($research->getId(), $portal->getId(), $page, $limit, $orderBy);
    }

    public static function findByResearchIdAndPortalId($researchid, $portalid, $page, $limit = null, $orderBy = null)
    {
        return static::_findBy([
            'researchid',
            'portalid',
            'deleted',
        ], [
            $researchid,
            $portalid,
            null,
        ], $page, $limit, $orderBy);
    }
}

?>