<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class My_Form_Decorator_DefaultElement extends Zend_Form_Decorator_Abstract
{

    public function render($content)
    {
        $element = $this->getElement();
        if (!$element instanceof Zend_Form_Element) {
            return $content;
        }

        if (null === $element->getView()) {
            return $content;
        }

        switch ($element->getType()) {
            case 'Zend_Form_Element_Hidden':
                $output = $this->_buildInput();
                break;

            case 'Zend_Form_Element_Button':
                $output = $this->_buildButton();
                break;

            case 'My_Zend_Form_Element_Html':
                $output = $this->_buildHtml();
                break;

            case 'My_Zend_Form_Element_LanguageBar':
                $output = $this->_buildLanguageBar();
                break;

            case 'Zend_Form_Element_File':
                $output =
                    $this->_buildFile() .
                    $this->_buildInfo() .
                    $this->_buildDescription();
                break;

            default:
                $output =
                    $this->_buildInput() .
                    $this->_buildInfo() .
                    $this->_buildDescription();
        }

        $separator = $this->getSeparator();
        switch ($this->getPlacement()) {
            case (self::PREPEND):
                return $output . $separator . $content;
            case (self::APPEND):
            default:
                return $content . $separator . $output;
        }
    }

    protected function _buildHtml()
    {
        $element = $this->getElement();
        $element->setAttrib('escape', false);

        return $element->getView()->formHtml(
            $element->getFullyQualifiedName(), $element->getValue(), $element->getAttribs()
        );
    }

    protected function _buildLanguageBar()
    {
        $element = $this->getElement();

        return $element->getView()->formLanguageBar(
            $element->getFullyQualifiedName(), $element->getValue(), $element->getAttribs()
        );
    }

    protected function _buildInput()
    {
        $element = $this->getElement();

        return $element->getView()->{$element->helper}(
            $element->getFullyQualifiedName(), $element->getValue(), $element->getAttribs(), $element->options
        );
    }

    protected function _buildDescription()
    {
        $element = $this->getElement();
        $description = $element->getDescription();

        if (empty($description)) {
            return null;
        }

        return ' <span class="description">' . $element->getTranslator()->translate($description) . '</span>';
    }

    protected function _buildInfo()
    {
        $element = $this->getElement();
        $info = $element->getAttrib('info');

        $element->setAttrib('info', null);

        if (empty($info)) {
            return null;
        }
        return $element->getView()->info($info);
    }

    protected function _buildButton()
    {
        $element = $this->getElement();

        switch ($element->getAttrib('type')) {
            case 'cancel':
                $selector = '[id=' . $element->getId() . ']:last';
                $icon = 'ui-icon-close';
                break;

            case 'submit':
                $selector = '[id=' . $element->getId() . ']:last';
                $icon = 'ui-icon-check';
                break;

            default:
                $selector = '[id=' . $element->getId() . ']:last';
                $icon = $element->getAttrib('icon');
                $element->setAttrib('icon', null);
                break;
        }

        $element->getView()->jQuery()->addOnLoad(
            sprintf('%s("%s").button({icons:{primary: "%s"}});', ZendX_JQuery_View_Helper_JQuery::getJQueryHandler(),
                $selector, $icon)
        );

        return $element->getView()->formButton(
            $element->getFullyQualifiedName(), $element->getTranslator()->translate($element->getLabel()),
            $element->getAttribs()
        );
    }

    protected function _buildFile()
    {
        $element = $this->getElement();
        $translator = $element->getTranslator();
        $attribs = $element->getAttribs() + [
                'id' => $element->getId(),
            ];

        $content = $element->getView()->formFile($element->getName(), $attribs);

        $onLoad = sprintf('%s(".ui-dialog [id=%s]:last").fileinput({inputText:%1$s(".ui-dialog [id=%2$s]:last").val(),buttonText:"%s"}).prev().removeClass("ui-widget-header");',
            ZendX_JQuery_View_Helper_JQuery::getJQueryHandler(), $element->getId(),
            $translator->translate('field.file.upload.add'));

        if (!empty($attribs['data-resource'])) {
            $onLoad .= sprintf(
                '%s(".ui-dialog [id=%s]:last").closest("div").hide();',
                ZendX_JQuery_View_Helper_JQuery::getJQueryHandler(), $element->getId()
            );
        }

        $element->getView()->jQuery()->addOnLoad($onLoad);

        return $content;
    }

}

?>