<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\Resource\AudioResource;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\AudioTool;

/**
 * @property AudioTool $tool
 */
class My_View_Helper_AudioToolDecorator extends My_View_Helper_ToolDecorator
{

    /**
     * @var AudioTool
     */
    protected $tool;

    /**
     * @param AudioTool $tool
     * @param string $renderFor
     * @return mixed
     */
    public function audioToolDecorator($tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    public function _getToolContentForView($options = [])
    {
        $mediaContainer = $this->tool->getMediaContainer();
        if (!$this->getMediaHelper()->canBeRendered($mediaContainer)) {
            return $this->getMediaHelper()->getErrorMessage($mediaContainer);
        }

        $this->addHoverPopupIfNecessary();

        return '<div id="audio-' . $this->tool->getId() . '" width="' . $this->tool->getWidth() . '" height="' . $this->tool->getHeight() . '"
                data-file="' . $this->_getDerivateUrl(AudioResource::DERIVATIVE_TYPE_MP3) . '"
                data-image-width="100" data-image-height="50"
                data-loop="' . $this->tool->isLoop() . '" data-autoplay="' . $this->tool->isAutoPlay() . '"
                data-hide-player="' . $this->tool->isHidePlayer() . '" data-stop-other-players="' . $this->tool->isStopOtherPlayers() . '"
                data-display-volume="' . $this->tool->isDisplayVolume() . '" data-display-minimal="' . $this->tool->isDisplayMinimal() . '"></div>';
    }

    /**
     * @param string $type
     * @return string|null
     */
    protected function _getDerivateUrl($type)
    {
        $entity = $this->getMedia();
        if (!isset($entity)) {
            return null;
        }
        $type = $entity->getDerivative($type);
        if (!isset($type)) {
            return null;
        }
        return $type->getExternalUri();
    }
}

?>