<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool\CommentTool;

/**
 * @property CommentTool $tool
 */
class My_View_Helper_CommentToolDecorator extends My_View_Helper_ToolDecorator
{

    public function CommentToolDecorator(CommentTool $tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    protected function _getToolContentForView($options = [])
    {
        return $this->_getToolContentForEditor();
    }

    protected function _getToolContentForEditor($options = [])
    {
        $this->attributes['data']['resolution'] = $this->tool->getResolution() ?: 'open';
        $this->attributes['data']['assignee-id'] = $this->tool->getAssignee() ? $this->tool->getAssignee()->getId() : '-1';
        $this->attributes['data']['assignee-name'] = $this->tool->getAssignee() ? $this->tool->getAssignee()->getName() : 'none';
        $this->attributes['data']['deadline'] = $this->tool->getDeadline();
        if ($this->attributes['data']['resolution'] == 'resolved') {
            $this->attributes['class'] .= ' comment-resolved';
        }
        if ($this->tool->getTextAuthor()) {
            $this->attributes['data']['text-author'] = $this->tool->getTextAuthor()->getName();
            $this->attributes['data']['text-modified'] = $this->tool->getTextModified();
        }
        $retVal = '<div class="comment-header">' . $this->tool->getAuthor()->getName() . ' - ' . $this->_getModificationDate() . '</div>';
        if ($this->tool->isEditableBy(Identity::getUserOrNull())) {
            $retVal .= '<div class="comment-content"><textarea>' . $this->tool->getTextContent() . '</textarea></div>';
        } else {
            $retVal .= '<div class="comment-content">' . $this->view->text(nl2br($this->tool->getTextContent())) . '</div>';
        }
        return $retVal;
    }

    protected function _getIconContainer()
    {
        $checked = $this->tool->getResolution() == 'resolved' ? 'checked="checked"' : '';
        if ($this->tool->isEditableBy(Identity::getUserOrNull())) {
            return '<div class="icon icon-remove" title="remove comment"></div><input type="checkbox" ' . $checked . '/>';
        } else {
            return parent::_getIconContainer() . '<input type="checkbox" ' . $checked . '/>';
        }
    }

    protected function _showToolInView()
    {
        return true;
    }

    private function _getModificationDate()
    {
        $date = $this->tool->getTextModified();
        if (empty($date)) {
            return 'n/a';
        }
        return trim(trim($date, '['), ']');
    }
}

?>