<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\MediaContainerInterface;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Repository\Research\DegreeRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\Research\ProjectRepository;
use Rc\Domain\Repository\Research\WorkRepository;
use Rc\Domain\Repository\ResearchRepository;

class My_View_Helper_EntityDetailPage extends Zend_View_Helper_Abstract
{

    public function entityDetailPage()
    {
        return $this;
    }

    public function getRelations(Research $entity, array $relations)
    {
        $ids = $entity->getRelations();

        $entities = [
            'exposition' => [],
            'work'       => [],
            'project'    => [],
            'degree'     => [],
        ];
        foreach ($relations as $entity) {
            $entities[$entity->getName()][$entity->getId()] = $entity;
        }
        foreach (ExpositionRepository::findByIds($ids) as $entity) {
            $entities[$entity->getName()][$entity->getId()] = $entity;
        }
        foreach (WorkRepository::findByIds($ids) as $entity) {
            $entities[$entity->getName()][$entity->getId()] = $entity;
        }
        foreach (ProjectRepository::findByIds($ids) as $entity) {
            $entities[$entity->getName()][$entity->getId()] = $entity;
        }
        foreach (DegreeRepository::findByIds($ids) as $entity) {
            $entities[$entity->getName()][$entity->getId()] = $entity;
        }

        return $this->createRelations($entities['exposition'], 'exposition')
            . $this->createRelations($entities['work'], 'work')
            . $this->createRelations($entities['project'], 'project')
            . $this->createRelations($entities['degree'], 'degree');
    }

    public function getMetaTable(array $list)
    {
        $rows = [];
        foreach (array_filter($list) as $th => $td) {
            $rows[] = '<tr><th>' . $this->view->translate($th) . '<td>' . $td;
        }
        return '<table class="meta-table">' . implode('', $rows) . '</table>';
    }

    /**
     * @param Research|Exposition|Degree|Project|Work $entity
     * @return string
     */
    public function getMediaSlides(Research $entity)
    {
        if ($entity instanceof Exposition) {
            if ($entity->getMediaContainer()->hasMedia()) {
                return '
            <div class="meta-media">
                <div class="meta-media-preview">' . $this->sizeMedia($entity->getMediaContainer()) . '</div>
            </div>';
            } else {
                return '<div class="meta-medias" style="padding:5px 0">no media files associated</div>';
            }
        }

        $files = [];
        foreach ($entity->getMediaList() as $simpleMedia) {
            try {

                if (!$entity->isMediaTypeAsThumbSupported($simpleMedia->getMediaType())) {
                    continue;
                }
            } catch (\Exception $e) {
                trigger_error("#" . $this->getId() . "\t" . $e->getMessage(), E_USER_WARNING);
                if (strpos($e->getMessage(),'could not be found.') !== false) {
                    $entity->removeResource($simpleMedia);
                    ResearchRepository::persist($simpleMedia);
                    continue;
                }
            }

            $mediaContainer = $simpleMedia->getMediaContainer();
            $downloadUrl = '/profile/download-media?work=' . $entity->getId() . '&file=' . $simpleMedia->getId();
            $downloadLink = '<a href="' . $downloadUrl . '" target="_blank">download ' . ToolFactory::getToolNameByMediaType($simpleMedia->getMediaType()) . '</a>';

            $files[] = '<li>
                <span class="meta-media-list-thumb">' . ($mediaContainer->hasThumb() ? '<img src="' . $mediaContainer->getThumb()->getExternalUri() . '"/>' : '') . '</span>
                <span class="meta-media-list-preview">' . $this->sizeMedia($mediaContainer, $simpleMedia->getOptions()) . '</span>
                <span class="meta-media-list-title">' . $simpleMedia->getName() . '</span>
                <span class="meta-media-list-description">' . $simpleMedia->getDescription() . '<br/><br/>&copy; ' . $simpleMedia->getCopyright() . ' ' . $downloadLink . '</span>
                <span class="meta-media-list-download">' . $downloadLink . '</span>
            </li>';
        }

        if (empty($files)) {
            return '<div class="meta-medias" style="padding:5px 0">no media files associated</div>';
        }

        return '

            <div>
                <h2 class="meta-headline">Available media</h2>
            </div>
            <ul class="meta-medias">' . implode('', $files) . '</ul>
            <script>
                $(function() {
                    $(".meta-media-list-preview img[href]").lightBox({
                        imageLoading: "/rc/images/lightbox/lightbox-ico-loading.gif",
                        imageBtnClose: "/rc/images/lightbox/lightbox-btn-close.gif",
                        imageBtnPrev: "/rc/images/lightbox/lightbox-btn-prev.gif",
                        imageBtnNext: "/rc/images/lightbox/lightbox-btn-next.gif"
                    });

                    $(".meta-media-list-title, .meta-media-list-thumb").click(function() {
                        var entry = $(this).closest("li");

                        entry.closest("ul").find("li.active").removeClass("active");
                        entry.addClass("active");
                    }).first().trigger("click");
                });
            </script>';
    }

    public function getTitle(Research $entity)
    {
        $date = $this->view->profile()->getDateTitleSuffix($entity);
        $date = !empty($date) ? ' <span style="font-weight:normal;font-size:10px">(' . $date . ')</span>' : '';
        $titleList = $entity->getTitleList()->toArray();
        if (isset($titleList['en'])) {
            return $titleList['en'] . $date;
        }
        $title = reset($titleList);
        $languageKey = key($titleList);
        return '<span style="font-weight: bold; display: block">' . $languageKey . '</span>' . $title . $date;
    }

    public function getDescription(Research $entity)
    {
        $descriptionContent = [];
        $descriptionList = $entity->getDescriptionList()->toArray();
        foreach ($descriptionList as $languageKey => $description) {
            $description = preg_replace('~(https?://([-\w\.]+[-\w])+(:\d+)?(/([\w/_\.#-]*(\?\S+)?[^\.\s])?)?)~', '<a href="$1" target="_blank">$1</a>',
                $this->view->text($description));
            if (count($descriptionList) == 1 && $languageKey == 'en') {
                $descriptionContent[] = $description;
            } else {
                $descriptionContent[] = '<div><div class="content-profile-description-language">' . $languageKey . '</div>' . $description . '</div>';
            }
        }
        return implode('<br/>', $descriptionContent);
    }

    public function getSocialToolbox(Research $entity, $export = false)
    {
        if ($export && Identity::exists() && $entity instanceof Exposition) {
            $export = '<a href="/research/export?research=' . $entity->getid() . '" target="_blank" title="export" class="addthis_default_style at300b"><span class="ui-icon ui-icon-disk"></span></a>';
        }

        return '
    <div class="meta-social-toolbox addthis_toolbox addthis_default_style" addthis:url="' . $this->view->url()->relationUrl($entity) . '" addthis:title="' . htmlentities($entity->getTitle()) . '" addthis:description="' . htmlentities($entity->getDescription()) . '">
        ' . $export . '
        <a class="addthis_button_facebook"></a>
        <a class="addthis_button_twitter"></a>
        <a class="addthis_button_email"></a>
    </div>
    <script type="text/javascript" src="https://s7.addthis.com/js/250/addthis_widget.js"></script>';
    }

    /**
     * @param Research[] $entityList
     * @param                      $type
     * @return string
     */
    protected function createRelations(array $entityList, $type)
    {
        $retVal = [];
        foreach ($entityList as $entity) {
            $retVal[] = '<li><a href="' . $this->view->url()->relationUrl($entity) . '">' . $entity->getTitle() . '</a> (' . $type . ')';
        }
        return implode('', $retVal);
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param array $options
     * @return string
     */
    protected function sizeMedia($mediaContainer, array $options = [])
    {
        $options += [
            'thumb' => $mediaContainer->getContentType() == MediaType::PDF,
            'class' => 'research-image',
        ];
        return $this->view->getHelper('sizeMedia')->auto($mediaContainer, 300, 250, $options);
    }

}

?>