<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class My_View_Helper_FeedEntryTitle extends Zend_View_Helper_Abstract
{

    /**
     * @param EntityChange $change
     * @return string
     */
    public function feedEntryTitle(EntityChange $change)
    {
        $retVal = $this->getPortalMessage($change);
        if (!empty($retVal)) {
            return $retVal;
        }

        $retVal = $this->getPersonMessage($change);
        if (!empty($retVal)) {
            return $retVal;
        }

        return $this->getEntityMessage($change);
    }

    /**
     * @param EntityChange $change
     * @return string
     */
    protected function getPortalMessage(EntityChange $change)
    {
        if (!in_array($change->getType(), [EntityChange::TYPE_PUBLISH, EntityChange::TYPE_SHARE])) {
            return null;
        }

        if (Identity::getUser()->followsPortal($change->getPublishedIn())) {
            return sprintf($this->view->translate('change.portal.' . $change->getType(), [
                $this->view->translate('entity.' . $change->getEntity()->getName()),
                $this->view->link()->portalLink($change->getPublishedIn()),
            ]));
        } elseif (Identity::getUser()->followsPortal($change->getConnectedTo())) {
            return sprintf($this->view->translate('change.portal.' . $change->getType(), [
                $this->view->translate('entity.' . $change->getEntity()->getName()),
                $this->view->link()->portalLink($change->getConnectedTo()),
            ]));
        }

        return null;
    }

    /**
     * @param EntityChange $change
     * @return string
     */
    protected function getEntityMessage(EntityChange $change)
    {
        return sprintf($this->view->translate('change.' . $change->getType(), [
            $this->view->translate('entity.' . $change->getEntity()->getName()),
            $this->view->profileLink($change->getOwner()),
        ]));
    }

    /**
     * @param EntityChange $change
     * @return string
     */
    protected function getPersonMessage(EntityChange $change)
    {
        if (!$change->getUser()) {
            return null;
        }
        if (!Identity::getUser()->followsUser($change->getUser())) {
            return null;
        }

        try {
            $entity = $change->getEntity();
        } catch (Exception $e) {
            return null;
        }

        return sprintf($this->view->translate('change.person.' . $change->getType(), [
            $this->view->translate('entity.' . $entity->getName()),
            $this->view->profileLink($change->getOwner()),
        ]));
    }

}
