<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class My_View_Helper_HtmlElement extends Zend_View_Helper_HtmlElement
{

    protected $_defaultParams = [
        'render' => true,
    ];

    public function htmlElement($tag, $content, $attribs = [], $params = [])
    {
        if (isset($attribs['style']) && is_array($attribs['style'])) {
            $attribs['style'] = $this->_htmlStyles($attribs['style']);
        }

        $params = $params + $this->_defaultParams;
        $tag = $this->view->escape($tag);
        $content = $params['render'] ? $this->view->escape($content) : $content;

        if (isset($params['shortTag'])) {
            return '<' . $tag . $this->_htmlAttribs($attribs) . ' />';
        } else {
            return '<' . $tag . $this->_htmlAttribs($attribs) . '>' . $content . '</' . $tag . '>';
        }
    }

    protected function _htmlAttribs($attribs)
    {
        foreach ($attribs as $key => $value) {
            if (isset($value) && is_array($value)) {
                foreach ($value as $sKey => $sValue) {
                    $attribs[$key . '-' . $sKey] = $sValue;
                }
                unset($attribs[$key]);
            }
        }
        return parent::_htmlAttribs($attribs);
    }

    protected function _htmlStyles(array $styles)
    {
        $retVal = [];
        foreach ($styles as $name => $value) {
            if (isset($value) && is_array($value)) {
                foreach ($value as $subName => $subValue) {
                    if (!empty($subValue) || $value == 0) {
                        $retVal[] = $this->_styleProperty($name . '-' . $subName, $subValue);
                    }
                }
            } else if (!empty($value) || $value == 0) {
                $retVal[] = $this->_styleProperty($name, $value);
            }
        }
        return implode(';', $retVal);
    }

    protected function _styleProperty($name, $value)
    {
        return $name . ':' . $value;
    }

}

?>