<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\User;

class My_View_Helper_JsonLD extends Zend_View_Helper_Abstract
{

    public function jsonLD()
    {
        return $this;
    }

    public function user(User $user = null)
    {
        if (!isset($user)) {
            return null;
        }

        $memberOfPortals = $user->getNonDefaultPortals() ?: $user->getPortals();

        $data = [
            '@context'            => 'http://schema.org/',
            '@type'               => 'Person',
            'address'             => [
                '@type'          => 'PostalAddress',
                'addressCountry' => $user->getTranslatedCountry(),
            ],
            'birthDate' => $user->getYearOfBirth(),
            'memberOf' => $this->view->url()->portalUrl($memberOfPortals[0], ['absolute' => true]),
            'url' => $this->view->url()->profileUrl($user),
            'name' => $user->getTitle(),
        ];

        return '<script type="application/ld+json">' . json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES) . '</script>';
    }

    /**
     * @param Exposition $exposition
     * @return string
     */
    public function exposition(Exposition $exposition = null)
    {
        if (!isset($exposition)) {
            return null;
        }

        $data = [
            '@context'            => 'http://schema.org/',
            '@type'               => 'CreativeWork',
            '@id'                 => $exposition->getDoiUrl(),
            'author'              => array_map(function (User $author) {
                return [
                    '@type' => 'Person',
                    'name'  => $author->getTitle(),
                    'url'   => $this->view->url()->profileUrl($author),
                ];
            }, array_merge([$exposition->getAuthor()], $exposition->getCoAuthors(false))),
            'copyrightYear'       => $exposition->getPublishedDate(false)->format('Y'),
            'datePublished'       => $exposition->getPublishedDate(false)->format('Y-m-d'),
            'discussionUrl'       => $this->view->url()->entityUrl($exposition),
            'isAccessibleForFree' => true,
            'keywords'            => $exposition->getKeywordList(),
            'publisher'           => [
                '@type' => 'Organization',
                'url'   => $this->view->url()->portalUrl($exposition->getPortal(), ['absolute' => true]),
                'name'  => $exposition->getPortal()->getTitle(),
            ],
            'thumbnailUrl'        => $exposition->hasMedia() ? $exposition->getMedia()->getMedia()->getExternalUri() : null,
            'url'                 => $this->view->viewUrl($exposition),
        ];

        return '<script type="application/ld+json">' . json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES) . '</script>';
    }
}
