<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool\ObjectviewerTool;

/**
 * @property ObjectviewerTool $tool
 */
class My_View_Helper_ObjectviewerToolDecorator extends My_View_Helper_ToolDecorator
{

    /**
     * @var array
     */
    protected $imageResourceList;

    /**
     * @var ObjectviewerTool
     */
    protected $tool;

    public function objectviewerToolDecorator(ObjectviewerTool $tool, $renderFor = self::RENDER_FOR_EDITOR)
    {
        $this->imageResourceList = null;
        $this->_init($tool, $renderFor);
        return $this->_renderTool();
    }

    /**
     * @inheritdoc
     */
    protected function getMedia()
    {
        if (!$this->tool->hasMedia()) {
            return parent::getMedia();
        }

        if (isset($this->getImageResourceList()[$this->tool->getStartLevel()])) {
            $index = $this->tool->getStartLevel();
        } else {
            $index = key($this->getImageResourceList());
        }
        return $this->getImageResourceList()[$index];
    }

    protected function _getToolAttribs()
    {
        if ($this->renderFor == self::RENDER_FOR_VIEW) {
            $this->attributes['data']['resources'] = implode(',', $this->getResourceList());
            $this->attributes['data']['levels'] = $this->tool->getLevels();
            $this->attributes['data']['start-image'] = $this->getStartImage();
            $this->attributes['data']['auto-rotate'] = (int)$this->tool->isAutoRotate();
        }
        return $this->attributes;
    }

    protected function getResourceList()
    {
        $resourceUrlList = [];
        foreach ($this->getImageResourceList() as $resource) {
            $resourceUrlList[] = $resource->getBestFittingSplashScreen($this->tool->getWidth(), $this->tool->getHeight())->getExternalUri();
        }
        return $resourceUrlList;
    }

    protected function getStartImage()
    {
        if ($this->tool->getLevels() == 0) {
            return 1;
        }
        $index = array_search($this->tool->getStartLevel(), array_keys($this->getImageResourceList()));
        if ($index === false) {
            $index = 0;
        }
        $index += 1;
        $perLevel = count($this->getImageResourceList()) / $this->tool->getLevels();
        return floor($index / $perLevel) * $perLevel + @($index % $perLevel);
    }

    /**
     * @throws Exception
     * @return \Rc\Domain\Model\MediaInterface[]
     */
    protected function getImageResourceList()
    {
        if (!$this->tool->hasMedia()) {
            return [];
        }
        $simpleMedia = $this->tool->getFirstResource();
        if (!$simpleMedia instanceof \Rc\Domain\Model\Research\Exposition\SimpleMedia\ArchiveSimpleMedia) {
            throw new Exception('SimpleMedia of type ' . $simpleMedia->getName() . ' is not supported by the object viewer!');
        }
        return $simpleMedia->getMediaList();
    }

}

?>