<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\AbstractDilpsModel;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\StatusAwareInterface;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\ResearchRepository;

class My_View_Helper_Profile extends Zend_View_Helper_Abstract
{

    /**
     * @var array
     */
    protected $supervisedExpositions;

    /**
     * @var array
     */
    protected $supervisedOutput;

    /**
     * @return array
     */
    public function getSupervisedOutput()
    {
        return $this->supervisedOutput;
    }

    public function profile()
    {
        return $this;
    }

    public function renderPrivateResearchList(User $user)
    {
        ProfileGroupService::assignGrouplessElementsToDefaultGroup($user);

        $output = [];
        $supervisedOutput = [];
        $entityOccurrenceList = [];
        foreach (ProfileGroupRepository::findByUserId($user->getId(), 0, 500) as $group) {
            $entities = [];

            foreach (ProfileGroupElementRepository::findByGroupId($group->getId(), 0, 5000) as $groupElement) {
                $entities[$groupElement->getEntityId()] = [
                    'entity' => null,
                    'layout' => $groupElement->getLayout(),
                ];
            }

            foreach (ResearchRepository::findByIds(array_keys($entities)) as $entity) {
                $entities[$entity->getId()]['entity'] = $entity;
            }

            $groupOutput = '';
            $supervisedGroupOutput = '';
            foreach ($entities as $entity) {
                if (!isset($entity['entity'])) {
                    continue;
                }
                $pEntry = &$entityOccurrenceList[$entity['entity']->getName()];
                if ($entity['entity']->isSupervisor($user)) {
                    $pEntry['supervised'] = true;
                    $supervisedGroupOutput .= $this->entity($entity['entity'], $entity['layout']);
                } else if ($entity['entity']->isCollaborateur($user)) {
                    $pEntry['collaborated'] = true;
                    $groupOutput .= $this->entity($entity['entity'], $entity['layout']);
                } else {
                    $pEntry['owner'] = true;
                    $groupOutput .= $this->entity($entity['entity'], $entity['layout']);
                }
            }

            $output[$group->getName()] = '
                <div data-id="' . $group->getId() . '" data-collapsed="' . (int)$group->isCollapsed() . '">
                    <h3 data-removable="' . (int)$group->isRemovable() . '" data-hidden="' . (int)$group->isHidden() . '">' . $group->getName() . '</h3>
                    <ul class="sortable" style="' . ($group->isCollapsed() ? 'display:none' : '') . '">' . $groupOutput . '</ul>
                </div>';
            if (!empty($supervisedGroupOutput)) {
                $supervisedOutput[$group->getName()] = '
                <div data-id="' . $group->getId() . '" data-collapsed="' . (int)$group->isCollapsed() . '">
                    <h3 data-removable="' . (int)$group->isRemovable() . '" data-hidden="' . (int)$group->isHidden() . '">' . $group->getName() . '</h3>
                    <ul class="sortable" style="' . ($group->isCollapsed() ? 'display:none' : '') . '">' . $supervisedGroupOutput . '</ul>
                </div>';
            }

            if (!empty($groupOutput) || !empty($supervisedGroupOutput)) {
                $hasElements[$group->getName()] = true;
            }
        }

        $this->supervisedOutput = implode(PHP_EOL, $supervisedOutput);

        if (!isset($entityOccurrenceList['research']['owner']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_EXPOSITION])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_EXPOSITION]);
        }
        if (!isset($entityOccurrenceList['work']['owner']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_WORK])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_WORK]);
        }
        if (!isset($entityOccurrenceList['project']['owner']) && !isset($entityOccurrenceList['degree']['owner']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_ACTIVITY])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_ACTIVITY]);
        }
        if (!isset($entityOccurrenceList['research']['supervised']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_EXPOSITION_SUPERVISED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_EXPOSITION_SUPERVISED]);
        } else {
            unset($output[ProfileGroupRepository::GROUP_NAME_EXPOSITION_SUPERVISED]);
        }
        if (!isset($entityOccurrenceList['research']['collaborated']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_EXPOSITION_COLLABORATED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_EXPOSITION_COLLABORATED]);
        }
        if (!isset($entityOccurrenceList['work']['supervised']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_WORK_SUPERVISED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_WORK_SUPERVISED]);
        }
        if (!isset($entityOccurrenceList['work']['collaborated']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_WORK_COLLABORATED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_WORK_COLLABORATED]);
        }
        if (!isset($entityOccurrenceList['project']['supervised']) && !isset($entityOccurrenceList['degree']['supervised']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_ACTIVITY_SUPERVISED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_ACTIVITY_SUPERVISED]);
        } else {
            unset($output[ProfileGroupRepository::GROUP_NAME_ACTIVITY_SUPERVISED]);
        }
        if (!isset($entityOccurrenceList['project']['collaborated']) && !isset($entityOccurrenceList['degree']['collaborated']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_ACTIVITY_COLLABORATED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_ACTIVITY_COLLABORATED]);
        }
        if (!isset($entityOccurrenceList['application']['owner']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_APPLICATION])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_APPLICATION]);
        }
        if (!isset($entityOccurrenceList['application']['collaborated']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_APPLICATION_COLLABORATED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_APPLICATION_COLLABORATED]);
        }
        if (!isset($entityOccurrenceList['application']['supervised']) && !isset($hasElements[ProfileGroupRepository::GROUP_NAME_APPLICATION_SUPERVISED])) {
            unset($output[ProfileGroupRepository::GROUP_NAME_APPLICATION_SUPERVISED]);
        } else {
            unset($output[ProfileGroupRepository::GROUP_NAME_APPLICATION_SUPERVISED]);
        }
        if (empty($entityOccurrenceList)) {
            return null;
        }
        return implode(PHP_EOL, $output);
    }

    public function renderPublicResearchList(User $user)
    {
        ProfileGroupService::assignGrouplessElementsToDefaultGroup($user);

        $output = [];
        foreach (ProfileGroupRepository::findByUserId($user->getId(), 0, 500) as $group) {
            if ($group->isHidden()) {
                continue;
            }

            $entities = [];
            foreach (ProfileGroupElementRepository::findByGroupId($group->getId(), 0, 500) as $groupElement) {
                $entities[$groupElement->getEntityId()] = [
                    'entity' => null,
                    'layout' => $groupElement->getLayout(),
                ];
            }

            foreach (ResearchRepository::findPublicAccessibleByIds(array_keys($entities)) as $entity) {
                if ($entity->isContributor($user) || $entity->isSupervisor($user)) {
                    continue;
                }
                $entities[$entity->getId()]['entity'] = $entity;
            }

            $groupOutput = '';
            foreach ($entities as $entity) {
                if (!isset($entity['entity'])) {
                    continue;
                }
                $groupOutput .= $this->entity($entity['entity'], $entity['layout'], false);
            }

            if (!empty($groupOutput)) {
                $output[] = '
                    <div data-id="' . $group->getId() . '">
                        <h3>' . $group->getName() . '</h3>
                        <ul class="sortable">' . $groupOutput . '</ul>
                    </div>';
            }
        }

        if (empty($output)) {
            return '';
        }

        if (count($output) == 1) {
            $output[] = '<script>$(".profile-research-list > div > h3").hide()</script>';
        }

        return implode('', $output) . $this->renderToggleHeadlineJavascript(false);
    }

    /**
     * @param Research $entity
     * @param int $layout
     * @param bool $isPrivateProfile
     * @return string
     */
    public function entity(Research $entity, $layout, $isPrivateProfile = true)
    {
        if ($entity instanceof Project) {
            return $this->project($entity, $layout, $isPrivateProfile);
        }
        if ($entity instanceof Degree) {
            return $this->degree($entity, $layout, $isPrivateProfile);
        }
        if ($entity instanceof Exposition) {
            return $this->exposition($entity, $layout, $isPrivateProfile);
        }
        if ($entity instanceof Work) {
            return $this->work($entity, $layout, $isPrivateProfile);
        }
        if ($entity instanceof Application) {
            return $this->application($entity, $layout, $isPrivateProfile);
        }
        throw new InvalidArgumentException('Unsupported object type "' . get_class($entity) . '".');
    }

    /**
     * @param Exposition $research
     * @param string $layout
     * @param bool|true $isPrivateProfile
     * @return string
     * @throws Exception
     */
    public function exposition(Exposition $research, $layout, $isPrivateProfile = true)
    {
        if ($research->isDoubleBlindReviewer()) {
            $authorList = 'author name hidden';
        } else {
            $authorList = $this->renderAuthorList($research);
        }

        $content = '<div class="content-research-description">' . $research->getDescription() . '</div>';
        if ($research->isAbstractOnly()) {
            $content .= '<br/>' . preg_replace('~((?:[\w\d]+\:\/\/)?(?:[\w\-\d]+\.)+[\w\-\d]+(?:\/[\w\-\d]+)*(?:\/|\.[\w\-\d]+)?(?:\?[\w\-\d]+\=[\w\-\d]+\&?)?(?:\#[\w\-\d]*)?)~i',
                    '<a href="$1" target="_blank">$1</a>', $research->getReference());
        }

        if (!config('app.allow.publishing')) {
            $publishAction = null;
        } else if ($research->getStatus() == Exposition::STATUS_PROGRESS) {
            $publishAction = '<li onclick="selfPublishResearchDialog(' . $research->getId() . ')"><span class="ui-icon ui-icon-signal-diag"></span>' . $this->view->translate('action.self-publish') . '</li>';
            $publishAction .= '<li onclick="publishResearchDialog(' . $research->getId() . ')"><span class="ui-icon ui-icon-signal-diag"></span>' . $this->view->translate('action.publish') . '</li>';
        } else if ($research->getStatus() == Exposition::STATUS_PUBLISHED) {
            $publishAction = '<li onclick="republishResearchDialog(' . $research->getId() . ')"><span class="ui-icon ui-icon-signal-diag"></span>' . $this->view->translate('action.republish') . '</li>';
        } else {
            $publishAction = null;
        }

        $isOwner = Identity::exists() && $research->isOwner(Identity::getUser());
        $isCoAuthor = Identity::exists() && $research->isCoAuthor(Identity::getUser());

        if ($isPrivateProfile) {
            if ($research->isStatus(Exposition::STATUS_PROGRESS)) {
                $actions = '
                    <span class="content-research-action icon-menu" title="' . $this->view->translate('profile.researches.action.open-menu') . '" onclick="openMenu($(\'#exposition-menu-' . $research->getId() . '\'), this)"></span>
                    <ul class="content-research-menu" id="exposition-menu-' . $research->getId() . '">
                        <li>' . $this->renderChangeLayout($research) . '</li>
                        ' . ($research->hasEditRights(Identity::getUserOrNull()) && !$research->isAbstractOnly() ? '<li><a href="/editor?research=' . $research->getId() . '&weave=' . $research->getDefaultWeaveId() . '" target="_blank"><span class="ui-icon ui-icon-wrench"></span>' . $this->view->translate('profile.researches.action.open-editor') . '</a></li>' : '') . '
                        ' . ($isOwner || $isCoAuthor ? '<li><a onclick="editResearchDialog(' . $research->getId() . ')"><span class="ui-icon ui-icon-pencil"></span>' . $this->view->translate('profile.researches.action.edit') . '</a></li>' : '') . '
                        ' . ($isOwner ? '<li><a onclick="shareProfileEntity(' . $research->getId() . ')"><span class="ui-icon ui-icon-person"></span>' . $this->view->translate('profile.researches.action.share') . '</a></li>' : '') . '
                        ' . ($isOwner ? $publishAction : '') . '
                        <li onclick="addShortcut(' . $research->getId() . ')"><span class="ui-icon ui-icon-star"></span>add to shortcut</li>
                        ' . ($isOwner && count(PortalRepository::findByAllowExpositionConnecting()) > 0 ? '<li onclick="connectProfileEntity(' . $research->getId() . ')"><span class="ui-icon ui-icon-link"></span>connect to portal</li>' : '') . '
                        <li onclick="versioning(' . $research->getId() . ')"><span class="ui-icon ui-icon-refresh"></span>' . $this->view->translate('action.versioning') . '</li>
                        ' . (!$isOwner ? '<li onclick="leaveCollaborationDialog(' . $research->getId() . ',\'' . $this->getRole($research) . '\')"><span class="ui-icon ui-icon-extlink"></span>' . $this->view->translate('action.collaboration.leave') . '</li>' : '') . '
                        <li><span class="ui-icon ui-icon-disk"></span><a href="/research/export?research=' . $research->getid() . '" target="_blank">export</a></li>
                        ' . ($isOwner ? '<li onclick="removeProfileEntity(' . $research->getId() . ')"><span class="ui-icon ui-icon-trash"></span>' . $this->view->translate('action.remove') . '</li>' : '') . '
                    </ul>';
            } else {
                $actions = '
                    <span class="content-research-action icon-menu" title="' . $this->view->translate('profile.researches.action.open-menu') . '" onclick="openMenu($(\'#exposition-menu-' . $research->getId() . '\'), this)"></span>
                    <ul class="content-research-menu" id="exposition-menu-' . $research->getId() . '">
                        ' . ($isOwner ? $publishAction : '') . '
                        <li>' . $this->renderChangeLayout($research) . '</li>
                        <li onclick="addShortcut(' . $research->getId() . ')"><span class="ui-icon ui-icon-star"></span>add to shortcut</li>
                        ' . ($isOwner && count(PortalRepository::findByAllowExpositionConnecting()) > 0 ? '<li onclick="connectProfileEntity(' . $research->getId() . ')"><span class="ui-icon ui-icon-link"></span>connect to portal</li>' : '') . '
                        <li><span class="ui-icon ui-icon-disk"></span><a href="/research/export?research=' . $research->getid() . '" target="_blank">export</a></li>
                        <li onclick="versioning(' . $research->getId() . ')"><span class="ui-icon ui-icon-refresh"></span>' . $this->view->translate('action.versioning') . '</li>
                    </ul>';
            }
        } else {
            $actions = '<a class="content-research-action" style="width:auto;text-decoration:none" href="' . $this->view->url()->entityUrl($research,
                    ['show-comments' => 1, 'show-reports' => 1]) . '">comments (' . count(CommentRepository::findByResearch($research, 0)) . ')</a>';
            $actions .= $this->view->feedLink($research, 'entity', ['class' => 'content-research-action-public']);
        }

        return '
            <li class="content-research ' . $this->renderAdditionalClasses($research, $layout, $isPrivateProfile) . '" data-id="' . $research->getId() . '">
                ' . $this->renderMedia($research, $isPrivateProfile) . '
                <span class="content-research-metadata">
                    ' . $this->renderTitle($research, $this->getDateTitleSuffix($research)) . '
                    <span class="content-research-subtitle">author(s): ' . $authorList . '</span>
                    ' . $this->renderConnectedToPortal($research) . '
                    ' . $this->renderPublishedInPortal($research) . '
                    ' . $content . '
                </span>
                <span class="content-research-footer">
                    <a class="button consult-research" style="margin:-2px 0;display:inline-block;text-decoration:none" href="' . $this->view->viewUrl($research) . '" target="_blank">open exposition</a>
                    ' . $actions . '
                </span>
            </li>';
    }

    public function work(Work $work, $layout, $isPrivateProfile = true)
    {
        $previewLink = $this->view->url()->workUrl($work);
        $descriptionPieces = [
            $work->getMainGenre() ? ucfirst($this->view->translate('genre.' . $work->getMainGenre())) . ': ' . ucfirst($this->view->translate('genre.' . $work->getMainGenre() . '.' . $work->getGenre())) : '',
            $work->getPublisher(),
            $work->getPlace(),
            'artist(s)/author(s): ' . $this->renderAuthorList($work),
        ];
        $isOwner = Identity::exists() && $work->isOwner(Identity::getUser());
        $isCoAuthor = Identity::exists() && $work->isCoAuthor(Identity::getUser());

        if ($isPrivateProfile) {
            $actions = '
            <span class="content-research-action icon-menu" title="' . $this->view->translate('profile.researches.action.open-menu') . '" onclick="openMenu($(\'#work-menu-' . $work->getId() . '\'), this)"></span>
            <ul class="content-research-menu" id="work-menu-' . $work->getId() . '">
                <li>' . $this->renderChangeLayout($work) . '</li>
                ' . ($isOwner || $isCoAuthor ? '<li><a onclick="editWorkDialog(' . $work->getId() . ')"><span class="ui-icon ui-icon-pencil"></span>' . $this->view->translate('profile.researches.action.edit') . '</a></li>' : '') . '
                ' . ($isOwner ? '<li><a onclick="shareProfileEntity(' . $work->getId() . ')"><span class="ui-icon ui-icon-person"></span>' . $this->view->translate('profile.researches.action.share') . '</a></li>' : '') . '
                <li><span onclick="$(this).next().click()" class="ui-icon ui-icon-search"></span><a href="' . $previewLink . '"></span>' . $this->view->translate('action.preview') . '</a></li>
                <li onclick="addShortcut(' . $work->getId() . ')"><span class="ui-icon ui-icon-star"></span>add to shortcut</li>
                ' . ($isOwner && count(PortalRepository::findByAllowWorkConnecting()) > 0 ? '<li onclick="connectProfileEntity(' . $work->getId() . ')"><span class="ui-icon ui-icon-link"></span>connect to portal</li>' : '') . '
                ' . (!$isOwner ? '<li onclick="leaveCollaborationDialog(' . $work->getId() . ',\'' . $this->getRole($work) . '\')"><span class="ui-icon ui-icon-extlink"></span>' . $this->view->translate('action.collaboration.leave') . '</li>' : '') . '
                ' . ($isOwner ? '<li onclick="removeWorkDialog(' . $work->getId() . ')"><span class="ui-icon ui-icon-trash"></span>' . $this->view->translate('action.remove') . '</li>' : '') . '
            </ul>';
        } else {
            $actions = $this->view->feedLink($work, 'entity', ['class' => 'content-research-action-public']);
        }

        return '
            <li class="content-research ' . $this->renderAdditionalClasses($work, $layout, $isPrivateProfile) . '" data-id="' . $work->getId() . '">
                ' . $this->renderMedia($work, $isPrivateProfile) . '
                ' . $this->renderTitle($work, $this->getDateTitleSuffix($work)) . '
                ' . $this->renderConnectedToPortal($work) . '
                <div class="content-research-description">' . implode(', ', array_filter($descriptionPieces)) . '<br/>' . $work->getDescription() . '</div>
                <span class="content-research-footer">
                    ' . $actions . '
                </span>
            </li>';
    }

    public function project(Project $project, $layout, $isPrivateProfile = true)
    {
        $previewLink = $this->view->url()->projectUrl($project);
        $descriptionPieces = [
            'Project',
            $this->view->translate('field.principal-investigator') . ': ' . $this->view->link()->personLink($project->getPrincipalInvestigator()),
            $project->hasCoAuthors() ? 'researcher(s): ' . $this->renderAuthorList($project, false) : null,
        ];
        $isOwner = Identity::exists() && $project->isOwner(Identity::getUser());

        if ($isPrivateProfile) {
            $actions = '
                <span title="' . $this->view->translate('profile.researches.action.open-menu') . '" class="content-research-action icon-menu" onclick="openMenu($(\'#project-menu-' . $project->getId() . '\'), this)"></span>
                <ul class="content-research-menu" id="project-menu-' . $project->getId() . '">
                    <li>' . $this->renderChangeLayout($project) . '</li>
                    ' . ($isOwner ? '<li><a onclick="editProfileEntity(' . $project->getId() . ')"><span class="ui-icon ui-icon-pencil"></span>' . $this->view->translate('profile.researches.action.edit') . '</a></li>' : '') . '
                    ' . ($isOwner ? '<li><a onclick="shareProfileEntity(' . $project->getId() . ')"><span class="ui-icon ui-icon-person"></span>' . $this->view->translate('profile.researches.action.share') . '</a></li>' : '') . '
                    <li><span onclick="$(this).next().click()" class="ui-icon ui-icon-search"></span><a href="' . $previewLink . '"></span>' . $this->view->translate('action.preview') . '</a></li>
                    <li onclick="addShortcut(' . $project->getId() . ')"><span class="ui-icon ui-icon-star"></span>add to shortcut</li>
                    ' . ($isOwner && count(PortalRepository::findByAllowProjectConnecting()) > 0 ? '<li onclick="connectProfileEntity(' . $project->getId() . ')"><span class="ui-icon ui-icon-link"></span>connect to portal</li>' : '') . '
                    ' . (!$isOwner ? '<li onclick="leaveCollaborationDialog(' . $project->getId() . ',\'' . $this->getRole($project) . '\')"><span class="ui-icon ui-icon-extlink"></span>' . $this->view->translate('action.collaboration.leave') . '</li>' : '') . '
                    ' . ($isOwner ? '<li onclick="removeProfileEntity(' . $project->getId() . ')"><span class="ui-icon ui-icon-trash"></span>' . $this->view->translate('action.remove') . '</li>' : '') . '
                </ul>';
        } else {
            $actions = $this->view->feedLink($project, 'entity', ['class' => 'content-research-action-public']);
        }

        return '
            <li class="content-research ' . $this->renderAdditionalClasses($project, $layout, $isPrivateProfile) . '" data-id="' . $project->getId() . '">
                ' . $this->renderMedia($project, $isPrivateProfile) . '
                ' . $this->renderTitle($project, $this->getDateTitleSuffix($project)) . '
                ' . $this->renderConnectedToPortal($project) . '
                <div class="content-research-description">' . implode(', ', array_filter($descriptionPieces)) . '<br/>' . $project->getDescription() . '</div>
                <span class="content-research-footer">
                    ' . $actions . '
                </span>
            </li>';
    }

    public function degree(Degree $degree, $layout, $isPrivateProfile = true)
    {
        $previewLink = $this->view->url()->degreeUrl($degree);
        $descriptionPieces = [
            'Degree: ' . $this->view->translate('degree.level.' . $degree->getLevel()),
            $degree->hasStatus(Degree::STATUS_PUBLISHED) ? 'conferred' : '',
            implode(', ', $degree->getTranslatedAwardingInstitutions()),
            'recipient: ' . $this->view->link()->personLink($degree->getAuthor()),
        ];
        $isOwner = Identity::exists() && $degree->isOwner(Identity::getUser());

        if (!config('app.allow.publishing')) {
            $publishAction = null;
        } else if ($degree->hasStatus(Degree::STATUS_PROGRESS)) {
            $publishAction = '<li onclick="publishDegreeDialog(' . $degree->getId() . ')"><span class="ui-icon ui-icon-signal-diag"></span>' . $this->view->translate('action.publish') . '</li>';
        } else {
            $publishAction = null;
        }

        if ($isPrivateProfile) {
            $actions = '
                <span title="' . $this->view->translate('profile.researches.action.open-menu') . '" class="content-research-action icon-menu" onclick="openMenu($(\'#degree-menu-' . $degree->getId() . '\'), this)"></span>
                <ul class="content-research-menu" id="degree-menu-' . $degree->getId() . '">
                     <li>' . $this->renderChangeLayout($degree) . '</li>
                    ' . ($isOwner ? '<li><a onclick="editProfileEntity(' . $degree->getId() . ')"><span class="ui-icon ui-icon-pencil"></span>' . $this->view->translate('profile.researches.action.edit') . '</a></li>' : '') . '
                    ' . ($isOwner ? '<li><a onclick="shareProfileEntity(' . $degree->getId() . ')"><span class="ui-icon ui-icon-person"></span>' . $this->view->translate('profile.researches.action.share') . '</a></li>' : '') . '
                    ' . ($isOwner ? $publishAction : '') . '
                    <li><span onclick="$(this).next().click()" class="ui-icon ui-icon-search"></span><a href="' . $previewLink . '"></span>' . $this->view->translate('action.preview') . '</a></li>
                    <li onclick="addShortcut(' . $degree->getId() . ')"><span class="ui-icon ui-icon-star"></span>add to shortcut</li>
                    ' . ($isOwner && count(PortalRepository::findByAllowDegreeConnecting()) > 0 ? '<li onclick="connectProfileEntity(' . $degree->getId() . ')"><span class="ui-icon ui-icon-link"></span>connect to portal</li>' : '') . '
                    ' . (!$isOwner ? '<li onclick="leaveCollaborationDialog(' . $degree->getId() . ',\'' . $this->getRole($degree) . '\')"><span class="ui-icon ui-icon-extlink"></span>' . $this->view->translate('action.collaboration.leave') . '</li>' : '') . '
                    ' . ($isOwner ? '<li onclick="removeProfileEntity(' . $degree->getId() . ')"><span class="ui-icon ui-icon-trash"></span>' . $this->view->translate('action.remove') . '</li>' : '') . '
                </ul>';
        } else {
            $actions = $this->view->feedLink($degree, 'entity', ['class' => 'content-research-action-public']);
        }

        return '
            <li class="content-research ' . $this->renderAdditionalClasses($degree, $layout, $isPrivateProfile) . '" data-id="' . $degree->getId() . '">
                ' . $this->renderMedia($degree, $isPrivateProfile) . '
                ' . $this->renderTitle($degree, $this->getDateTitleSuffix($degree)) . '
                ' . $this->renderConnectedToPortal($degree) . '
                <div class="content-research-description">' . implode(', ', array_filter($descriptionPieces)) . '<br/>' . $degree->getDescription() . '</div>
                <span class="content-research-footer">
                    ' . $actions . '
                </span>
            </li>';
    }

    /**
     * @param Application $application
     * @param string $layout
     * @param bool|true $isPrivateProfile
     * @return string
     * @throws Exception
     */
    public function application(Application $application, $layout, $isPrivateProfile = true)
    {
        $content = '<span class="content-research-subtitle">This application is ' . $this->view->translate('application.status.' . $application->getStatus()) . '.</span>';
        $content .= '<span class="content-research-subtitle">Application Program: ' . $application->getProgram()->getTitle() . '</span>';

        $isOwner = Identity::exists() && $application->isOwner(Identity::getUser());
        $isCoAuthor = Identity::exists() && $application->isCoAuthor(Identity::getUser());

        if ($isPrivateProfile) {
            if ($application->isInProgress()) {
                $readyForSubmit = false;
                if ($isOwner) {
                    $form = ApplicationFormGenerator::getEdit($application)->setDefaultsFrom($application);
                    $readyForSubmit = (new My_Validate_Application)->isValid($application, $form);
                }

                $actions = '
                    <span class="content-research-action icon-menu" title="' . $this->view->translate('profile.researches.action.open-menu') . '" onclick="openMenu($(\'#application-menu-' . $application->getId() . '\'), this)"></span>
                    <ul class="content-research-menu" id="application-menu-' . $application->getId() . '">
                        <li>' . $this->renderChangeLayout($application) . '</li>
                        ' . ($isOwner || $isCoAuthor ? '<li><a onclick="editApplicationDialog(' . $application->getId() . ')"><span class="ui-icon ui-icon-pencil"></span>' . $this->view->translate('profile.researches.action.edit') . '</a></li>' : '') . '
                        ' . ($readyForSubmit ? '<li onclick="publishApplicationDialog(' . $application->getId() . ')"><span class="ui-icon ui-icon-signal-diag"></span>' . $this->view->translate('action.submit.application') . '</li>' : '') . '
                        <li onclick="addShortcut(' . $application->getId() . ')"><span class="ui-icon ui-icon-star"></span>add to shortcut</li>
                        ' . (!$isOwner ? '<li onclick="leaveCollaborationDialog(' . $application->getId() . ',\'' . $this->getRole($application) . '\')"><span class="ui-icon ui-icon-extlink"></span>' . $this->view->translate('action.collaboration.leave') . '</li>' : '') . '
                        ' . ($isOwner ? '<li onclick="removeProfileEntity(' . $application->getId() . ')"><span class="ui-icon ui-icon-trash"></span>' . $this->view->translate('action.remove') . '</li>' : '') . '
                    </ul>';
            } else {
                $actions = '
                    <span class="content-research-action icon-menu" title="' . $this->view->translate('profile.researches.action.open-menu') . '" onclick="openMenu($(\'#exposition-menu-' . $application->getId() . '\'), this)"></span>
                    <ul class="content-research-menu" id="exposition-menu-' . $application->getId() . '">
                        <li>' . $this->renderChangeLayout($application) . '</li>
                        <li onclick="addShortcut(' . $application->getId() . ')"><span class="ui-icon ui-icon-star"></span>add to shortcut</li>
                        ' . ($isOwner && count(PortalRepository::findByAllowExpositionConnecting()) > 0 ? '<li onclick="connectProfileEntity(' . $application->getId() . ')"><span class="ui-icon ui-icon-link"></span>connect to portal</li>' : '') . '
                    </ul>';
            }
        } else {
            $actions = '';
        }

        return '
            <li class="content-research ' . $this->renderAdditionalClasses($application, $layout, $isPrivateProfile) . '" data-id="' . $application->getId() . '">
                <span class="content-research-metadata">
                ' . $this->renderTitle($application, $this->getDateTitleSuffix($application)) . '
                    <span class="content-research-subtitle">author(s): ' . $this->renderAuthorList($application) . '</span>
                    ' . $content . '
                </span>
                <span class="content-research-footer">
                    ' . $actions . '
                </span>
            </li>';
    }

    public function getDateTitleSuffix(Research $entity)
    {
        if ($entity instanceof Work) {
            return $entity->getDate();
        }
        if ($entity instanceof Project) {
            return substr($entity->getStartDate(), -4) . ($entity->getEndDate() ? ' - ' . substr($entity->getEndDate(), -4) : '');
        }
        if ($entity instanceof Exposition) {
            return $entity->isPublished() ? substr($entity->getPublishedDate(), -4) : 'last edited: ' . date('Y', $entity->getLastModified());
        }
        if ($entity instanceof Degree) {
            return implode('-', array_filter([
                $this->getFormattedDate($entity->getStartDate()),
                $this->getFormattedDate($entity->getConferredDate()),
            ]));
        }
        return null;
    }

    public function renderToggleHeadlineJavascript($isEditable)
    {
        $isEditable = $isEditable ? 'true' : 'false';
        return <<<EOF
            <script>
            $('.profile-research-list h3').each(function () {
                var metaNode = $(this).closest('div');
                $(this).prepend('<span class="ui-icon ui-icon-triangle-1-s"></span>')
                    .bind('click', function () {
                        var icon = $('span', this),
                            content = $(this).next('ul');
                        if (icon.hasClass('ui-icon-triangle-1-s')) {
                            icon.removeClass('ui-icon-triangle-1-s');
                            icon.addClass('ui-icon-triangle-1-e');
                            content.css('height', 0).hide();
                        } else {
                            icon.removeClass('ui-icon-triangle-1-e');
                            icon.addClass('ui-icon-triangle-1-s');
                            content.css('height', '').show();
                        }
                        if ($isEditable) {
                            $.post('/profile/collapse-group?profilegroup='+metaNode.attr('data-id')+'&state='+(content.is(':visible') ? 0 : 1));
                        }

                        lazyLoadMedias();
                    });
                if (metaNode.attr('data-collapsed') == 1) {
                    $(this).trigger('click');
                }
            });
        </script>
EOF;
    }

    public function renderSortableJavascript()
    {
        return <<<EOF
        <script>
            lazyLoadMedias();

            var sortingTarget = null;
            $('.profile-research-list h3').each(function () {
                var headline = $(this);
                if (headline.attr('data-removable') == 1) {
                    headline.prepend($('<span class="profile-delete-group ui-icon ui-icon-pencil" title="rename"></span>').bind('click', function () {
                        new DialogForm({
                            url: '/profile/edit-group?profilegroup=' + $(this).closest('div[data-id]').attr('data-id'),
                            reloadOnSuccess: true
                        });
                        return false;
                    }))
                    headline.prepend($('<span class="profile-delete-group ui-icon ui-icon-close" title="delete"></span>').bind('click', function () {
                        new DialogForm({
                            url: '/profile/remove-group?profilegroup=' + $(this).closest('div[data-id]').attr('data-id'),
                            reloadOnSuccess: true
                        });
                        return false;
                    }))
                }
                headline.prepend($('<span class="profile-delete-group ui-icon ui-icon-' + (headline.attr('data-hidden') == 1 ? 'radio-on' : 'radio-off') + '" title="' + (headline.attr('data-hidden') == 1 ? 'show on public profile' : 'hide on public profile') + '"></span>').bind('click', function () {
                    if (headline.attr('data-hidden') == 1) {
                        headline.attr('data-hidden', 0);
                        $(this).attr('title', 'hide on public profile').removeClass('ui-icon-radio-on').addClass('ui-icon-radio-off');
                    } else {
                        headline.attr('data-hidden', 1);
                        $(this).attr('title', 'show on public profile').removeClass('ui-icon-radio-off').addClass('ui-icon-radio-on');
                    }
                    $.post('/profile/hide-group?profilegroup=' + $(this).closest('div[data-id]').attr('data-id') + '&hidden=' + headline.attr('data-hidden'));
                    return false;
                }))
                headline.droppable({
                    accept: '.sortable li',
                    hoverClass: 'ui-state-hover',
                    tolerance: 'pointer',
                    drop: function(event, ui) {
                        sortingTarget = headline.next('ul');
                        headline.css('background-color', '');
                    },
                    over: function() {
                        headline.css('background-color', 'rgba(10,100,100,0.10)');
                    },
                    out: function() {
                        headline.css('background-color', '');
                    }
                });
            });
            $('.sortable').sortable({
                delay: 150,
                tolerance: 'pointer',
                appendTo: '#container-content',
                containment: '#container-content',
                helper: 'clone',
                connectWith: '.sortable',
                start: function(event, ui) {
                    $(this).css('min-height', $(this).height());
                },
                stop: function (event, ui) {
                    $(this).css('min-height', '');
                    if (sortingTarget && sortingTarget[0] !== this) {
                        sortingTarget.append(ui.item);
                        sortingTarget.sortable('option', 'receive').call(sortingTarget[0], event, ui);
                        sortingTarget = null;
                    }
                    $.post('/profile/update-group-element-order', {
                        profilegroup: $(this).closest('div[data-id]').attr('data-id'),
                        order: $.map($(this).children('li'), function (element) {
                            return $(element).attr('data-id');
                        })
                    });
                },
                receive: function (event, ui) {
                    if (sortingTarget && sortingTarget[0] !== this) {
                        sortingTarget.append(ui.item);
                        sortingTarget.sortable('option', 'receive').call(sortingTarget[0], event, ui);
                        sortingTarget = null;
                        return;
                    }
                    $.post('/profile/update-group-element-order', {
                        profilegroup: $(this).closest('div[data-id]').attr('data-id'),
                        order: $.map($(this).children('li'), function (element) {
                            return $(element).attr('data-id');
                        }),
                        entity: ui.item.attr('data-id')
                    });
                }
            }).disableSelection();
            $('.profile-research-list:last').sortable({
                containment: '.profile-research:last',
                axis: 'y',
                handle: 'h3',
                delay: 150,
                helper: 'clone',
                stop: function (event, ui) {
                    $.post('/profile/update-group-order', {
                        order: $.map($(this).children('div'), function (element) {
                            return $(element).attr('data-id');
                        })
                    });
                }
            }).disableSelection();
        </script>
EOF;
    }

    /**
     * @param Research|null $entity
     * @return null
     */
    public function getThumb(Research $entity = null)
    {
        if (!isset($entity)) {
            return null;
        }
        return $this->view->getHelper('sizeMedia')->auto($entity->getMediaContainer(), 100, 83, ['class' => 'research-image']);
    }

    /**
     * @param Research $entity
     * @param bool|true $prependAuthor
     * @return string
     */
    public function renderAuthorList(Research $entity, $prependAuthor = true)
    {
        $authorList = [];
        if ($prependAuthor) {
            if ($entity instanceof Work && $entity->hasFreeTextAuthor()) {
                $authorList[] = $entity->getFreeTextAuthor();
            } elseif ($entity->getAuthor()) {
                $authorList[] = $this->view->link()->personLink($entity->getAuthor());
            }
        }
        foreach ($entity->getCoAuthors() as $author) {
            $authorList[] = $this->view->profileLink(\Rc\Domain\Repository\UserRepository::findById($author));
        }
        return implode(', ', $authorList);
    }

    protected function getAuthorLinks(array $authorIds)
    {
        $links = [];
        foreach (\Rc\Domain\Repository\UserRepository::findByIds($authorIds) as $author) {
            $links[] = $this->view->link()->personLink($author);
        }
        return $links;
    }

    protected function renderAdditionalClasses(Research $entity, $layout, $isPrivateProfile)
    {
        if (!isset($layout)) {
            $layout = $entity instanceof Exposition ? ProfileGroupElement::LAYOUT_2x2 : ProfileGroupElement::LAYOUT_1x1;
        }
        $classes = ' content-research-' . $layout;
        if (!$isPrivateProfile) {
            return $classes;
        }
        if ($entity->getAccessLevel() == AbstractDilpsModel::ACCESS_LEVEL_PUBLIC) {
            return $classes;
        }
        if ($entity instanceof StatusAwareInterface && $entity->isPublished() && !$entity instanceof Application) {
            return $classes;
        }
        return $classes . ' content-private';
    }

    /**
     * @param Research $entity
     * @return string
     */
    protected function renderConnectedToPortal(Research $entity)
    {
        if (!$entity->isPartOfPortal()) {
            return '';
        }
        if ($entity instanceof Exposition) {
            foreach ($entity->getPublishedInPortals() as $portal) {
                if ($portal->getId() == $entity->getPartOfPortalId()) {
                    return '';
                }
            }
        }
        return '<span class="content-research-subtitle">' . $this->view->translate('connected-to-portal') . ': <a href="' . $this->view->url()->portalUrl($entity->getPartOfPortal()) . '">' . $entity->getPartOfPortal()->getName() . '</a></span>';
    }

    protected function renderPublishedInPortal(Research $entity)
    {
        if (!$entity instanceof Exposition && !$entity instanceof Degree) {
            return '';
        }
        if (!$entity->isPublished()) {
            return '<span class="content-research-subtitle">This exposition is ' . $this->view->translate('research.status.' . $entity->getStatus()) . '.</span>';
        }

        if ($entity->getPublishRange() == 'limited') {
            $label = $this->view->translate('published-with-portal-range');
        } else {
            $label = $this->view->translate('published-in-portal');
        }

        $publishedIn = [];
        foreach ($entity->getPublishedInPortals() as $portal) {
            $publishedIn[] = '<a href="' . $this->view->url()->portalUrl($portal) . '">' . $portal->getName() . '</a>';
        }
        return '<span class="content-research-subtitle">' . $label . ': ' . implode(', ', $publishedIn) . '</span>';
    }

    protected function renderMedia(Research $entity, $lazy = false)
    {
        if ($lazy) {
            $content = '<div class="container-loading"></div>';
        } else {
            $content = $this->getThumb($entity);
        }

        if (empty($content)) {
            return null;
        }

        if ($entity instanceof Exposition) {
            $url = $this->view->viewUrl($entity);
            $title = 'Open Exposition';
        } else {
            $title = '';
            $url = $this->view->url()->entityUrl($entity);
        }

        return '<div class="research-image-container" data-id="' . $entity->getId() . '" title="' . $title . '">
                    <a href="' . $url . '">' . $content . '</a>
                </div>';
    }

    protected function renderTitle(Research $entity, $suffix = null)
    {
        return '
            <span class="content-research-title">
                <a href="' . $this->view->url()->entityUrl($entity) . '" title="Open Details Page">' . $entity->getTitle() . '</a>
                ' . (!empty($suffix) ? '(' . $suffix . ')' : '') . '
            </span>';
    }

    protected function getFormattedDate($date)
    {
        if (empty($date)) {
            return null;
        }
        return substr($date, -4);
    }

    /**
     * @param Research $entity
     * @return null|string
     */
    protected function getRole(Research $entity)
    {
        if (!Identity::exists()) {
            return null;
        }

        $user = Identity::getUser();
        if ($entity->isSupervisor($user)) {
            return 'supervisor';
        }
        if (method_exists($entity, 'isEditor') && $entity->isEditor($user)) {
            return 'editor';
        }
        if (!$entity->isOwner($user)) {
            return 'collaborateur';
        }
        return null;
    }

    protected function renderChangeLayout(Research $entity)
    {
        return '<a onclick="changeLayout(' . $entity->getId() . ')"><span class="ui-icon ui-icon-image"></span>' . $this->view->translate('profile.researches.action.change-layout') . '</a>';
    }
}
