function CommentList(init) {
	var $this = this;

	this.CONTAINER = $('#container-comment-view');
	this.CONTAINER_SCROLL = $('.comment-list', this.CONTAINER)
	this.CONTAINER_CONTENT = $('.comment-list', this.CONTAINER);
	this.CONTAINER_LOADING = $('.container-loading', this.CONTAINER);
	this.SELECTOR_CHILD = 'li.list-entry';
	this.CLASS_LOADING = 'container-loading';
	this.COMMENT_OWNER_FILTER = $('#comment-owner-filter');
	this.COMMENT_ASSIGNEE_FILTER = $('#comment-assignee-filter');
	this.COMMENT_RESOLUTION_FILTER = $('#comment-resolution-filter');

	this.load = function() {
		var data = $('<ul/>');

		Editor.ContentList
			.children()
			.filter('[data-tool="comment"]')
			.sort(function(a, b) {
				return parseInt($(a).css('z-index')) < parseInt($(b).css('z-index')) ? 1 : -1;
			})
			.each(function(i, item) {
				$this._createChild($(item)).appendTo(data);
			});

		List.prototype._eLoad.call(this, data.html());

		this.COMMENT_OWNER_FILTER.empty();
		this.createSelectFields();
	};

	this.eAfterLoad = function() {
		this.children().each(function() {
			$this.initChild($(this));
		});
	}

	this.loadChild = function(id) {
		if (this.child(id).length == 0) {
			this.CONTAINER_CONTENT.prepend('<li class="file loading" data-id="' + id + '"><em>Loading</em></li>');
		}
		List.prototype._eLoadChild.call(this, this._createChild(Editor.ContentList.child(id)));
	};

	this.eAfterLoadChild = function(data) {
		this.initChild(data);
	};

	this.initChild = function(item) {
		var textarea = $('textarea', item),
			previousTextContent = textarea.val();
			textarea.bind('focus', function() {
                Editor.enableHintOnTabClose();
            }).bind('blur', function() {
                Editor.disableHintOnTabClose();

                if (textarea.val() == previousTextContent) {
                    return;
                }
                textarea.attr('disabled', 'disabled');
                textarea.addClass($this.CLASS_LOADING);
                $.post('/item/update-text-content', {
                    item: item.data('id'),
                    research: Editor.getResearchId(),
                    textcontent: textarea.val()
                }, function() {
                    previousTextContent = textarea.val();
                    textarea.removeClass($this.CLASS_LOADING);
                    textarea.removeAttr('disabled');
                });
            });
		$('.button-up', item).click(function() {
			var prev = item.prev();
			item.insertBefore(prev);
			Editor.LayerList.bringInFrontOf(item.data('id'), prev.data('id'));
		});
		$('.button-down', item).click(function() {
			var next = item.next();
			item.insertAfter(next);
			Editor.LayerList.bringInBackOf(item.data('id'), next.data('id'));
		});

		var assignee = $('.assignee-select', item).bind('change', updateComment);
		var resolution = $('.resolution-select', item).bind('change', updateComment);
		var deadline = $('.comment-change-date', item).bind('change', updateComment).datepicker();
		assignee.val(item.data('assignee-id'));
		resolution.val(item.data('resolution'));
		deadline.val(item.data('deadline'));
		function updateComment() {
			$.post('/item/update-tracking', {
				item: item.data('id'),
				research: Editor.getResearchId(),
				assignee: assignee.val(),
				resolution: resolution.val(),
				deadline: deadline.val()
			}, function() {
				item.data('assignee-id', assignee.val());
				item.data('assignee-name', $('option:selected', assignee).text());
				item.data('resolution', resolution.val());
				item.data('deadline', deadline.val());
				$this.createSelectFields();
			})
		}

		var text = $('textarea', Editor.ContentList.child(item.data('id')));
		$('textarea', item).val(text.val());
	};

	this.init = function() {
		this.COMMENT_OWNER_FILTER.change(function() {
			updateCommentVisibility();
			Editor.ContentList.COMMENT_LAYER_SELECT.val($(this).val())
		});
		this.COMMENT_ASSIGNEE_FILTER.change(updateCommentVisibility);
		this.COMMENT_RESOLUTION_FILTER.change(updateCommentVisibility);

		function updateCommentVisibility() {
			var ownerValue = $this.COMMENT_OWNER_FILTER.val(),
				assigneeValue = $this.COMMENT_ASSIGNEE_FILTER.val(),
				resolutionValue = $this.COMMENT_RESOLUTION_FILTER.val();

			$this.children().each(function() {
				var el = $(this);
				el.toggle(
					(ownerValue == '' || ownerValue == el.data('creator-id')) &&
					(assigneeValue == '' || assigneeValue == el.data('assignee-id')) &&
					(resolutionValue == '' || resolutionValue == el.data('resolution'))
				);
			});
		}
	};

	this._createChild = function(contentItem) {
		var text = $('.comment-content', contentItem),
			lastEdit = '-';
		if (contentItem.data('text-author')) {
			lastEdit = contentItem.data('text-author') + ' ' + contentItem.data('text-modified')
		}

		var aside = $('<div class="aside"></div>')
			.append('<span class="header">created by</span>')
			.append(contentItem.data('creator-name'))
			.append('<span class="header">responsible</span>')
			.append(this.createAssigneeSelect())
			.append('<span class="header">due</span>')
			.append('<input class="comment-change-date"/>')
			.append('<span class="header">status</span>')
			.append(this.createResolutionSelect());

		return $('<li class="list-entry">')
			.html(
				$('<div/>')
					.addClass('content-header')
					.text(contentItem.data('title') + ' - last edit: ' + lastEdit)
					.append($('<button/>').addClass('button button-up').button())
					.append($('<button/>').addClass('button button-down').button())
			)
			.attr('data-id', contentItem.data('id'))
			.attr('data-creator-id', contentItem.data('creator-id') || '')
			.attr('data-creator-name', contentItem.data('creator-name') || '')
			.attr('data-assignee-id', contentItem.data('assignee-id') || '')
			.attr('data-assignee-name', contentItem.data('assignee-name') || '')
			.attr('data-resolution', contentItem.data('resolution') || '')
			.attr('data-deadline', contentItem.data('deadline') || '')
			.attr('data-tool', contentItem.data('tool'))
			.append(aside)
			.append(text.html());
	};

	this.toggle = function() {
		if (this.CONTAINER.is(':hidden')) {
			this.load();
		} else {
			Editor.ContentList.CONTAINER_LOADING.show();
			Editor.loadItems();
		}
		this.CONTAINER.toggle();
	}

	this.createAssigneeSelect = function() {
		var select = $('<select class="assignee-select"></select>'),
			collaborators = Editor.getCollaborators();
		select.append('<option value="-1">none</option>');
		for (var i  in collaborators) {
			select.append('<option value="' + i + '">' + collaborators[i] + '</option>');
		}
		return select;
	}

	this.createResolutionSelect = function() {
		return $('<select class="resolution-select"><option value="open">open</option><option value="resolved">resolved</option></select>');
	}

	this.createSelectFields = function() {
		var options = {
				owner: {},
				assignee: {},
				resolution: {}
			},
			selectedValue = {
				owner: this.COMMENT_OWNER_FILTER.val() || Editor.ContentList.COMMENT_LAYER_SELECT.val(),
				assignee: this.COMMENT_ASSIGNEE_FILTER.val(),
				resolution: this.COMMENT_RESOLUTION_FILTER.val()
			};

		this.children().each(function() {
			var el = $(this),
				ownerId = el.data('creator-id'),
				assigneeId = el.data('assignee-id'),
				resolution = el.data('resolution');

			if (options.owner[ownerId]) {
				options.owner[ownerId].count++;
			} else {
				options.owner[ownerId] = {
					count: 1,
					name: el.data('creator-name')
				};
			}

			if (options.assignee[assigneeId]) {
				options.assignee[assigneeId].count++;
			} else {
				options.assignee[assigneeId] = {
					count: 1,
					name: el.data('assignee-name')
				};
			}

			if (options.resolution[resolution]) {
				options.resolution[resolution].count++;
			} else {
				options.resolution[resolution] = {
					count: 1,
					name: resolution
				};
			}
		});

		this.COMMENT_OWNER_FILTER.empty().append('<option value="">all</option>');
		for (var ownerId in options.owner) {
			this.COMMENT_OWNER_FILTER.append(
				'<option value="' + ownerId + '">' + options.owner[ownerId].name + '</option>'
			);
		}
		this.COMMENT_OWNER_FILTER.val(selectedValue.owner);

		this.COMMENT_ASSIGNEE_FILTER.empty().append('<option value="">all</option>');
		for (var assigneeId in options.assignee) {
			this.COMMENT_ASSIGNEE_FILTER.append(
				'<option value="' + assigneeId + '">' + options.assignee[assigneeId].name + '</option>'
			);
		}
		this.COMMENT_ASSIGNEE_FILTER.val(selectedValue.assignee);

		this.COMMENT_RESOLUTION_FILTER.empty().append('<option value="">all</option>');
		for (var resolution in options.resolution) {
			this.COMMENT_RESOLUTION_FILTER.append(
				'<option value="' + resolution + '">' + options.resolution[resolution].name + '</option>'
			);
		}
		this.COMMENT_RESOLUTION_FILTER.val(selectedValue.resolution);
	}

	if (!CommentList.prototype.init) {
		CommentList.prototype = new List(init);
		CommentList.prototype.constructor = CommentList;
	}

	if (init) this.init();
}
new CommentList();