function ContentList(init) {
	var $this = this;

	this.URL = '/editor/content?weave=' + Editor.getWeaveId() + '&research=' + Editor.getResearchId();
	this.URL_CHILD = '/item/list?item=';
	this.CONTAINER_CONTENT = $('#content');
	this.CONTAINER_SCROLL = $('#container-content');
	this.CONTAINER_LOADING = $('.container-loading', this.CONTAINER_SCROLL);
	this.CLASS_LOADING = 'container-loading';
	this.SELECTOR_CHILD = '.tool';
	this.COMMENT_LAYER_SELECT = $('#toolbox-layout-comments');
	this.TEMPLATE_HANDLES_PLACEHOLDER = '<div class="outline"></div>';

	this.eAfterLoad = function() {
		this.children().each(function() {
			$this.initChild(this);
		});
		this.CONTAINER_SCROLL.css('overflow', 'auto');
        Editor.LayerList.draw();
        Editor.Overview.load();
		Editor.TextList.loadIfNecessary();
		$this.updateCommentLayerSelect();
		$this.adjustContainerSize();
		Editor.updateCommentLayer(false);
	};

	this.loadChildAnimation = function(id, show) {
		if (show) {
			var child = this.child(id);
			$('<div class="tool"></div>')
				.attr('data-for', id)
				.addClass(this.CLASS_LOADING)
				.css({
					left: child.css('left'),
					top: child.css('top'),
					width: child.outerWidth(),
					height: child.outerHeight()
				}).appendTo(this.CONTAINER_CONTENT);
		} else {
			$('[data-for=' + id + ']', this.CONTAINER_CONTENT).remove();
		}
	};

	this.eAfterLoadChild = function(data) {
		var id = $(data).data('id');
		this.loadChildAnimation(id, false);
		this.initChild(this.child(id));
		Editor.LayerList.loadChild(id);
	};

	this.eAfterSelectChild = function(id, options) {
		var item = this.child(id);
		if (options.context == this || item.length == 0) return;

		var _center = function(con, item, dir) {
			var pos = {
				vertical: 'top',
				horizontal: 'left'
			};
			var dim = {
				vertical: 'innerHeight',
				horizontal: 'innerWidth'
			};
			var itemPos = parseInt($(item).css(pos[dir]));
			var itemDim = $(item)[dim[dir]]() / 2;
			var conDim = $(con)[dim[dir]]() / 2;

			// adjust position, so that the nw corner is always visible
			return Math.min(itemPos - conDim + itemDim, itemPos - 25);
		};

		jQueryNew($this.CONTAINER_SCROLL).stop().scrollTo({
			left: _center($this.CONTAINER_SCROLL, item, 'horizontal'),
			top: _center($this.CONTAINER_SCROLL, item, 'vertical')
		}, this.SCROLL_DURATION);

		if (item.data('tool') == 'comment') {
			$('textarea', item).focus();
		}
	};

	this.eLockChildren = function(ids, options) {
		if (options.lock) {
			options.children.draggable('disable').resizable('disable');
		} else {
			options.children.draggable('enable').resizable('enable');
		}
	};

	this.initChild = function(item) {
		item = $(item);

		var minHeight = 0;
		var minWidth = 0;
		var snapToGrid = $('#sidebar-commands-snaptogrid');
		var gridSize = $('#sidebar-commands-grid');

		item
			.draggable({
				scrollSpeed: 50,
				distance: 5,
				scroll: true,
				start: function(e, ui) {
					var bBox = getBoundingBox(item);
					item.css({ // fix position when dragging rotated element
						marginLeft: parseInt(item.css('left')) - bBox.left,
						marginTop: parseInt(item.css('top')) - bBox.top
					});

					if (!$this.isSelected(item.data('id'))) {
						item.trigger({
							type: 'click',
							ctrlKey: item.draggable('option', 'ctrlKey')
						});
					}
					item.draggable('option', 'alsoDrag', $this.selectedChildren().not('.' + $this.CLASS_LOCKED).not(item));

					minHeight = 0;
					minWidth = 0;
					var notSelectedChilds = $this.notSelectedChilds();
					for (var i = 0; i < notSelectedChilds.length; i++) {
						var notSelectedChild = $(notSelectedChilds[i]);
						minWidth = Math.max(notSelectedChild.position().left + notSelectedChild.outerWidth(), minWidth);
						minHeight = Math.max(notSelectedChild.position().top + notSelectedChild.outerHeight(), minHeight);
					}
					minWidth += $this.CONTAINER_SCROLL.scrollLeft();
					minHeight += $this.CONTAINER_SCROLL.scrollTop();
				},
				drag: function(event, ui) {
					var grid = 1;
					if (snapToGrid.is(':checked')) {
						grid = Math.max(parseInt(gridSize.val()) / 10, 1);
						ui.position.left = Math.ceil(ui.position.left / grid) * grid;
						ui.position.top = Math.ceil(ui.position.top / grid) * grid;
					}

					var orgPos = ui.originalPosition;
					var currentWidth = ui.position.left + $(ui.helper).outerWidth();
					var currentHeight = ui.position.top + $(ui.helper).outerHeight();
					item.draggable('option', 'alsoDrag').each(function() {
						$(this).css({
							marginLeft: ui.position.left - orgPos.left,
							marginTop: ui.position.top - orgPos.top
						});
					});

                    if ($this.CONTAINER_CONTENT.hasClass('weave')) {
                        $this.CONTAINER_CONTENT.css({
                            width: Math.max(minWidth, currentWidth) + parseInt($this.CONTAINER_CONTENT.attr('data-padding-right')),
                            height: Math.max(minHeight, currentHeight) + parseInt($this.CONTAINER_CONTENT.attr('data-padding-bottom'))
                        });

                        $this.adjustContainerSize();
                    }
				},
				stop: function(event, ui) {
					item.css({ // fix position when dragging rotated element
						left: parseInt(item.css('left')) + parseInt(item.css('marginLeft')),
						top: parseInt(item.css('top')) + parseInt(item.css('marginTop')),
						marginLeft: 0,
						marginTop: 0
					});

					item.draggable('option', 'alsoDrag').each(function() {
						if (item.data('revert') == true) {
							$(this).animate({
								'margin-left': 0,
								'margin-top': 0
							});
						} else {
							$(this).css({
								margin: '',
								left: parseInt($(this).css('left')) + parseInt($(this).css('margin-left')),
								top: parseInt($(this).css('top')) + parseInt($(this).css('margin-top'))
							});
						}
					});
					if (item.data('revert') == true) {
						item.animate({
							left: ui.originalPosition.left,
							top: ui.originalPosition.top
						});
						return;
					}
					$this._updateChildren(item.draggable('option', 'alsoDrag').andSelf());
				}
			})
			.resizable({
				handles: 'all',
				start: function(event, ui) {
					if (!$this.isSelected(item.data('id'))) {
						item.trigger({
							type: 'click',
							ctrlKey: item.draggable('option', 'ctrlKey')
						});
					}

					var bBox = getBoundingBox(item);
					item.css({ // fix position when dragging rotated element
						marginLeft: $this.CONTAINER_SCROLL.scrollLeft() + parseInt(item.css('left')) - bBox.left,
						marginTop: $this.CONTAINER_SCROLL.scrollTop() + parseInt(item.css('top')) - bBox.top
					});

					minHeight = 0;
					minWidth = 0;
					var notSelectedChilds = $this.notSelectedChilds();
					for (var i = 0; i < notSelectedChilds.length; i++) {
						var notSelectedChild = $(notSelectedChilds[i]);
						minWidth = Math.max(notSelectedChild.position().left + notSelectedChild.outerWidth(), minWidth);
						minHeight = Math.max(notSelectedChild.position().top + notSelectedChild.outerHeight(), minHeight);
					}
					minWidth += $this.CONTAINER_SCROLL.scrollLeft();
					minHeight += $this.CONTAINER_SCROLL.scrollTop();
				},
				resize: function(event, ui) {
					var grid = 1;
					if (snapToGrid.is(':checked')) {
						grid = Math.max(parseInt(gridSize.val()) / 10, 1);
						if (ui.originalPosition.left !== ui.position.left) {
							ui.position.left = Math.floor(ui.position.left / grid) * grid;
						}
						if (ui.originalPosition.top !== ui.position.top) {
							ui.position.top = Math.floor(ui.position.top / grid) * grid;
						}
						if (ui.originalSize.width !== ui.size.width) {
							var paddingWidth = parseInt(ui.element.css('paddingLeft')) + parseInt(ui.element.css('paddingRight'));
							var borderWidth = parseInt(ui.element.css('borderLeftWidth')) + parseInt(ui.element.css('borderRightWidth'));
							ui.size.width = Math.ceil((ui.size.width + paddingWidth + borderWidth) / grid) * grid - ui.position.left % grid - paddingWidth - borderWidth;
						}
						if (ui.originalSize.height !== ui.size.height) {
							var paddingHeight = parseInt(ui.element.css('paddingTop')) + parseInt(ui.element.css('paddingBottom'));
							var borderHeight = parseInt(ui.element.css('borderTopWidth')) + parseInt(ui.element.css('borderBottomWidth'));
							ui.size.height = Math.ceil((ui.size.height + paddingHeight + borderHeight) / grid) * grid - ui.position.top % grid - paddingHeight - borderHeight;
						}
					}

					var aspectRatio = $(ui.helper).find('[data-aspectratio]').filter(function() {
						return $(this).attr('data-aspectratio') !== '';
					}).attr('data-aspectratio');
					if (aspectRatio) {
						if (Math.abs(ui.originalSize.width - ui.size.width) >= Math.abs(ui.originalSize.height - ui.size.height)) {
							$(ui.helper).outerHeight($(ui.helper).outerWidth() / parseFloat(aspectRatio));
						} else {
							$(ui.helper).outerWidth($(ui.helper).outerHeight() * parseFloat(aspectRatio));
						}
					}

					var currentWidth = $this.CONTAINER_SCROLL.scrollLeft() + $(ui.helper).offset().left - $this.CONTAINER_SCROLL.offset().left + $(ui.helper).outerWidth();
					var currentHeight = $this.CONTAINER_SCROLL.scrollTop() + $(ui.helper).offset().top - $this.CONTAINER_SCROLL.offset().top + $(ui.helper).outerHeight();

					$this._sizeContent($('[data-size]', item));
					$this._positionContent($('[data-position]', item));

                    if ($this.CONTAINER_CONTENT.hasClass('weave')) {
                        $this.CONTAINER_CONTENT.css({
                            width: Math.max(minWidth, currentWidth) + parseInt($this.CONTAINER_CONTENT.attr('data-padding-right')),
                            height: Math.max(minHeight, currentHeight) + parseInt($this.CONTAINER_CONTENT.attr('data-padding-bottom'))
                        });
                    }
				},
				stop: function(event, ui) {
					ui.position.left += parseInt(item.css('margin-left'));
					ui.position.top += parseInt(item.css('margin-top'));
					item.css({
						margin: '',
						left: ui.position.left,
						top: ui.position.top
					});
					if (item.data('revert') == true) {
						item.animate({
							left: ui.originalPosition.left,
							top: ui.originalPosition.top,
							width: ui.originalSize.width,
							height: ui.originalSize.height
						});
						return;
					}
					$this._updateChildren(item, {
						resize: true
					});
				}
			}).bind('mousedown.select',function(event) {
				item.data('revert', false);
				item.draggable('option', 'ctrlKey', event.ctrlKey || event.metaKey);
				if (event.shiftKey) {
					$this.selectedChildren().not('.' + $this.CLASS_LOCKED).draggable('option', 'axis', 'x');
				} else {
					item.draggable('option', 'axis', false);
				}
			}).bind('click.select', function(event) {
				if (item.data('revert') == true) {
					return;
				}

				var options = {
					context: $this
				};

				if (event.ctrlKey || event.metaKey) {
					$this.toggleChildSelection(item.data('id'), options);
					Editor.LayerList.toggleChildSelection(item.data('id'), options);
				} else {
					$this.selectChild(item.data('id'), options);
					Editor.LayerList.selectChild(item.data('id'), options);
				}
			})
			.prepend(this.TEMPLATE_HANDLES_PLACEHOLDER); // shim for chrome/ie/opera drawing outline not correctly

		if (item.attr('data-tool') == 'comment') {
			var textarea = $('textarea', item),
				previousTextContent = textarea.val();
			textarea.bind('focus', function() {
				Editor.enableHintOnTabClose();
			}).bind('blur', function() {
				Editor.disableHintOnTabClose();

				if (textarea.val() == previousTextContent) {
					return;
				}
				textarea.attr('disabled', 'disabled');
				item.addClass($this.CLASS_LOADING);
				$.post('/item/update-text-content', {
					item: item.attr('data-id'),
					research: Editor.getResearchId(),
					textcontent: textarea.val()
				}, function() {
					previousTextContent = textarea.val();
					item.removeClass($this.CLASS_LOADING);
					textarea.removeAttr('disabled');
				})
			});
			$('.icon-remove', item).click(function() {
				Dialog.removeItems([item.attr('data-id')]);
			});
			$('input[type="checkbox"]', item).bind('change', function() {
				var resolution = $(this).is(':checked') ? 'resolved' : 'open';
				item.toggleClass('comment-resolved');
				$.post('/item/update-tracking', {
					item: item.data('id'),
					research: Editor.getResearchId(),
					resolution: resolution,
					assignee: item.data('assignee-id'),
					deadline: item.data('deadline')
				}, function() {
					item.data('resolution', resolution);
				})
			});
		} else {
			item.bind('dblclick.edit',function() {
                if(item.hasClass('tool-simpletext')) {
                    var orgTopPosition;
                    if (parseInt(item.css('top')) < 50) {
                        orgTopPosition = item.position().top;
                        item.css('top', 50);
					}

					item.addClass('inline-editing');

					Editor.enableHintOnTabClose();
                    tinyMCE.baseURL = '/rc/plugin/tiny_mce';
                    item.find('.mceContentBody').tinymce(
                        new DialogForm()._getSimpleTextEditorOptions(item, function () {
                        	var toolbar = item.find('.mceLayout .mceFirst > .mceToolbar').parent();
                            var toolbarOptions = {
                                border: '1px solid',
                                position: 'fixed',
                                top: -40
                            };

                            if (item.position().left + 500 > $this.CONTAINER_SCROLL.width()) {
                                toolbarOptions.right = '-7px';
                            } else {
                                toolbarOptions.left = '-7px';
                            }

                            toolbar.css(toolbarOptions);

                            item.find('.mceLayout').css({
                                'border': 'none'
                            });
                            item.find('.mceLayout .mceLast > .mceIframeContainer').css({
                                'vertical-align': 'top',
                                'border': 'none'
                            });
                            item.find('iframe').contents().find('#tinymce').css({
                                background: 'none',
                                margin: 0
                            });
                        })
                    );

                    $(document).bind('click.simpletext mousedown.simpletext', function (e) {
                        if ($(e.target).closest('.mceListBoxMenu').length) {
                            return;
                        }
                        if ($(e.target).closest('.tool')[0] == item[0] && !$(e.target).hasClass('ui-resizable-handle')) {
                            return;
                        }
                        if (!item.find('.mceContentBody').tinymce()) {
                            return;
                        }

                        item.addClass($this.CLASS_LOADING);
                        if (orgTopPosition) {
                            item.css('top', orgTopPosition);
                        }

                        item.removeClass('inline-editing');

                        $.post('/item/update-simple-text-content', {
                            item: item.attr('data-id'),
                            research: Editor.getResearchId(),
                            textcontent: item.find('.mceContentBody').tinymce().getContent()
                        }, function () {
							Editor.loadItem(item.data('id'));
							Editor.PopoverList.reload();
                        });

                        item.find('.mceContentBody').tinymce().remove();
                        $(document).unbind('click.simpletext');
                        $(document).unbind('mousedown.simpletext');

						Editor.disableHintOnTabClose();
                    });
                } else {
                    Dialog.editItem(item.data('id'), {
                        context: $this
                    });
                }
			}).disableSelection();
		}

		$('.menu', item).click(function (event) {
			event.type = 'contextmenu';
			item.trigger(event);
			return false;
		});

		item.rotatable({
			snapToGrid: function(angle) {
				if (snapToGrid.is(':checked')) {
					var step = Math.max(parseInt(gridSize.val()) / 10, 1);
					angle = Math.floor(angle / step) * step;
				}
				return angle;
			},
			stopCallback: function() {
				$this._updateChildren(item)
			}
		});

		if (item.get(0).style.borderStyle && item.get(0).style.borderStyle != 'none') {
			this._updatePositionOfResizeHandles(item);
		}

		if (item.data('locked') == 1) {
			this.lockChildren([item.data('id')], {
				lock: true,
				children: item
			});
		}

		this._positionContent($('[data-position]', item));
	};

	// update the position of resize handles (border-correction)
	this._updatePositionOfResizeHandles = function(item) {
		var handles = ['nw', 'n', 'ne', 'e', 'se', 's', 'sw', 'w'];
		var cmargins = ['left', 'top'];
		var amargins = ['right', 'bottom', 'left', 'top'];
		var marginToApply;
		var border;

		for (var i = 0, pushDirection = null; i < handles.length; i++, pushDirection = !pushDirection) {
			if (pushDirection === true) {
                cmargins.push(amargins[0]);
                amargins.shift();
            } else if (pushDirection === false) {
				cmargins.shift();
			}

			marginToApply = {};
			$(cmargins).each(function() {
                border = 'border' + this.substr(0, 1).toUpperCase() + this.substr(1) + 'Width';
                marginToApply['margin-' + this] = -parseInt(item.get(0).style[border]);
			});

			$('.ui-resizable-' + handles[i], item).css(marginToApply);
		}

		$('.outline', item).css('margin', -parseInt(item.get(0).style.borderWidth));
	};

	this._isLoading = function(item) {
		return item.hasClass(this.CLASS_LOADING);
	};

	this._updateChildren = function(children, data) {
		data = $.extend({
			item: {},
			left: {},
			top: {},
			width: {},
			height: {},
			rotate: {}
		}, data || {});

		data.research = Editor.getResearchId();

		children.each(function() {
			var id = $(this).data('id');

			data.item[id] = id;
			data.left[id] = parseInt($(this).css('left'));
			data.top[id] = parseInt($(this).css('top'));
			data.width[id] = parseInt($(this).css('width'));
			data.height[id] = parseInt($(this).css('height'));
			data.rotate[id] = parseFloat($(this).attr('data-rotate'));
		});

		$.post('/item/update', data, function(data) {
			if (data) {
				$this._eLoadChild(data);
			}
			Editor.Overview.load();
		});
	};

	this._sizeContent = function(content) {
		if (content.length != 1) return;
		var factor;
		var fitToWidth = parseInt(content.parent().css('width'));
		var fitToHeight = parseInt(content.parent().css('height'));
		var contentWidth = parseInt(content.css('width'));
		var contentHeight = parseInt(content.css('height'));
		var widthFactor = fitToWidth / contentWidth;
		var heightFactor = fitToHeight / contentHeight;
		var size = {
			width: contentWidth,
			height: contentHeight
		};

		if (content.attr('data-size') == 'contain') {
			factor = (widthFactor < heightFactor) ? widthFactor : heightFactor;
			size.width = Math.ceil(factor * contentWidth);
			size.height = Math.ceil(factor * contentHeight);
		} else if (content.attr('data-size') == 'cover') {
			factor = (widthFactor > heightFactor) ? widthFactor : heightFactor;
			size.width = Math.ceil(factor * contentWidth);
			size.height = Math.ceil(factor * contentHeight);
		} else if (content.attr('data-size') == 'fit') {
			size.width = fitToWidth;
			size.height = fitToHeight;
		}

		content.css(size);
	};

	this._positionContent = function(content) {
		if (content.length != 1) return;

		var rotatedParent = content.closest('[style*=rotate]');
		var tmpTransform = (rotatedParent.css('transform') || rotatedParent.css('-moz-transform') || rotatedParent.css('-webkit-transform') ||
		                    rotatedParent.css('-o-transform') || rotatedParent.css('-ms-transform'));
		rotatedParent.css({
			'-moz-transform': '',
			'-o-transform': '',
			'-webkit-transform': '',
			'-ms-transform': '',
			'transform': ''
		});

		var options = $.parseJSON(content.attr('data-position'));
		content.position({
			at: options.at,
			my: options.my,
			of: options.of || content.parent(),
			collision: options.collision
		});

		rotatedParent.css({
			'-moz-transform': tmpTransform,
			'-o-transform': tmpTransform,
			'-webkit-transform': tmpTransform,
			'-ms-transform': tmpTransform,
			'transform': tmpTransform
		});
	};

	this.adjustContainerSize = function() {
        if ($this.CONTAINER_CONTENT.hasClass('popover')) {
            return;
        }

		$this.CONTAINER_CONTENT.css({
			width: $this.CONTAINER_SCROLL.get(0).scrollWidth,
			height: $this.CONTAINER_SCROLL.get(0).scrollHeight
		});

		Editor.Overview.CONTAINER_SIZE.html($this.CONTAINER_CONTENT.width() + 'x' + $this.CONTAINER_CONTENT.height() + ' Pixel');
	};

	this.eRemoveChildren = function() {
		this.updateCommentLayerSelect();
	};

	this.updateCommentLayerSelect = function() {
		var options = {},
			selectedValue = this.COMMENT_LAYER_SELECT.val();

		this.COMMENT_LAYER_SELECT.empty();

		this.children().each(function() {
			var el = $(this);
			if (el.attr('data-tool') != 'comment') {
				return;
			}

			var creatorId = el.attr('data-creator-id');
			if (!options[creatorId]) {
				options[creatorId] = {
					count: 0,
					name: el.attr('data-creator-name')
				};
			}

			options[creatorId].count++;
		});

		if (Object.keys(options).length == 0) {
			$('#sidebar-content').addClass('no-comments');
			return;
		}

		this.COMMENT_LAYER_SELECT.append('<option value="">show all notes</option>');
		this.COMMENT_LAYER_SELECT.append('<option value="hide">hide all notes</option>');
		this.COMMENT_LAYER_SELECT.append('<option value="open">show open notes</option>');
		this.COMMENT_LAYER_SELECT.append('<option value="resolved">show resolved notes</option>');
		for (var creatorId in options) {
			this.COMMENT_LAYER_SELECT.append('<option value="' + creatorId + '">show ' + options[creatorId].name + '\'s notes (' + options[creatorId].count + ')</option>');
		}
		this.COMMENT_LAYER_SELECT.val(selectedValue);
		$('#sidebar-content').removeClass('no-comments');
	};

	this.init = function() {
		var snapToGrid = $('#sidebar-commands-snaptogrid');
		var gridSize = $('#sidebar-commands-grid');

		this.CONTAINER_CONTENT.droppable({
			accept: function(draggable) {
				return draggable.hasClass('tool-draggable') && !draggable.hasClass('canceled');
			},
			drag: function (event, ui) {
				var grid = 1;
				if (snapToGrid.is(':checked')) {
					grid = Math.max(parseInt(gridSize.val()) / 10, 1);
					ui.position.left = Math.ceil(ui.position.left / grid) * grid;
					ui.position.top = Math.ceil(ui.position.top / grid) * grid;
				}

				var currentWidth = ui.position.left + $(ui.helper).outerWidth();
				var currentHeight = ui.position.top + $(ui.helper).outerHeight();

				$this.CONTAINER_CONTENT.css({
					width: Math.max(minWidth, currentWidth) + parseInt($('#content').attr('data-padding-right')),
					height: Math.max(minHeight, currentHeight) + parseInt($('#content').attr('data-padding-bottom'))
				});
			},
			drop: function(event, ui) {
				var top = ui.position.top;
				var left = ui.position.left;
				var width = ui.helper.innerWidth();
				var height = ui.helper.innerHeight();
				var fileId = ui.draggable.data('id');

				var placeholder = $('<div class="tool container-loading locked"></div>').css({
					width: width,
					height: height,
					top: top,
					left: left
				}).appendTo($this.CONTAINER_CONTENT);

				$.post('/item/add', {
					research: Editor.getResearchId(),
					weave: Editor.getWeaveId(),
					tool: ui.draggable.data('tool'),
					file: fileId || '',
					left: left,
					top: top,
					width: width,
					height: height
				}, function(data) {
					var itemId = $(data).data('id');
					placeholder.attr('data-id', itemId);
					$this._eLoadChild.apply($this, arguments);
					$this.updateCommentLayerSelect();

					$this.loadChildAnimation(itemId, true);
					if (ui.draggable.data('tool') == 'simpletext') {
						$this.loadChildAnimation(itemId, false);
                        Editor.selectItem(itemId);
                        Editor.editItem(itemId);
					} else if (ui.draggable.data('tool') == 'comment') {
						$this.loadChildAnimation(itemId, false);
						$('textarea', $this.child(itemId)).focus();
					} else {
						$this.loadChildAnimation(itemId, false);
						Editor.selectItem(itemId);
						Editor.editItem(itemId);
					}
					Editor.Overview.load();
				});
			}
		});

		this.CONTAINER_SCROLL.scroll(function() {
			Editor.Overview.currentAspect({
				left: $this.CONTAINER_SCROLL.scrollLeft(),
				top: $this.CONTAINER_SCROLL.scrollTop(),
				width: $this.CONTAINER_SCROLL.innerWidth(),
				height: $this.CONTAINER_SCROLL.innerHeight(),
				maxWidth: $this.CONTAINER_CONTENT.innerWidth(),
				maxHeight: $this.CONTAINER_CONTENT.innerHeight()
			});
		});

		$(document).keypress(function(event) {
			if (event.keyCode == 27) { // esc
				$this.selectedChildren().data('revert', true).trigger('mouseup');
				//$('#tool-dragging').addClass('canceled').trigger('mouseup');
			} else if (event.keyCode == 46) { // entf
				if ($('#dialog').length == 0 && event.target.tagName != 'TEXTAREA' && event.target.tagName != 'INPUT') {
					Dialog.removeItems($this.selectedChildrenIds());
				}
			}
		});
	};

	if (!ContentList.prototype.init) {
		ContentList.prototype = new List(init);
		ContentList.prototype.constructor = ContentList;
	}

	if (init) this.init();
}
new ContentList();

(function($) {

	$.fn.rotatable = function(options) {

		// Default Values
		var defaults = {
				rotatorClass: 'rotatable-handle',
				mtx: [1, 0, 0, 1],
				autoHide: true,
				snapToGrid: function(angle) {
					return angle;
				},
				stopCallback: function() {
				}
			}, opts = $.extend(defaults, options),
			_this = this,
			_rotator, center_coords;

		// Initialization
		this.initialize = function() {
			this.createHandler();
		};

		// Create Rotation Handler
		this.createHandler = function() {
			_rotator = $('<div class="' + opts.rotatorClass + '"></div>');
			_this.append(_rotator);
			_this.rotating = false;

			if (opts.autoHide) {
				$(_this).addClass('rotatable-autohide')
					.mouseenter(function() {
						$(this).removeClass('rotatable-autohide');
					})
					.mouseleave(function() {
						if (!_this.rotating) {
							$(this).addClass('rotatable-autohide');
						}
					});
			}

			this.bindRotation();
		};

		// Bind Rotation to Handler
		this.bindRotation = function() {

			_rotator.draggable({
				helper: 'clone',
				revert: false,
				start: function(e, ui) {
					e.stopPropagation();
					e.stopImmediatePropagation();

					var bBox = getBoundingBox(_this);
					center_coords = {
						'x': _this.parent().offset().left + bBox.left + bBox.width / 2,
						'y': _this.parent().offset().top + bBox.top + bBox.height / 2
					};

					ui.helper.hide();
				},
				drag: function(e) {
					var mouse_coords, angle;

					e.stopPropagation();
					e.stopImmediatePropagation();

					mouse_coords = {
						'x': e.originalEvent.pageX,
						'y': e.originalEvent.pageY
					};

					angle = _this.radToDeg(_this.getAngle(mouse_coords, center_coords)) - 90;
					angle = opts.snapToGrid(angle);

					return _this.rotate(angle);
				},
				stop: function() {
					opts.stopCallback();
				}
			});
		};

		// Get Angle
		this.getAngle = function(ms, ctr) {
			var x = ms.x - ctr.x,
				y = -ms.y + ctr.y,
				hyp = Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2)),
				angle = Math.acos(x / hyp);

			if (y < 0) {
				angle = 2 * Math.PI - angle;
			}

			return angle;
		};

		// Convert from Degrees to Radians
		this.degToRad = function(d) {
			return (d * (Math.PI / 180));
		};

		// Convert from Radians to Degrees
		this.radToDeg = function(r) {
			return (r * (180 / Math.PI));
		};

		// Rotate Element to the Given Degree
		this.rotate = function(degree) {
			var transform = 'rotate(' + -degree + 'deg)';
			_this.attr('data-rotate', -degree);
			_this.css({
				'-moz-transform': transform,
				'-o-transform': transform,
				'-webkit-transform': transform,
				'-ms-transform': transform,
				'transform': transform
			});
			return;
			var cos = Math.cos(_this.degToRad(-degree)),
				sin = Math.sin(_this.degToRad(-degree)),
				mtx = [cos, sin, (-sin), cos];

			this.updateRotationMatrix(mtx);
		};

		// Update CSS Transform Matrix (transform: matrix)
		this.updateRotationMatrix = function(m) {
			var matrix = 'matrix(' + m[0] + ', ' + m[1] + ', ' + m[2] + ', ' + m[3] + ', 0, 0)'

			_this.css({
				'-moz-transform': matrix,
				'-o-transform': matrix,
				'-webkit-transform': matrix,
				'-ms-transform': matrix,
				'transform': matrix
			});
		};

		return this.initialize();
	};
})(jQuery);

var getBoundingBox = function(item) {
	var shape = {
		left: parseInt(item.css('left')),
		top: parseInt(item.css('top')),
		width: parseInt(item.css('width')),
		height: parseInt(item.css('height')),
		rotate: parseFloat(item.attr('data-rotate'))
	};

	if (shape.rotate === 0 || shape.rotate === null) {
		return {
			left: shape.left,
			top: shape.top,
			right: shape.left + shape.width,
			bottom: shape.top + shape.height,
			rotate: shape.rotate,
			width: shape.width,
			height: shape.height
		};
	}

	var min = {x: 1e9, y: 1e9};
	var max = {x: -1e9, y: -1e9};
	var rad = Math.PI * shape.rotate / 180;
	var center = {x: shape.left + shape.width / 2, y: shape.top + shape.height / 2};
	var points = [
		{x: shape.left, y: shape.top},
		{x: shape.left + shape.width, y: shape.top },
		{x: shape.left, y: shape.top + shape.height},
		{x: shape.left + shape.width, y: shape.top + shape.height}
	];

	$.each(points, function (key, point) {
        var newPoint = { // @see: http://de.softuses.com/79869
            x: center.x + (point.x - center.x) * Math.cos(rad) + (point.y - center.y) * Math.sin(rad),
            y: center.y - (point.x - center.x) * Math.sin(rad) + (point.y - center.y) * Math.cos(rad)
        };

        min.x = Math.min(newPoint.x, min.x);
        min.y = Math.min(newPoint.y, min.y);
        max.x = Math.max(newPoint.x, max.x);
        max.y = Math.max(newPoint.y, max.y);
    });

	return {
		left: min.x,
		top: min.y,
		right: max.x,
		bottom: max.y,
		rotate: shape.rotate,
		width: max.x - min.x,
		height: max.y - min.y
	};
};