var openDialogs = {};

function Dialog() {

    Dialog.toParam = function (ids, key) {
        return $(ids).map(function () {
            return key + '[]=' + this;
        }).toArray().join('&');
    };

    Dialog.addWork = function () {
        new DialogForm({
            url: '/work/add',
            eAfterValidSubmit: function (data) {
                Editor.addWork(data);
                Dialog.editWork(data);
            }
        });
    };

    Dialog.editWork = function (id) {
        new DialogForm({
            url: '/work/edit?work=' + id,
            eAfterValidSubmit: function () {
                Editor.loadWork(id);
            }
        });
    };

    Dialog.showWork = function (id) {
        new DialogForm({
            url: '/work/show?work=' + id,
            dialog: {
                width: 720
            }
        });
    };

    Dialog.removeWorks = function (ids) {
        new DialogForm({
            url: '/work/remove?' + Dialog.toParam(ids, 'work'),
            eAfterValidSubmit: function () {
                Editor.removeWorks(ids);
            }
        });
    };

    Dialog.addFile = function (workId, options) {
        new DialogForm({
            url: '/work/add-file?work=' + workId + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                Editor.loadSelectedTab(options.context);
                Editor.loadWork(workId);
                Editor.SimpleMediaList.load();
            }
        });
    };

    Dialog.uploadFile = function (workId, options) {
        new DialogForm({
            url: '/work/upload-file?work=' + workId + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                Editor.loadSelectedTab(options.context);
                Editor.loadWork(workId);
                Editor.SimpleMediaList.load();
            }
        });
    };

    Dialog.updateThumbFileId = function (workId, options) {
        $.post('/work/update-thumb-file?work=' + workId + '&file=' + $(options.context).val());
    };

    Dialog.addPopover = function (toolId) {
        new DialogForm({
            url: '/popover/add?research=' + Editor.getResearchId() + '&weave=' + Editor.getWeaveId() + '&tool=' + toolId,
            eAfterOpen: function () {
                var duplicatePopover = $('#frm-meta-popover', this.DIALOG_FORM).parent(),
                    textPopover = $('#frm-meta-text', this.DIALOG_FORM).parent();

                $('#frm-meta-creation_type', this.DIALOG_FORM).bind('change', function () {
                    duplicatePopover.hide();
                    textPopover.hide();

                    switch ($(this).val()) {
                        case 'duplicate':
                            duplicatePopover.show();
                            break;
                        case 'text':
                            textPopover.show();
                            break;
                    }
                }).trigger('change');
            },
            eAfterValidSubmit: function (popoverId) {
                Editor.PopoverList.reload();
                Editor.ContentList.child(toolId).attr('data-popover', popoverId);
            }
        });
    };

    Dialog.editPopover = function (id) {
        new DialogForm({
            url: '/popover/edit?research=' + Editor.getResearchId() + '&popover=' + id,
            reloadOnSuccess: true,
            eAfterOpen: function () {
                var duplicatePopover = $('#frm-meta-popover', this.DIALOG_FORM).parent(),
                    textPopover = $('#frm-meta-text', this.DIALOG_FORM).parent();

                $('#frm-meta-creation_type', this.DIALOG_FORM).bind('change', function () {
                    duplicatePopover.hide();
                    textPopover.hide();

                    switch ($(this).val()) {
                        case 'duplicate':
                            duplicatePopover.show();
                            break;
                        case 'text':
                            textPopover.show();
                            break;
                    }
                }).trigger('change');
            }
        });
    };

    Dialog.removePopover = function (id, toolId) {
        new DialogForm({
            url: '/popover/remove?popover=' + id + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                if (Editor.isPopover()) {
                    $('#cancel-popover-edit').trigger('click');
                } else {
                    Editor.PopoverList.removeChildren([id]);

                    if (toolId) {
                        Editor.loadItem(toolId);
                    }
                }
            }
        });
    };

    Dialog.addTemplate = function (content) {
        new DialogForm({
            url: '/simple-media/add-template?research=' + Editor.getResearchId(),
            type: 'POST',
            data: {
                mediatype: 'text',
                textcontent: content
            },
            eAfterValidSubmit: function () {
                Editor.SimpleMediaList.load();
            }
        });
    };

    Dialog.editFile = function (id, options) {
        new DialogForm({
            url: '/file/edit?file=' + id,
            eAfterValidSubmit: function () {
                Editor.loadSelectedTab(options.context);
            }
        });
    };

    Dialog.removeFile = function (id, options) {
        new DialogForm({
            url: '/work/remove-file?work=' + options.workId + '&file=' + id + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                Editor.loadSelectedTab(options.context);
                Editor.loadWork(options.workId);
                Editor.SimpleMediaList.load();
            }
        });
    };

    Dialog.addSimpleMedia = function (researchId, options) {
        Editor.enableHintOnTabClose();
        new DialogForm({
            url: '/simple-media/add?research=' + researchId,
            eAfterValidSubmit: function () {
                Editor.SimpleMediaList.load();
            },
            eAfterClose: function () {
                Editor.disableHintOnTabClose();
            }
        });
    };

    Dialog.editSimpleMedia = function (id, options) {
        Editor.enableHintOnTabClose();
        new DialogForm({
            url: '/simple-media/edit?file=' + id,
            eAfterValidSubmit: function () {
                Editor.SimpleMediaList.load();
            },
            eAfterClose: function () {
                Editor.disableHintOnTabClose();
            }
        });
    };

    Dialog.showSimpleMedia = function (id) {
        new DialogForm({
            url: '/simple-media/show?file=' + id
        });
    };

    Dialog.showMadekMedia = function (id) {
        new DialogForm({
            url: '/madek-media/show?file=' + id
        });
    };

    Dialog.removeSimpleMedia = function (ids, options) {
        new DialogForm({
            url: '/simple-media/remove?' + Dialog.toParam(ids, 'file'),
            eAfterValidSubmit: function () {
                Editor.removeSimpleMedia(ids);
            }
        });
    };

    Dialog.editItem = function (itemId, options) {
        Editor.enableHintOnTabClose();
        openDialogs.editItem = new DialogForm({
            url: '/item/edit?item=' + itemId + '&research=' + Editor.getResearchId(),
            eAfterOpenAsync: function () {
                $(this.SELECTOR_TAB_CONTAINER, this.DIALOG_FORM).tabs('option', 'selected', 1);
            },
            eBeforeValidSubmit: function() {
                $('#_exposition').find('[data-id='+ itemId + ']').find('td').get(1).innerHTML = $('[name="common[title]"]', this.DIALOG_FORM).val();

                Editor.LayerList.draw();
            },
            eAfterValidSubmit: function () {
                Editor.loadItem(itemId);
                Editor.PopoverList.reload();
            },
            eAfterClose: function () {
                Editor.disableHintOnTabClose();
            }
        });
    };

    Dialog.convertItem = function (itemId, options) {
        new DialogForm({
            url: '/item/convert?item=' + itemId + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                Editor.loadItem(itemId);
            }
        });
    };

    Dialog.editToolTemplate = function (toolName, options) {
        new DialogForm({
            url: '/tool/edit-template?tool=' + toolName + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                Editor.ToolList.reload();
            }
        });
    };

    Dialog.removeItems = function (ids) {
        if (typeof ids !== 'object' || ids.length === 0) {
            return;
        }

        new DialogForm({
            url: '/item/remove?' + Dialog.toParam(ids, 'item') + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                Editor.removeItems(ids);
            }
        });
    };

    Dialog.cantPasteItems = function () {
        new DialogForm({
            url: '/item/paste-error'
        });
    };

    Dialog.selectResource = function (itemId, options) {
        new DialogForm({
            url: '/item/select-resource?research=' + Editor.getResearchId() + '&item=' + itemId,
            eAfterValidSubmit: function () {
                Editor.loadSelectedTab(options.context);
                Editor.loadItem(itemId);
            }
        });
    };

    Dialog.loadResource = function (itemId, options) {
        new DialogForm({
            url: '/item/load-resource?research=' + Editor.getResearchId() + '&item=' + itemId,
            eAfterValidSubmit: function (data) {
                tinyMCE.activeEditor.setContent(data);
            }
        });
    };

    Dialog.uploadResource = function (itemId, options) {
        new DialogForm({
            url: '/item/upload-resource?research=' + Editor.getResearchId() + '&item=' + itemId,
            eAfterValidSubmit: function () {
                Editor.loadSelectedTab(options.context);
                Editor.SimpleMediaList.load();
                Editor.loadItem(itemId);
            }
        });
    };

    Dialog.removeResource = function (resourceId, options) {
        new DialogForm({
            url: '/item/remove-resource?file=' + resourceId + '&item=' + options.itemId,
            eAfterValidSubmit: function (data) {
                Editor.loadSelectedTab(options.context);
                Editor.loadItem(options.itemId);
            }
        });
    };

    Dialog.addWeave = function (researchId) {
        new DialogForm({
            url: '/weave/add?research=' + researchId,
            eAfterValidSubmit: function (data) {
                location.href = '/editor?research=' + researchId + '&weave=' + data;
            }
        });
    };

    Dialog.editWeave = function (id) {
        new DialogForm({
            url: '/weave/edit?weave=' + id,
            reloadOnSuccess: true
        });
    };

    Dialog.removeWeave = function (id) {
        new DialogForm({
            url: '/weave/remove?weave=' + id,
            reloadOnSuccess: true
        });
    };

    Dialog.editResearch = function (id) {
        new DialogForm({
            url: '/research/edit?research=' + id
        });
    };

    Dialog.editTableOfContent = function (id) {
        new DialogForm({
            url: '/research/edit?research=' + id,
            eAfterOpenAsync: function () {
                $(this.SELECTOR_TAB_CONTAINER, this.DIALOG_FORM).tabs('option', 'selected', 1);
            }
        });
    };

    Dialog.audio = function (resourceId, options) {
        new DialogForm({
            url: '/item/select-audio?file=' + resourceId + '&item=' + options.itemId + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function (data) {
                Editor.loadSelectedTab(options.context);
                Editor.loadItem(options.itemId);
            }
        });
    };

    Dialog.image = function (resourceId, options) {
        new DialogForm({
            url: '/item/select-image?file=' + resourceId + '&item=' + options.itemId + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function (data) {
                Editor.loadSelectedTab(options.context);
                Editor.loadItem(options.itemId);
            }
        });
    };

    Dialog.restoreTool = function (toolId, revision, wasDeleted) {
        new DialogForm({
            url: '/item/restore?item=' + toolId + '&revision=' + revision + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function (data) {
                if (wasDeleted) {
                    $(data).appendTo(Editor.ContentList.CONTAINER_CONTENT);
                    Editor.Overview.load();
                }

                Editor.loadItem(toolId, function () {
                    Editor.selectItem(toolId);
                });
                $('#dialog').dialog('close');
            }
        });
    };

    Dialog.restoreFootnote = function (popoverId) {
        new DialogForm({
            url: '/popover/restore?popover=' + popoverId + '&research=' + Editor.getResearchId(),
            eAfterValidSubmit: function () {
                $('#dialog').dialog('close');

                Editor.PopoverList.reload();
            }
        });
    };

    Dialog.sendCollaborationMessage = function () {
        new DialogForm({
            url: '/editor/collaboration-message?research=' + Editor.getResearchId(),
        });
    };
}
new Dialog();