var formIsValid = false;
var formAction = '';

function DialogForm(params) {
    var $this = this;
    var files;
    var submitButton;
    var settings = {
        url: null,
        type: 'GET',
        data: null,
        overlay: true,
        content: null,
        asyncFileUpload: true,
        closeOnSuccess: true,
        reloadOnSuccess: false,
        insertResponseOnSuccess: null,
        submitInNewTab: false
    };
    var dialogSettings = {
        zIndex: 99999,
        width: 605,
        minHeight: 50,
        modal: true,
        resizable: false,
        autoOpen: true,
        dialogClass: 'dialog',
        overflow: null,
        closeOnEscape: false,
        open: function (event) {
            $this._eOpen(event);
        },
        close: function (event) {
            $this._eClose(event);
        }
    };

    this.DIALOG = null;
    this.DIALOG_FORM = null;
    this.CONTAINER_LOADING = $('<div id="dialog-loading-screen" class="container-loading"></div>').css({
        width: $($this.DIALOG_FORM).width,
        height: $($this.DIALOG_FORM).height,
        top: $($this.DIALOG_FORM).top,
        left: $($this.DIALOG_FORM).left
    });

    this.SELECTOR_AUTOCOMPLETE_SIMPLE = 'input[data-autocomplete]';
    this.SELECTOR_AUTOCOMPLETE = 'select[data-autocomplete]';
    this.SELECTOR_MULTI_SELECT = 'select[data-multiselect]';
    this.SELECTOR_TEXT_EDITOR = 'textarea[data-editor]';
    this.SELECTOR_TEXT_EDITOR_AUTO_GROW = 'textarea[data-autogrow]';
    this.SELECTOR_SIMPLE_TEXT_EDITOR = 'textarea[data-simpleeditor]';
    this.SELECTOR_WRITEABLE_FIELDS = 'p [name][readonly!="readonly"]';
    this.SELECTOR_INVALID_FIELDS = 'p [name].error';
    this.SELECTOR_TAB_CONTAINER = '.ui-tabs';
    this.SELECTOR_TAB_PANEL = '.ui-tabs-panel';
    this.SELECTOR_ACCORDION_CONTAINER = '.ui-accordion';
    this.SELECTOR_ACCORDION_PANEL = '.ui-accordion-content';
    this.SELECTOR_OBJECT_VIEWER_IMAGE_SELECTION_OPTION = '#frm-options-startlevel';
    this.SELECTOR_SIDE_SHOW_IMAGE_SELECTION_OPTION = '#frm-options-slidesspeed';
    this.SELECTOR_SIDE_SHOW_PLAY_OPTION = '#frm-options-playoption';
    this.SELECTOR_ADD_FIELD_BUTTON = '.form-add-field';
    this.SELECTOR_REMOVE_FIELD_BUTTON = '.form-remove-field';
    this.SELECTOR_COLOR_PICKER = 'input[data-colorpicker]';

    this.init = function (params) {
        $this._initEvents();

        if (params) {
            settings = $.extend({}, settings, params);
            if (params.dialog) {
                dialogSettings = $.extend({}, dialogSettings, params.dialog);
            }
        }

        if (dialogSettings.autoOpen) {
            $this._eLoad();
        }
    };

    this.get = function () {
        return $this.DIALOG_FORM;
    };


    this.eBeforeLoad = function () {

    };

    this.eLoad = function (data) {
        if (!settings.content && settings.url) {
            $.ajax({
                type: settings.type,
                url: settings.url,
                data: settings.data,
                beforeSend: function () {
                    if (settings.overlay) {
                        $('<div/>').attr('id', 'overlay').addClass('container-loading').css({
                            width: $(document).width(),
                            height: $(document).height()
                        }).appendTo('body')
                    }
                },
                complete: function () {
                    $('#overlay').remove();
                },
                success: function (data) {
                    settings.content = data;
                    $this._eLoad()
                }
            });
        } else {
            if (!$(settings.content).attr('id')) {
                settings.content = $(settings.content).attr('id', 'dialog');
            }

            $(settings.content).first().dialog(dialogSettings);
            $this._appendAllNodesToDialog(settings.content);
        }
    };

    this.eAfterLoad = function () {

    };


    this.eBeforeOpen = function () {

    };

    this.eOpen = function (event) {
        if (event) {
            $this.DIALOG_FORM = $(event.target);
            $this.DIALOG = $this.DIALOG_FORM.parent().attr('id', 'dialog-' + new Date().getTime());
        }

        var dialogTitle = $('.ui-dialog-title', $this.DIALOG),
            subTitle = $this.DIALOG_FORM.attr('data-subtitle');
        if (dialogTitle.find('.ui-dialog-subtitle').length == 0 && subTitle) {
            dialogTitle.append($('<span class="ui-dialog-subtitle">' + subTitle + '</span>'));
        }

        if (dialogSettings.overflow) {
            $this.DIALOG.css('overflow', dialogSettings.overflow);
        }

        $this.DIALOG.next('.ui-widget-overlay').insertBefore($this.DIALOG);

        $($this.DIALOG_FORM).unbind('submit').submit(function () {
            if (submitButton && !submitButton.is(':visible')) {
                return false;
            }

            $this._eSubmit();
            return false;
        });

        $('button[type=submit]', $this.DIALOG_FORM).click(function (event) {
            if (event.target.nodeName == 'SPAN') {
                submitButton = $(event.target).closest('button');
            } else {
                submitButton = $(event.target);
            }
        });

        $('input', $this.DIALOG_FORM).bind('keydown', function (event) {
            if ((event.keyCode ? event.keyCode : (event.which ? event.which : event.charCode)) == 13) {
                $('button[type=submit]', $this.DIALOG_FORM).trigger('click');
                return false;
            }
        });

        $this._hideLoadingScreen();
        $this._initAutocomplete();
        $this._initAutocompleteSimple();
        $this._initFileUpload();
        $this._initFieldErrorMessages();
        $this._initTextEditor();
        $this._initAutoGrowTextEditor();
        $this._initSimpleTextEditor();
        $this._initMultiSelect();
        $this._initObjectViewer();
        $this._initSlideshow();
        $this._initKeywords();
        $this._initAddFieldButton();
        $this._initRemoveFieldButton();
        $this._focusField();
    };

    this.eAfterOpen = function () {

    };

    this.eAfterOpenAsync = function () {

    };


    this.eBeforeBackground = function () {

    };

    this.eBackground = function () {
        Editor.UploadList.loadChild($this.DIALOG.attr('id'), {dialogForm: $this});

        $this.DIALOG.hide();
        $this.DIALOG.prev('.ui-widget-overlay').hide();

        if ($.ui.dialog.overlay) {
            $.ui.dialog.overlay.maxZ = $('.ui-widget-overlay:visible:last').css('z-index') || 0;
        }

        Metro.toast.create("You can find your running uploads under Media -> Uploads.", null, 10000);
    };

    this.eAfterBackground = function () {

    };


    this.eBeforeForeground = function () {

    };

    this.eForeground = function () {
        if ($this.DIALOG.is(':visible')) {
            return;
        }

        $this.DIALOG.show();
        $this.DIALOG.prev('.ui-widget-overlay').show();

        if ($.ui.dialog.overlay) {
            $.ui.dialog.overlay.maxZ = $('.ui-widget-overlay:visible:last').css('z-index') || dialogSettings.zIndex;
        }

        if (typeof Editor !== 'undefined') {
            Editor.UploadList.removeChildren([$this.DIALOG.attr('id')]);
        }
    };

    this.eAfterForeground = function () {

    };


    this.eBeforeSubmit = function () {
        $('[disabled="disabled"]', $this.DIALOG_FORM).removeAttr('disabled');
    };

    this.eSubmit = function () {
        $this._showLoadingScreen();
        $this._saveTextEditorContent();
        $this._saveSimpleTextEditorContent();
        $this._handleSubmit();
    };

    this.eAfterSubmit = function () {

    };


    this.eBeforeEvalSubmit = function () {
        $this._hideLoadingScreen();
    };

    this.eEvalSubmit = function (data, b, xhr, formData) {
        if ($this._isFormValid(data, b, xhr)) {
            $this._eValidSubmit(data, b, xhr, formData);
        } else {
            $this._eInvalidSubmit(data, b, xhr, formData);
        }
    };

    this.eAfterEvalSubmit = function () {

    };


    this.eBeforeValidSubmit = function () {

    };

    this.eValidSubmit = function (data, b, xhr) {
        if (settings.closeOnSuccess) {
            $this._eClose();
        }
        if (settings.insertResponseOnSuccess) {
            $(settings.insertResponseOnSuccess).html(data);
        } else if (settings.reloadOnSuccess) {
            location.reload();
        }
    };

    this.eAfterValidSubmit = function () {

    };


    this.eBeforeInvalidSubmit = function () {

    };

    this.eInvalidSubmit = function (data, b, xhr) {
        $($this.DIALOG_FORM).html($(data).html());
        $this._appendAllNodesToDialog(data);
        $this._eOpen();
        $this._eForeground();
    };

    this.eAfterInvalidSubmit = function () {

    };


    this.eBeforeClose = function () {
        this._removeTextEditor();
        this._removeSimpleTextEditor();
        this._removeColorpicker();
    };

    this.eClose = function () {
        $this._eForeground();

        var form = $($this.DIALOG_FORM);
        if (form.fileupload && form.data('fileupload')) {
            form.fileupload('destroy');
        }
        form.remove();
        hideTitle();
    };

    this.eAfterClose = function () {

    };

    this._initEvents = function () {
        $this._eLoad = function () {
            this._apply('eBeforeLoad', arguments) !== false &&
            this._apply('eLoad', arguments) !== false &&
            this._apply('eAfterLoad', arguments);
        };
        $this._eOpen = function () {
            this._apply('eBeforeOpen', arguments) !== false &&
            this._apply('eOpen', arguments) !== false &&
            this._apply('eAfterOpen', arguments);
            setTimeout(function () {
                $this._apply('eAfterOpenAsync', arguments);
            }, 0);
        };
        $this._eBackground = function () {
            this._apply('eBeforeBackground', arguments) !== false &&
            this._apply('eBackground', arguments) !== false &&
            this._apply('eAfterBackground', arguments);
        };
        $this._eForeground = function () {
            this._apply('eBeforeForeground', arguments) !== false &&
            this._apply('eForeground', arguments) !== false &&
            this._apply('eAfterForeground', arguments);
        };
        $this._eSubmit = function () {
            this._apply('eBeforeSubmit', arguments) !== false &&
            this._apply('eSubmit', arguments) !== false &&
            this._apply('eAfterSubmit', arguments);
        };
        $this._eEvalSubmit = function () {
            this._apply('eBeforeEvalSubmit', arguments) !== false &&
            this._apply('eEvalSubmit', arguments) !== false &&
            this._apply('eAfterEvalSubmit', arguments);
        };
        $this._eValidSubmit = function () {
            this._apply('eBeforeValidSubmit', arguments) !== false &&
            this._apply('eValidSubmit', arguments) !== false &&
            this._apply('eAfterValidSubmit', arguments);
        };
        $this._eInvalidSubmit = function () {
            this._apply('eBeforeInvalidSubmit', arguments) !== false &&
            this._apply('eInvalidSubmit', arguments) !== false &&
            this._apply('eAfterInvalidSubmit', arguments);
        };
        $this._eClose = function () {
            this._apply('eBeforeClose', arguments) !== false &&
            this._apply('eClose', arguments) !== false &&
            this._apply('eAfterClose', arguments);
        };
        $this._apply = function (method, args) {
            if ($.isFunction(settings[method])) {
                settings[method].apply($this, args);
            } else if ($.isFunction(this[method])) {
                $this[method].apply($this, args);
            }
        };
    };

    this._initFileUpload = function () {
        var form = $($this.DIALOG_FORM);
        if ($(':file', form).length == 0) {
            return;
        }

        if (form.data('fileupload')) {
            form.fileupload('destroy');
        }

        form.fileupload({
            replaceFileInput: false,
            forceIframeTransport: false,
            dataType: 'html',
            add: function (e, data) {
                files = data;
            },
            done: function (e, data) {
                $this.CONTAINER_LOADING.html('');
                $this._eEvalSubmit(data.result, null, data.jqXHR);
            },
            start: function (e, data) {
                $this.CONTAINER_LOADING.html('\
                    <div style="background:white;padding:1px 10px; width: 500px">\
                        <p>\
                            Uploading file(s)\
                            <a href="#" class="ui-dialog-titlebar-minimize">to background</a>\
                        </p>\
                        <div id="progressbar"></div>\
                        <p id="progress"></p>\
                    </div>\
                ');
                $this.CONTAINER_LOADING.find('.ui-dialog-titlebar-minimize').click(function () {
                    $this._eBackground();
                    return false;
                });

                var progressbar = $('#progressbar', $this.CONTAINER_LOADING);
                progressbar.parent().position({
                    my: 'center',
                    at: 'center',
                    of: progressbar.parent().parent()
                });
            },
            progress: function (e, data) {
                var percent = Math.round(data.loaded / data.total * 100);

                var progress = $('#progress', $this.CONTAINER_LOADING);
                progress.html('<span style="float:right;color:grey">' + $this._formatNumber(data.loaded) + 'kb of ' + $this._formatNumber(data.total) + 'kb</span>' + percent + '% complete');

                var progressbar = $('#progressbar', $this.CONTAINER_LOADING);
                progressbar.progressbar({
                    value: percent
                });
            }
        });
    };

    this._initAutocomplete = function () {
        $($this.SELECTOR_AUTOCOMPLETE, $this.DIALOG_FORM).each(function () {
            var addItem = function (element, list, item, init) {
                var entry = $('<li></li>');
                entry.data('item', item);
                if (!init && options.formatNewValue) {
                    entry.html(options.formatNewValue.replace(/%s/, item.label));
                } else {
                    entry.html(item.label);
                }
                entry.appendTo(list);

                $('<i class="icon icon-remove"></i>').click(function () {
                    if (options.formatOldValue) {
                        entry.contents()[0].textContent = options.formatOldValue.replace(/%s/, item.label.replace(/(\[|\{)(.*)(\]|\})/, '$2'));
                        $(this).hide();
                    } else {
                        $(this).closest('li').remove();
                    }

                    element.find('option[value="' + item.value + '"]').remove();

                    if (list.find('li').length < options.maxcount) {
                        autocomplete.removeClass('hide');
                    }
                    if (list.find('li').length == 0) {
                        removeAll.hide();
                    }
                }).appendTo(entry);

                if (list.find('li').length >= options.maxcount) {
                    removeAll.hide();
                    autocomplete.addClass('hide');
                } else {
                    removeAll.css('display', 'inline-block');
                }

            };
            var element = $(this).hide();
            var options = {
                url: null,
                placeholder: '',
                maxcount: 1e3,
                excludeCurrentUser: false,
                formatNewValue: '',
                formatOldValue: ''
            };
            if ($(this).attr('data-autocomplete-options')) {
                options = $.parseJSON($(this).attr('data-autocomplete-options'));
            }

            var list = $('<ul class="autocomplete-selected"></ul>').insertAfter(element);
            var removeAll = $('<a href="" class="selection-remove-all icon icon-remove" title="remove all"></a>').click(function () {
                list.find('.icon-remove').trigger('click');
                return false;
            }).hide().insertBefore(list);

            var source = [];
            var selected = [];
            var select = function (event, ui) {
                var option = element.find('option[value=' + ui.item.value + ']');

                if (!option.length) {
                    option = $('<option></option>').attr('value', ui.item.value).attr('selected', 'selected').appendTo(element);
                    addItem(element, list, ui.item);
                }

                this.value = '';
                return false;
            };

            var autocomplete = $('<input type="text" name="autocomplete"/>')
                .attr('class', element.attr('class') || '')
                .attr('placeholder', options.placeholder)
                .autocomplete({
                    delay: 0,
                    source: function (request, response) {
                        $.getJSON(options.url, {
                            name: request.term,
                            excludeCurrentUser: options.excludeCurrentUser
                        }, function (data) {
                            source = data.slice(0, 15);
                            response(source);
                        });
                    },
                    focus: function (event, ui) {
                        $(this).val(ui.item.label);
                        return false;
                    },
                    select: function (event, ui) {
                        select.apply(this, [event, ui]);
                        this.value = '';
                        return false;
                    }
                }).bind('keypress', function (e) {
                    var code = e.keyCode || e.which;
                    if (code == 13) { //Enter keycode
                        var items = $.ui.autocomplete.filter(source, autocomplete.val());
                        if (items.length > 0) {
                            select.apply(this, [e, {item: items[0]}]);
                        }
                        autocomplete.val('');
                        $('.ui-autocomplete.ui-menu.ui-widget').hide();
                        return false;
                    }
                })
                .attr('autocomplete', 'nope')
                .insertAfter(this);

            element.find('option').each(function () {
                source.push({
                    value: $(this).attr('value'),
                    label: $(this).html()
                });
                if ($(this).is(':selected')) {
                    addItem(element, list, {
                        value: $(this).attr('value'),
                        label: $(this).html()
                    }, true);
                }
            });
        });
    };

    this._initAutocompleteSimple = function () {
        $($this.SELECTOR_AUTOCOMPLETE_SIMPLE, $this.DIALOG_FORM).each(function () {
            var $element = $(this);
            var options = {
                url: null,
                change: null,
                property: 'name'
            };
            if ($element.attr('data-autocomplete-options')) {
                options = $.extend({}, options, $.parseJSON($(this).attr('data-autocomplete-options')));
            }

            var autocomplete = $element.autocomplete({
                delay: 0,
                source: function (request, response) {
                    $.getJSON(options.url, {
                        property: options.property,
                        value: request.term
                    }, function (source) {
                        response(source.slice(0, 10));
                    });
                },
                focus: function (event, ui) {
                    $element.val(ui.item.label);
                    return false;
                },
                select: function (event, ui) {
                    $element.val(ui.item.label);
                    if ($.browser.msie || $.browser.safari) {
                        $element.trigger('change');
                    }
                    return false;
                }
            }).bind('keypress', function (e) {
                var code = e.keyCode || e.which;
                if (code == 13) { //Enter keycode
                    var items = $.ui.autocomplete.filter(source, autocomplete.val());
                    if (items.length > 0) {
                        select.apply(this, [e, {item: items[0]}]);
                    }
                    autocomplete.val('');
                    $('.ui-autocomplete.ui-menu.ui-widget').hide();
                    return false;
                }
            })
                .attr('autocomplete', 'nope');
        });
    };

    this._initMultiSelect = function () {
        $($this.SELECTOR_MULTI_SELECT, $this.DIALOG_FORM).each(function () {
            var options = {
                sortable: false,
                addAll: false
            };
            if ($(this).attr('data-multiselect-options')) {
                options = $.parseJSON($(this).attr('data-multiselect-options'));
            }
            $(this).addClass('multiselect').multiselect(options);
        });
    };

    this._initTextEditor = function () {
        if (typeof tinymce === 'undefined') {
            return;
        }

        tinyMCE.baseURL = '/rc/plugin/tiny_mce';

        $($this.SELECTOR_TEXT_EDITOR, $this.DIALOG_FORM).each(function () {
            DialogForm.loadTextEditor($(this)).next('.mceEditor').css('float', 'left');

            $('<br/><br/>').insertBefore(this);
        });
    };

    this._initAutoGrowTextEditor = function () {
        $($this.SELECTOR_TEXT_EDITOR_AUTO_GROW, $this.DIALOG_FORM).each(function () {
            $(this).bind('keyup', function () {
                var element = $(this);
                element.css('height', '5px');
                element.css('height', Math.max(250, element[0].scrollHeight + 10) + 'px');
            });
        });
    };

    this._saveTextEditorContent = function () {
        if (typeof tinymce === 'undefined') {
            return;
        }
        if (typeof tinymce.editors['text-editor'] === 'undefined') {
            return;
        }

        tinymce.editors['text-editor'].save();
    };

    this._removeTextEditor = function () {
        if (typeof tinymce === 'undefined') {
            return;
        }
        if (typeof tinymce.editors['text-editor'] === 'undefined') {
            return;
        }

        tinymce.editors['text-editor'].editorManager.remove();
    };

    this._initSimpleTextEditor = function () {
        if (typeof tinymce === 'undefined') {
            return;
        }

        $($this.SELECTOR_SIMPLE_TEXT_EDITOR, $this.DIALOG_FORM).each(function () {
            var element = $(this);

            element.tinymce($this._getSimpleTextEditorOptions(element))
                .next('.mceEditor').css('float', 'left');
        });
    };

    this._getSimpleTextEditorOptions = function (element, callback) {
        return {
            body_class: 'simple-text-editor-content',
            popup_css_add: '/rc/plugin/tiny_mce/themes/advanced/skins/default/dialog.css',
            content_css: '/rc/css/simpletexteditor_research.css?q=1385298610',
            mode: 'exact',
            theme: 'advanced',
            width: element.width() + 'px',
            height: element.height() + 'px',
            convert_urls: false,
            theme_advanced_toolbar_location: 'top',
            theme_advanced_toolbar_align: 'left',
            theme_advanced_statusbar_location: 'none',
            theme_advanced_fonts: "Arial=arial,helvetica,sans-serif;" +
            "Avant Garde=avant garde,arial black;" +
            "Courier New=courier new,courier;" +
            "Georgia=georgia,palatino;" +
            "Helvetica=helvetica;" +
            "Tahoma=tahoma,arial,helvetica,sans-serif;" +
            "Times New Roman=times new roman,times;" +
            "Palatino=palatino,georgia;",
            theme_advanced_font_sizes: "8pt,10pt,12pt,14pt,18pt,24pt,36pt",
            plugins: 'autolink,lists,print,preview,searchreplace,visualblocks,fullscreen,insertdatetime,paste,inlinepopups',
            theme_advanced_buttons1: 'undo,redo,|,bold,italic,|,fontselect,fontsizeselect,|,sub,sup,|,bullist,numlist,|,outdent,indent,|,link,unlink',
            theme_advanced_buttons2: '',
            theme_advanced_buttons3: '',
            paste_strip_class_attributes: 'all',
            setup: function (ed) {
                ed.onPreProcess.add(function (pl, o) {
                    o.content = $this.strip_tags(o.content, '<strong><em><b><i><sub><sup><ul><ol><li><p><a>');
                });
                ed.onInit.add(function (ed, e) {
                    ed.focus();
                });

                if (typeof callback !== 'undefined') {
                    ed.onInit.add(callback);
                }
            }
        };
    };

    this._saveSimpleTextEditorContent = function () {
        if (typeof tinymce === 'undefined') {
            return;
        }
        if (typeof tinymce.editors['simple-text-editor'] === 'undefined') {
            return;
        }

        tinymce.editors['simple-text-editor'].save();
    };

    this._removeSimpleTextEditor = function () {
        if (typeof tinymce === 'undefined') {
            return;
        }
        if (typeof tinymce.editors['simple-text-editor'] === 'undefined') {
            return;
        }

        tinymce.editors['simple-text-editor'].editorManager.remove();
    };

    this._removeColorpicker = function () {
        $($this.SELECTOR_COLOR_PICKER, $this.DIALOG_FORM).each(function () {
            $(this).colorpickerRemove();
        });
    };

    this._initObjectViewer = function () {
        $($this.SELECTOR_OBJECT_VIEWER_IMAGE_SELECTION_OPTION, $this.DIALOG_FORM).each(function () {
            var resourceList = JSON.parse($(this).attr('data-resources')),
                textField = $(this);
            $('<a href="#" style="line-height:19px">edit</a>').appendTo($(this).parent()).bind('click', function () {
                var imageList = [];
                for (var i in resourceList) {
                    imageList.push('<img src="' + resourceList[i] + '" data-id="' + i + '"/>');
                }
                var dialogBox = $('<div id="object-viewer-image-selection-dialog" title="SELECT START IMAGE"></div>')
                    .html('<div class="object-viewer-image-selection-container">' + imageList.join('') + '</div>')
                    .dialog({
                        zIndex: 99999,
                        width: 605,
                        minHeight: 50,
                        modal: true,
                        resizable: false,
                        autoOpen: true,
                        dialogClass: 'dialog',
                        position: 'center'
                    });

                var images = $('img', dialogBox).bind('click', function () {
                    images.removeClass('active');
                    $(this).addClass('active');
                    textField.val($(this).data('id'));
                    dialogBox.parent().fadeOut(function () {
                        dialogBox.dialog('close')
                    });
                });
                images.filter('[data-id="' + textField.val() + '"]').addClass('active');
                return false;
            });
        });
    };

    this._initSlideshow = function () {
        var openImageInPopup = $('#frm-options-openpictureinpopup', $this.DIALOG_FORM), hintAlreadyDisplayed = false;
        $($this.SELECTOR_SIDE_SHOW_PLAY_OPTION, $this.DIALOG_FORM)
            .bind('change', function () {
                var disabled = $(this).val() != '';
                openImageInPopup.prop('disabled', disabled);
                if (!hintAlreadyDisplayed && openImageInPopup.is(':checked') && disabled) {
                    hintAlreadyDisplayed = true;
                    var hint = $('<div class="open-image-in-popup-disabled-hint">Automated slide shows use \'click\' to start/stop the slide show. The option \'on click open image in popup\' has been disabled.</div>')
                        .appendTo($(this).closest('.ui-accordion'))
                        .bind('click', function () {
                            $(this).remove();
                        });
                    setTimeout(function () {
                        hint.fadeOut('slow', function () {
                            $(this).remove();
                        });
                    }, 6000);
                }
            })
            .trigger('change');
        hintAlreadyDisplayed = openImageInPopup
            .bind('change', function () {
                hintAlreadyDisplayed = false
            })
            .is('disabled');

        $($this.SELECTOR_SIDE_SHOW_IMAGE_SELECTION_OPTION, $this.DIALOG_FORM).each(function () {
            var textField = $(this),
                resourceList = JSON.parse($(this).attr('data-resources'));

            $('<a href="#" style="line-height:19px">edit</a>').appendTo($(this).parent()).bind('click', function () {
                var imageList = [],
                    slideSpeedList = JSON.parse(textField.val()),
                    placeholderValue = $('#frm-options-defaultslidespeed').val(),
                    slideSpeed;
                for (var i in resourceList) {
                    slideSpeed = slideSpeedList[resourceList[i].id] || '';
                    imageList.push('<div><img src="' + resourceList[i].url + '"/><input type="text" data-id="' + resourceList[i].id + '" value="' + slideSpeed + '" placeholder="' + placeholderValue + '"/></div>');
                }
                var dialogBox = $('<form id="slide-show-image-selection-dialog" title="SPECIFY SLIDE SPEED"></form>')
                    .html('<div class="slide-show-image-selection-container">' + imageList.join('') + '</div>')
                    .append($('<button type="reset">cancel</button>').button({
                        icons: {
                            primary: "ui-icon-close"
                        }
                    }).bind('click', function () {
                        dialogBox.parent().fadeOut(function () {
                            dialogBox.dialog('close')
                        });
                    }))
                    .append($('<button type="submit">submit</button>').button({
                        icons: {
                            primary: "ui-icon-check"
                        }
                    }).bind('click', function () {
                        var data = {};
                        $('input', dialogBox).each(function () {
                            if ($(this).val() == '') {
                                return;
                            }
                            data[$(this).attr('data-id')] = $(this).val();
                        });
                        textField.val(JSON.stringify(data));
                        dialogBox.parent().fadeOut(function () {
                            dialogBox.dialog('close')
                        });
                    }))
                    .bind('submit', function () {
                        $('button[type="submit"]').trigger('click');
                        return false;
                    })
                    .dialog({
                        zIndex: 99999,
                        width: 605,
                        minHeight: 50,
                        modal: true,
                        resizable: false,
                        autoOpen: true,
                        dialogClass: 'dialog',
                        position: 'center'
                    });
                return false;
            });
        });
    };

    this._initKeywords = function () {
        if (!$.isFunction(jQuery.ui.tagit)) {
            return;
        }
        $('[data-tagit]', $this.DIALOG_FORM).each(function () {
            var el = $(this),
                options = el.attr('data-tagit');

            if (options) {
                options = JSON.parse(options);
            } else {
                options = {
                    autocomplete: {
                        delay: 0,
                        minLength: 1,
                        singleFieldDelimiter: ',',
                        source: function (search, showChoices) {
                            var $this = this;
                            return $.get('/keyword/suggest?search=' + search.term, function (response) {
                                showChoices($this._subtractArray($.parseJSON(response), $this.assignedTags()));
                            })
                        }
                    },
                    allowSpaces: true
                };
            }

            el.tagit(options);
        });
    };

    this._initAddFieldButton = function () {
        var index = $($this.SELECTOR_ADD_FIELD_BUTTON, $this.DIALOG_FORM).find('select').length;
        $($this.SELECTOR_ADD_FIELD_BUTTON, $this.DIALOG_FORM).bind('click', function () {
            var select = $(this).prev('input').prev('select').clone().val('').addClass('form-additional-multi-field').insertAfter(this),
                input = $(this).prev('input').clone().val('').addClass('form-additional-multi-field').insertAfter(select);
            ++index;
            input.attr('name', input.attr('name').replace(/\[[\d]+]/, '[' + index + ']'));
            select.attr('name', select.attr('name').replace(/\[[\d]+]/, '[' + index + ']'));
            $('<span class="ui-icon ui-icon-minus form-remove-field"></span>').bind('click', function () {
                select.add(input).add(this).remove();
            }).insertAfter(input);
        });
    };

    this._initRemoveFieldButton = function () {
        $($this.SELECTOR_REMOVE_FIELD_BUTTON, $this.DIALOG_FORM).bind('click', function () {
            $(this).prev('input').remove();
            $(this).prev('select').remove();
            $(this).remove();
        });
    };

    this._initFieldErrorMessages = function () {
        $($this.SELECTOR_WRITEABLE_FIELDS, $this.DIALOG_FORM).each(function () {
            $(this).focus(function () {
                showTitle($(this).next('.error'));
            }).blur(hideTitle)
        });
    };


    this._showLoadingScreen = function () {
        $this.CONTAINER_LOADING.appendTo($this.DIALOG);
    };

    this._hideLoadingScreen = function () {
        $this.CONTAINER_LOADING.remove();
    };


    this._handleSubmit = function () {
        var formData = $($this.DIALOG_FORM).serializeArray();
        if (submitButton) {
            formData.push({
                name: submitButton.attr('name'),
                value: submitButton.attr('name')
            });
        }


        if ($(':file', $($this.DIALOG_FORM)).length == 0) {
            $this._submitData();
        } else {
            var originalFileInputs = $(':file', $this.DIALOG_FORM).map(function (e, element) {
                if (!submitButton) {
                    return element;
                }
                var container = submitButton.parent();
                if ($('#' + $(element).attr('id'), container).length > 0) {
                    return element;
                } else {
                    return null;
                }
            });
            if (settings.asyncFileUpload && files) {
                if ($($this.DIALOG_FORM).serializeArray().length > 1) {
                    $this._submitAsyncData(originalFileInputs, formData);
                } else {
                    $this._submitFile(originalFileInputs, formData);
                }
            } else if (!files) {
                $this._submitFile(originalFileInputs, formData);
            } else {
                $($this.DIALOG_FORM).fileupload('option', 'url', $($this.DIALOG_FORM).attr('action') || settings.url);
                $($this.DIALOG_FORM).fileupload('option', 'formData', formData);
                $($this.DIALOG_FORM).fileupload('send', files);
            }
        }

        delete(files);
    };

    this._submitData = function () {
        var formData = $($this.DIALOG_FORM).serializeArray();
        var url = $($this.DIALOG_FORM).attr('action') || settings.url;

        formAction = '';
        formIsValid = false;

        if (submitButton) {
            formData.push({
                name: submitButton.attr('name'),
                value: submitButton.attr('name')
            });
        }

        if (settings.submitInNewTab) {
            this._submitInNewTab(url, formData);
            this._eClose();
        } else {
            $.ajax({
                type: 'POST',
                url: url,
                data: formData,
                success: function (data, b, xhr) {
                    $this._eEvalSubmit(data, b, xhr, formData);
                }
            });
        }
    };

    this._submitFile = function (fileInputs, formData) {
        formAction = '';
        formIsValid = false;
        formData = formData || [];

        if (files && fileInputs.length <= 1) {
            $($this.DIALOG_FORM).fileupload('option', 'url', $($this.DIALOG_FORM).attr('action') || settings.url);
            $($this.DIALOG_FORM).fileupload('option', 'formData', formData);
            $($this.DIALOG_FORM).fileupload('send', files);
        } else {
            var fileCount = 0, progressInterval = null;
            fileInputs.each(function () {
                fileCount += $(this).val() != '';
            });
            if (fileCount > 0) {

                $this.CONTAINER_LOADING.html('\
                    <div style="background:white;padding:1px 10px; width: 500px">\
                        <p>\
                            Uploading ' + fileCount + ' file(s)\
                            <a href="#" class="ui-dialog-titlebar-minimize">to background</a>\
                        </p>\
                        <div id="progressbar"></div>\
                        <p>\
                            <span style="float:right;color:grey">progress: not available</span>\
                            uploading ... \
                        </p>\
                    </div>\
                ');
                $this.CONTAINER_LOADING.find('.ui-dialog-titlebar-minimize').click(function () {
                    $this._eBackground();
                    return false;
                });

                var progressbar = $('#progressbar', $this.CONTAINER_LOADING);
                var progressValue = 0;
                progressInterval = setInterval(function () {
                    if (progressValue == 100) progressValue = 0;
                    progressbar.progressbar({
                        value: ++progressValue
                    });
                }, 25);

                progressbar.parent().position({
                    my: 'center',
                    at: 'center',
                    of: progressbar.parent().parent()
                });
            }

            formData.push({
                name: 'iframe-submit',
                value: true
            });
            $.ajax({
                type: 'POST',
                url: $($this.DIALOG_FORM).attr('action') || settings.url,
                dataType: "iframe html",
                fileInput: fileInputs,
                formData: formData,
                success: function (data, b, xhr) {
                    clearInterval(progressInterval);
                    $this._eEvalSubmit(data, b, xhr);
                }
            });
        }

    };

    // Strips HTML and PHP tags from a string
    // returns 1: 'Kevin <b>van</b> <i>Zonneveld</i>'
    // example 2: strip_tags('<p>Kevin <img src="someimage.png" onmouseover="someFunction()">van <i>Zonneveld</i></p>', '<p>');
    // returns 2: '<p>Kevin van Zonneveld</p>'
    // example 3: strip_tags("<a href='http://kevin.vanzonneveld.net'>Kevin van Zonneveld</a>", "<a>");
    // returns 3: '<a href='http://kevin.vanzonneveld.net'>Kevin van Zonneveld</a>'
    // example 4: strip_tags('1 < 5 5 > 1');
    // returns 4: '1 < 5 5 > 1'
    this.strip_tags = function (str, allowed_tags) {

        var key = '', allowed = false;
        var matches = [];
        var allowed_array = [];
        var allowed_tag = '';
        var i = 0;
        var k = '';
        var html = '';
        var replacer = function (search, replace, str) {
            return str.split(search).join(replace);
        };
        // Build allowes tags associative array
        if (allowed_tags) {
            allowed_array = allowed_tags.match(/([a-zA-Z0-9]+)/gi);
        }
        str += '';

        // Match tags
        matches = str.match(/(<\/?[\S][^>]*>)/gi);
        // Go through all HTML tags
        for (key in matches) {
            if (isNaN(key)) {
                // IE7 Hack
                continue;
            }

            // Save HTML tag
            html = matches[key].toString();
            // Is tag not in allowed list? Remove from str!
            allowed = false;

            // Go through all allowed tags
            for (k in allowed_array) {            // Init
                allowed_tag = allowed_array[k];
                i = -1;

                if (i != 0) {
                    i = html.toLowerCase().indexOf('<' + allowed_tag + '>');
                }
                if (i != 0) {
                    i = html.toLowerCase().indexOf('<' + allowed_tag + ' ');
                }
                if (i != 0) {
                    i = html.toLowerCase().indexOf('</' + allowed_tag);
                }

                // Determine
                if (i == 0) {
                    allowed = true;
                    break;
                }
            }
            if (!allowed) {
                str = replacer(html, "", str); // Custom replace. No regexing
            }
        }
        return str;
    };

    this._submitAsyncFile = function (files) {
        var formData = [{
            name: 'submit-async-file',
            value: true
        }, {
            name: 'iframe-submit',
            value: true
        }];

        formAction = '';
        formIsValid = false;

        if (submitButton) {
            formData.push({
                name: submitButton.attr('name'),
                value: submitButton.attr('name')
            });
        }

        $this._submitFile(files, formData);
    };

    this._submitAsyncData = function (files, formData) {
        formAction = '';
        formIsValid = false;
        formData = formData || [];
        formData.push({
            name: 'iframe-submit',
            value: true
        });

        $.ajax({
            type: 'POST',
            url: $($this.DIALOG_FORM).attr('action') || settings.url,
            dataType: 'iframe html',
            fileInput: files.clone().val(''),
            formData: formData,
            success: function (data, b, xhr) {
                if ($this._isFormValid(data, b, xhr)) {
                    settings.url = xhr.getResponseHeader('Form-Action') || formAction || settings.url;
                    $this._submitAsyncFile(files);
                } else {
                    $($this.DIALOG_FORM).html($(data).html());

                    files.each(function () {
                        $('#' + $(this).attr('id'), $this.DIALOG_FORM).replaceWith(this);
                    });

                    $this._eOpen();
                    $this._appendAllNodesToDialog(data);
                }
            }
        });
    };


    this._isFormValid = function (data, b, xhr) {
        var retVal = !$.trim(data) || formIsValid; // formIsValid set by iframe

        if (typeof xhr.formIsValid != 'undefined') {
            retVal |= xhr.formIsValid;
        } else {
            retVal |= xhr.getResponseHeader('Form-Validation');
        }

        return retVal;
    };

    this._formatNumber = function (number) {
        number += '';

        while (number.match(/^(\d+)(\d{3}\b)/)) {
            number = number.replace(/^(\d+)(\d{3}\b)/, RegExp.$1 + '.' + RegExp.$2);
        }

        return number;
    };


    this._appendAllNodesToDialog = function (html) {
        for (var i = 1; i < $(html).length; i++) {
            $($this.DIALOG_FORM).append($(html).get(i));
        }
    };

    this._displayHiddenPanelField = function (field) {
        var tabContainer = field.closest($this.SELECTOR_TAB_CONTAINER);
        var tabPane = field.closest($this.SELECTOR_TAB_PANEL);
        tabContainer.tabs('option', 'active', $($this.SELECTOR_TAB_PANEL, tabContainer).index(tabPane));

        var accordionContainer = field.closest($this.SELECTOR_ACCORDION_CONTAINER);
        var accordionPane = field.closest($this.SELECTOR_ACCORDION_PANEL);
        accordionContainer.accordion('activate', $($this.SELECTOR_ACCORDION_PANEL, accordionContainer).index(accordionPane));
    };

    this._focusField = function () {
        var invalidFields = $($this.SELECTOR_INVALID_FIELDS, $this.DIALOG_FORM).filter(':visible');

        if (invalidFields.length > 0) {
            $this._displayHiddenPanelField(invalidFields.first());
            invalidFields.first().focus()
        } else {
            $($this.SELECTOR_WRITEABLE_FIELDS, $this.DIALOG_FORM).filter(':visible').first().focus();
        }
    };

    this._submitInNewTab = function (url, params) {
        var form = $('<form target="_blank" method="POST" style="display:none;"></form>').attr({
            action: url
        }).appendTo(document.body);

        $.each(params, function () {
            $('<input type="hidden" />').attr(this).appendTo(form);
        });

        form.submit();
        form.remove();
    };

    this.init(params);
}

DialogForm.loadTextEditor = function (element) {
    tinyMCE.baseURL = '/rc/plugin/tiny_mce';
    var options = {
        popup_css_add: '/rc/plugin/tiny_mce/themes/advanced/skins/default/dialog.css',
        mode: 'exact',
        theme: 'advanced',
        body_class: 'html-text-editor-content',
        width: element.width() + 'px',
        height: element.height() + 'px',
        convert_urls: false,
        // extended_valid_elements: "p,span[*],delete[*],insert[*]",
        theme_advanced_toolbar_location: 'top',
        theme_advanced_toolbar_align: 'left',
        theme_advanced_statusbar_location: 'bottom',
        setup: function (ed) {
            ed.onInit.add(function (ed) {
                if (element.data('background-color')) {
                    ed.getBody().style.backgroundColor = element.data('background-color');
                }
            });
        }
    };

    if (element.data('scope') === 'editor' || (!element.data('scope') && location.href.indexOf('/editor') >= 0)) {
        options.content_css = '/rc/css/texteditor_research.css?q=1385298610';
        if ($('#additionalCss').html() !== '') {
            options.content_css += ',/rc/css/texteditor_additional.css?q=1385298610'
        }
        options.plugins = 'pdw,table,advhr,emotions,iespell,inlinepopups,preview,searchreplace,contextmenu,directionality,fullscreen,advlist,ice';
        options.theme_advanced_font_sizes = "8pt,10pt,12pt,14pt,18pt,24pt,36pt";
        options.theme_advanced_buttons1 = 'pdw_toggle,bold,italic,underline,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,|,formatselect,fontselect,fontsizeselect,|,preview,fullscreen';
        options.theme_advanced_buttons2 = 'cut,copy,paste,pastetext,pasteword,|,search,replace,|,bullist,numlist,|,outdent,indent,|,undo,redo,|,link,unlink,anchor,image,code,|,forecolor,backcolor';
        options.theme_advanced_buttons3 = 'hr,removeformat,visualaid,|,sub,sup,|,charmap,emotions,|,tablecontrols';
        options.theme_advanced_buttons4 = 'ice_togglechanges,ice_toggleshowchanges,iceacceptall,icerejectall,iceaccept,icereject';
        options.ice = {
            isTracking: $(element.text()).find('.ins:first, .del:first').length > 0,
            user: {name: $('#container-main').data('username'), id: $('#container-main').data('userid')},
            preserveOnPaste: 'p,a[href],i,em,strong,',
            css: 'https://www.researchcatalogue.net/rc/plugin/tiny_mce/plugins/ice/css/ice.css'
        };
        options.pdw_toggle_on = true;
        options.pdw_toggle_toolbars = '2,3,4';
    } else if (element.data('scope') === 'admin' || (!element.data('scope') && location.href.indexOf('/admin') >= 0)) {
        $.extend(options, {
            content_css: '/rc/css/texteditor_portal.css?q=1385298610',
            plugins: 'advhr,emotions,inlinepopups,advlist,paste',
            theme_advanced_buttons1: 'bold,italic,underline,strikethrough,|,sub,sup,|,justifyleft,justifycenter,justifyright,justifyfull',
            theme_advanced_buttons2: 'bullist,numlist,|,link,unlink,anchor,image,hr,|,charmap,emotions,|,removeformat,code',
            theme_advanced_buttons3: '',
            valid_children: '+body[style]'
        });
    } else {
        $.extend(options, {
            content_css: '/rc/css/texteditor_portal.css?q=1385298610',
            plugins: 'advhr,emotions,inlinepopups,advlist,paste',
            theme_advanced_buttons1: 'bold,italic,underline,strikethrough,|,sub,sup,|,justifyleft,justifycenter,justifyright,justifyfull',
            theme_advanced_buttons2: 'bullist,numlist,|,link,unlink,anchor,image,hr,|,charmap,emotions,|,removeformat',
            theme_advanced_buttons3: ''
        });
    }

    return element.tinymce(options);
};
