function LayerList(init) {
    const $this = this;

    this.CONTAINER = $('#_exposition');
    this.CONTAINER_CONTENT = $('tbody', this.CONTAINER);
    this.CONTAINER_TABLE = $('.table', this.CONTAINER);
    this.CONTAINER_SCROLL = $('.dataTables_scrollBody', this.CONTAINER);
    this.CONTAINER_TOOLBAR = $('.t-menu', this.CONTAINER);
    this.CONTAINER_LOADING = $('.container-loading', this.CONTAINER);
    this.CONTAINER_TOOLBOX = $('#container-toolbox');
    this.BUTTON_LOCK = $('#layer-lock', this.CONTAINER);
    this.BUTTON_UNLOCK = $('#layer-unlock', this.CONTAINER);
    this.SELECTOR_CHILD = 'tr';

    this.load = function () {
        var data = $('<table/>');

        Editor.ContentList.children().each(function (i, item) {
            $this._createChild(item).appendTo(data);
        });

        List.prototype._eLoad.call(this, data.children('tbody').html());
    };

    this.eAfterLoad = function () {
        this.children().each(function () {
            $this.initChild(this);
        });

        $this._updateButtonState();
    };

    this.loadChild = function (id) {
        if (this.child(id).length === 0) {
            this.dataTable().row.add(this._createChild(Editor.ContentList.child(id)).get(0)).draw();
        }

        List.prototype._eLoadChild.call(this, this.child(id));
    };

    this.eAfterLoadChild = function (data) {
        this.initChild(data);

        this.dataTable().draw();
    };

    this.eLockChild = function (id, options) {
        this.BUTTON_LOCK.toggle(!options.lock);
        this.BUTTON_UNLOCK.toggle(options.lock);
        this._updateLock([id], options);
    };

    this.eLockChildren = function (ids, options) {
        this.BUTTON_LOCK.toggle(!options.lock);
        this.BUTTON_UNLOCK.toggle(options.lock);
        this._updateLock(ids, options);
    };

    this.eAfterSelectChild = function (id, options) {
        this._updateButtonState();

        if (options.context != this && options.child.length > 0) {
            jQueryNew(this.CONTAINER_SCROLL).stop().scrollTo(options.child, this.SCROLL_DURATION);
        }
    };

    this.eAfterToggleChildSelection = function () {
        this._updateButtonState();
    };

    this.initChild = function (item) {
        item = $(item);
        var itemId = item.data('id');

        item.click(function (event) {
            const options = {context: $this},
                selectedChildren = $this.selectedChildren();

            if (event.ctrlKey || event.metaKey || (selectedChildren.length === 1 && selectedChildren.is(item))) {
                $this.toggleChildSelection(itemId, options);
                Editor.ContentList.toggleChildSelection(itemId, options);
            } else {
                $this.selectChild(itemId, options);
                Editor.ContentList.selectChild(itemId, options);
            }
        }).dblclick(function () {
            if (item.data('tool') === 'comment') {
                return;
            }
            Dialog.editItem(itemId);
        }).disableSelection();

        if (item.data('locked') == 1) {
            this.lockChildren([itemId], {
                lock: true
            });
        }
    };

    this.layersToFront = function (children) {
        let newOrder = [];

        this.sortedChildren().each(function () {
            if (!$(children).is(this)) {
                newOrder.push(this);
            }
        });

        $(children).each(function () {
            newOrder.push(this);
        });

        this._updateOrder(newOrder);

        jQueryNew(this.CONTAINER_SCROLL).stop().scrollTo($(children).first(), this.SCROLL_DURATION);
    };

    this.layersToBack = function (children) {
        let newOrder = $(children).toArray();

        this.sortedChildren().each(function () {
            if (!$(children).is(this)) {
                newOrder.push(this);
            }
        });

        this._updateOrder(newOrder);

        jQueryNew(this.CONTAINER_SCROLL).stop().scrollTo($(children).first(), this.SCROLL_DURATION);
    };

    this.bringInFrontOf = function (itemId, childId) {
        let newOrder = [];

        this.sortedChildren().each(function () {
            if ($(this).attr('data-id') == itemId) {
                return;
            }

            if ($(this).attr('data-id') == childId) {
                newOrder.push($this.child(itemId).get(0));
            }

            newOrder.push(this);
        });

        this._updateOrder(newOrder);

        jQueryNew(this.CONTAINER_SCROLL).stop().scrollTo(this.child(itemId), this.SCROLL_DURATION);
    };

    this.bringInBackOf = function (itemId, childId) {
        let newOrder = [];

        this.sortedChildren().each(function () {
            if ($(this).attr('data-id') == itemId) {
                return;
            }

            newOrder.push(this);

            if ($(this).attr('data-id') == childId) {
                newOrder.push($this.child(itemId).get(0));
            }
        });

        this._updateOrder(newOrder);

        jQueryNew(this.CONTAINER_SCROLL).stop().scrollTo(this.child(itemId), this.SCROLL_DURATION);
    };

    this.init = function () {
        this.CONTAINER_CONTENT.sortable({
            items: 'tr',
            cancel: '.locked',
            axis: 'y',
            distance: 5,
            containment: 'parent',
            tolerance: 'pointer',
            update: function (event, ui) {
                if (ui.item.prev($this.SELECTOR_CHILD).length === 1) {
                    $this.bringInBackOf(ui.item.data('id'), ui.item.prev($this.SELECTOR_CHILD).data('id'));
                } else {
                    $this.bringInFrontOf(ui.item.data('id'), ui.item.next($this.SELECTOR_CHILD).data('id'));
                }
            }
        });

        this.selectChild(null);
    };

    this._updateOrder = function (children) {
        let data = {},
            index = 1;

        $.each(children, function (_, item) {
            const target = Editor.ContentList.child($(item).data('id'));

            target.css('zIndex', index);
            data[target.data('id')] = index;
            $('td:last', $this.child($(item).data('id'))).html(index);

            ++index;
        });

        $.post('/item/update-z-index', {
            index: data
        });

        $this.CONTAINER_TABLE.parent().css('height', '');

        jQueryNew($this.CONTAINER_TABLE)
            .DataTable()
            .cells('.transient')
            .invalidate()
            .draw();
    };

    this._updateLock = function (ids, options) {
        if (options.save) {
            var data = {};
            for (var i = 0; i < ids.length; i++) {
                data[ids[i]] = options.lock ? 1 : 0;
            }
            $.post('/item/update-lock', {
                lock: data
            });
        }
    };

    this._updateButtonState = function (children) {
        children = children || this.selectedChildren();
        switch (children.length) {
            case 0:
                $('li[data-context!="global"]', this.CONTAINER_TOOLBAR).addClass('disabled');

                this.BUTTON_LOCK.show();
                this.BUTTON_UNLOCK.hide();
                break;

            case 1:
                $('li[data-context!="global"]', this.CONTAINER_TOOLBAR).removeClass('disabled');

                const unlock = children.hasClass(this.CLASS_LOCKED);
                this.BUTTON_LOCK.toggle(!unlock);
                this.BUTTON_UNLOCK.toggle(unlock);
                break;

            default:
                $('li[data-context="single"]', this.CONTAINER_TOOLBAR).addClass('disabled');
                $('li[data-context!="single"]', this.CONTAINER_TOOLBAR).removeClass('disabled');

                const lock = children.filter(':not(.' + this.CLASS_LOCKED + ')').length > 0;
                this.BUTTON_LOCK.toggle(lock);
                this.BUTTON_UNLOCK.toggle(!lock);
        }
    };

    this._createChild = function (contentItem) {
        contentItem = $(contentItem);
        return $('<tr class="layer">' +
            '<td class="' + contentItem.data('tool') + '-icon mif-2x"></td>' +
            '<td>' + contentItem.data('title') + '</td>' +
            '<td>' + contentItem.data('date') + '</td>' +
            '<td class="transient" style="text-align:right">' + contentItem.css('zIndex') + '</td>' +
            '</tr>')
            .attr('data-id', contentItem.data('id'))
            .attr('data-locked', contentItem.data('locked'))
            .attr('data-tool', contentItem.data('tool'))
            .attr('data-creator-id', contentItem.attr('data-creator-id') + '')
            .attr('data-editable', contentItem.data('editable'));
    };

    this.sortedChildren = function () {
        return this.children().sort(function (a, b) {
            const aIndex = parseInt(Editor.ContentList.child($(a).attr('data-id')).css('zIndex')),
                bIndex = parseInt(Editor.ContentList.child($(b).attr('data-id')).css('zIndex'));

            return (aIndex < bIndex) ? -1 : ((aIndex > bIndex) ? 1 : 0);
        });
    };

    this.setupDataTable = function () {
        jQueryNew(this.CONTAINER_TABLE).DataTable({
            columnDefs: [
                {
                    orderable: true
                }
            ],
            columns: [
                {type: 'string', orderDataType: 'rc-tool', width: 40},
                {type: 'string'},
                {type: 'date', orderDataType: 'rc-date', width: 85},
                {type: 'num', width: 65}
            ],
            fixedColumns: true,
            bAutoWidth: false,
            preDrawCallback: function () {
                const parent = $this.CONTAINER_TABLE.parent();

                parent.css('height', $(window).height() - $this.CONTAINER_TABLE.offset().top - 15);
            },
            scrollY: $(window).height() - this.CONTAINER_TABLE.offset().top - 15,
            searching: true,
            paging: false,
            stateSave: false,
            info: false,
            classes: {
                sSortAsc: 'sort-asc sortable-column',
                sSortDesc: 'sort-desc sortable-column',
                sSortable: 'sortable-column',
                sFilterInput: 'border-left-none border-top-none border-bottom-none'
            },
            oLanguage: {
                sSearch: '' +
                '   <div class="border bd-default" style="margin-bottom:10px; overflow:hidden; width:357px">\n' +
                '        <ul class="t-menu horizontal open compact">\n' +
                '            <li data-context="global" style="width:237px">_INPUT_</li>\n' +
                '            <li data-context="single"><a href="#" onclick="Dialog.editItem(Editor.LayerList.selectedChildId());return false" data-role="hint" data-hint-position="bottom" data-hint-background="bg-black" data-hint-color="fg-white" data-hint-mode="2" data-hint-text="Edit"><span class="icon mif-pencil"></span></a></li>\n' +
                '            <li data-context="multiple" id="layer-lock"><a href="#" onclick="Editor.lockItems(Editor.LayerList.selectedChildrenIds(), true);return false" data-role="hint" data-hint-position="bottom" data-hint-background="bg-black" data-hint-color="fg-white" data-hint-mode="2" data-hint-text="Lock/Unlock"><span class="icon mif-lock"></span></a></li>\n' +
                '            <li data-context="multiple" id="layer-unlock"><a href="#" onclick="Editor.lockItems(Editor.LayerList.selectedChildrenIds(), false);return false"><span class="icon mif-unlock"></span></a></li>\n' +
                '            <li data-context="multiple"><a href="#" onclick="Dialog.removeItems(Editor.LayerList.selectedChildrenIds());return false" data-role="hint" data-hint-position="bottom" data-hint-background="bg-black" data-hint-color="fg-white" data-hint-mode="2" data-hint-text="Delete"><span class="icon mif-bin"></span></a></li>\n' +
                '        </ul>\n' +
                '    </div>',
                sSearchPlaceholder : 'Search...'
            }
        }).on('draw', function () {
            $this._updateButtonState();
        });

        this.BUTTON_LOCK = $('#layer-lock', this.CONTAINER);
        this.BUTTON_UNLOCK = $('#layer-unlock', this.CONTAINER);
    };

    if (!LayerList.prototype.init) {
        LayerList.prototype = new TableList(init);
        LayerList.prototype.constructor = LayerList;
    }

    if (init) this.init();
}

new LayerList();