function WeaveList(init) {
    var $this = this;

    this.URL = '/editor/weaves?research=' + Editor.getResearchId();
    this.CONTAINER = $('#_weaves');
    this.CONTAINER_CONTENT = $('tbody', this.CONTAINER);
    this.CONTAINER_TABLE = $('.table', this.CONTAINER);
    this.CONTAINER_SCROLL = $('.dataTables_scrollBody', this.CONTAINER);
    this.CONTAINER_TOOLBAR = $('.t-menu', this.CONTAINER);
    this.CONTAINER_LOADING = $('.container-loading', this.CONTAINER);
    this.SELECTOR_CHILD = 'tr';

    this.setupDataTable = function () {
        jQueryNew(this.CONTAINER_TABLE).DataTable({
            columnDefs: [
                {
                    orderable: true
                }
            ],
            columns: [
                {type: 'string'},
                {type: 'date', orderDataType: 'rc-date', width: 85}
            ],
            fixedColumns: true,
            bAutoWidth: false,
            preDrawCallback: function () {
                const parent = $this.CONTAINER_TABLE.parent();

                parent.css('height', $(window).height() - $this.CONTAINER_TABLE.offset().top - 15);
            },
            scrollY: $(window).height() - this.CONTAINER_TABLE.offset().top - 15,
            searching: true,
            paging: false,
            stateSave: false,
            info: false,
            classes: {
                sSortAsc: 'sort-asc sortable-column',
                sSortDesc: 'sort-desc sortable-column',
                sSortable: 'sortable-column',
                sFilterInput: 'border-left-none border-top-none border-bottom-none'
            },
            oLanguage: {
                sSearch: '' +
                '   <div class="border bd-default" style="margin-bottom:10px; overflow:hidden; width:357px; height: 36px;">\n' +
                '        <ul class="t-menu horizontal open compact">\n' +
                '            <li data-context="global" style="width:158px">_INPUT_</li>\n' +
                '            <li data-context="global" ' + tooltip('Create new page', 'left') + '><a href="#" onclick="Dialog.addWeave(Editor.getResearchId());return false"><span class="icon mif-plus" style="font-size: 14px; margin-top: 4px;"></span></a></li>\n' +
                '            <li data-context="single" ' + tooltip('Open preview', 'left') + '><a href="#" onclick="Editor.WeaveList.preview(Editor.WeaveList.selectedChildId());return false"><span class="icon mif-eye"></span></a></li>\n' +
                '            <li data-context="single" ' + tooltip('Open workspace', 'left') + '><a href="#" onclick="Editor.WeaveList.openEditor(Editor.WeaveList.selectedChildId());return false"><span class="icon mif-wrench"></span></a></li>\n' +
                '            <li data-context="single" ' + tooltip('Edit settings', 'left') + '><a href="#" onclick="Dialog.editWeave(Editor.WeaveList.selectedChildId());return false"><span class="icon mif-pencil"></span></a></li>\n' +
                '            <li data-context="single" ' + tooltip('Delete selected page(s)', 'left') + '><a href="#" onclick="Dialog.removeWeave(Editor.WeaveList.selectedChildId());return false"><span class="icon mif-bin"></span></a></li>\n' +
                '        </ul>\n' +
                '    </div>',
                sSearchPlaceholder: 'Search...'
            }
        }).on('draw', function () {
            $this._updateButtonState();
        });
    };

    this.reload = function () {
        this.dataTable().clear().draw();
        this.load();
    };

    this.openEditor = function (id) {
        if (id) {
            location.href = '/editor?research=' + Editor.getResearchId() + '&weave=' + id;
        }
    };

    this.preview = function (id) {
        if (id) {
            window.open('/view/' + Editor.getResearchId() + '/' + id, 'preview');
        }
    };

    this.eAfterLoad = function () {
        this.children().each(function () {
            $this.initChild(this);
        });

        $this._updateButtonState();
    };

    this.eAfterSelectChild = function (id, options) {
        this._updateButtonState();

        const item = this.child(id);
        if (options.context === this || item.length === 0) {
            return;
        }

        jQueryNew($this.CONTAINER_SCROLL).stop().scrollTo(item.position().top, 0);
    };

    this.eAfterToggleChildSelection = function () {
        this._updateButtonState();
    };

    this.initChild = function (item) {
        item = $(item);

        const itemId = item.data('id');

        item
            .click(function (event) {
                const options = {context: $this},
                    selectedChildren = $this.selectedChildren();

                if (event.ctrlKey || event.metaKey || (selectedChildren.length === 1 && selectedChildren.is(item))) {
                    $this.toggleChildSelection(itemId, options);
                } else {
                    $this.selectChild(itemId, options);
                }
            })
            .dblclick(function () {
                $this.openEditor(itemId);
            })
            .disableSelection();

        if (itemId == Editor.getWeaveId(false) || itemId == Editor.getWeaveId(true)) {
            item.addClass('highlight');
            this.selectChild(itemId);
        }
    };

    if (!WeaveList.prototype.init) {
        WeaveList.prototype = new TableList(init);
        WeaveList.prototype.constructor = WeaveList;
    }
}

new WeaveList();