<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application\Form\FormElement;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Application\Form\FormElement;

/**
 * @ODM\EmbeddedDocument
 */
class SelectFormElement extends FormElement
{
    const FORM_TYPE = 'select';

    /**
     * @var string[]
     * @ODM\Field(type="collection")
     */
    protected $options = [];

    /**
     * @return string[]
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @param string[] $options
     */
    public function setOptions($options)
    {
        $this->options = $options;
    }

    /**
     * @return string
     */
    public function getRawOptions()
    {
        return implode(',', $this->options);
    }

    /**
     * @param string $options
     */
    public function setRawOptions($options)
    {
        $this->options = preg_split('~\s*,\s*~', $options);
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $properties = [
            'rawoptions' => $this->getRawOptions(),
        ];
        return $properties + parent::toArray();
    }

    /**
     * @inheritDoc
     */
    public function getFormOptions()
    {
        $options = [
            'multiOptions' => ['' => ''] + array_combine($this->options, $this->options),
        ];
        return $options + parent::getFormOptions();
    }

}