<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Core\Acl;

final class AclGroup
{
    const RC_GUEST = 'rc:guest';
    const RC_USER = 'rc:user';

    const USER = 'user:%d';

    const PORTAL = 'portal:%d';
    const PORTAL_ADMIN = 'portal-admin:%d';
    const PORTAL_REVIEWER = 'portal-reviewer:%d';

    /**
     * @param string $group
     * @return string
     */
    public static function getGroupPrefix($group)
    {
        return explode(':', $group)[0];
    }

    /**
     * @return string[]
     */
    public static function RC_GUEST()
    {
        return [self::RC_GUEST];
    }

    /**
     * @return string[]
     */
    public static function RC_USER()
    {
        return [self::RC_USER];
    }

    /**
     * @param string|int|string[]|integer[] $ids
     * @return string|string[]
     */
    public static function USER($ids)
    {
        return self::format(self::USER, $ids);
    }

    /**
     * @param string[]|integer[] $ids
     * @return string[]
     */
    public static function PORTAL($ids)
    {
        return self::format(self::PORTAL, $ids);
    }

    /**
     * @param string|int|string[]|integer[] $ids
     * @return string|string[]
     */
    public static function PORTAL_ADMIN($ids)
    {
        return self::format(self::PORTAL_ADMIN, $ids);
    }

    /**
     * @param string|int|string[]|integer[] $ids
     * @return string|string[]
     */
    public static function PORTAL_REVIEWER($ids)
    {
        return self::format(self::PORTAL_REVIEWER, $ids);
    }

    /**
     * @param string $group
     * @param string[] $ids
     * @return string|string[]
     */
    protected static function format($group, $ids)
    {
        if (is_array($ids)) {
            $retVal = [];
            foreach ($ids as $key => $value) {
                $retVal[] = sprintf($group, $value);
            }
            return array_filter($retVal);
        } else {
            $ids = sprintf($group, $ids);
        }
        return $ids;
    }
}
