<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Media;

use Rc\Domain\Model\Research\Exposition\Weave\Generic\DimensionAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\DimensionAwareTrait;

class Dimension implements DimensionAwareInterface
{

    use DimensionAwareTrait;

    /**
     * @var int
     */
    protected $width;

    /**
     * @var int
     */
    protected $height;

    /**
     * @param int $width
     * @param int $height
     */
    public function __construct($width, $height)
    {
        $this->width = $width;
        $this->height = $height;
    }

    /**
     * @param int $width
     * @param int $height
     */
    public function fit($width, $height)
    {
        $aspectRatio = $this->getAspectRatio();
        if ($this->width != $width) {
            $this->width = $width;
            $this->height = round($this->width / $aspectRatio);
        }
        if ($this->height > $height) {
            $this->height = $height;
            $this->width = round($this->height * $aspectRatio);
        }
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return [
            'width'  => $this->width,
            'height' => $this->height,
        ];
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return $this->width . 'x' . $this->height;
    }
}