<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model;

use Rc\Domain\Model\Media\Dimension;
use Rc\Domain\Model\Media\Resource as MediaResource;
use Rc\Domain\Model\Media\Resource\ImageResource;

/**
 * @ODM\Document
 */
interface MediaInterface extends ModelInterface
{
    /**
     * @return string
     */
    public function getName();

    /**
     * @return boolean
     */
    public function hasMaster();

    /**
     * @return MediaResource
     */
    public function getMaster();

    /**
     * @return MediaResource
     */
    public function getPrimaryDerivative();

    /**
     * @param bool $relative
     * @return string
     */
    public function getUri($relative = false);

    /**
     * @param bool $relative
     * @return string
     */
    public function getExternalUri($relative = false);

    /**
     * @param MediaResource $master
     */
    public function setMaster(MediaResource $master);

    /**
     * @return ImageResource
     */
    public function getSplashScreen();

    /**
     * @param ImageResource|null $splashScreen
     */
    public function setSplashScreen($splashScreen = null);

    /**
     * @return boolean
     */
    public function hasSplashScreen();

    /**
     * @deprecated
     * @param int $width
     * @param int $height
     * @return null|ImageResource
     */
    public function getBestFittingSplashScreen($width, $height);

    /**
     * @return Dimension|null
     */
    public function getResolution();

    /**
     * @return bool
     */
    public function hasResolution();

    /**
     * @return MediaResource[]
     */
    public function getDerivativeList();

    /**
     * @param MediaResource[] $derivativeList
     */
    public function setDerivativeList(array $derivativeList);

    /**
     * @return void
     */
    public function clearDerivativeList();

    /**
     * @return bool
     */
    public function hasDerivatives();

    /**
     * @deprecated use getDerivative()
     * @param string $type
     * @return MediaResource|null
     */
    public function getDerivativeByType($type);

    /**
     * @param string $type
     * @return MediaResource|null
     */
    public function getDerivative($type);

    /**
     * @return MediaResource|null
     */
    public function getDefaultDerivative();

    /**
     * @param string $type
     * @return bool
     */
    public function hasDerivative($type);

    /**
     * @param MediaResource $derivative
     */
    public function addDerivative(MediaResource $derivative);

    /**
     * @return MediaResource[]
     */
    public function getResourceList();

    /**
     * @return string|null
     */
    public function getContentType();

    /**
     * @return bool
     */
    public function isImage();

    /**
     * @return bool
     */
    public function isVideo();

    /**
     * @return bool
     */
    public function isAudio();

    /**
     * @return bool
     */
    public function isPdf();

    /**
     * @return bool
     */
    public function isArchive();

    /**
     * @return \DateTime
     */
    public function getExpires();

    /**
     * @param \DateTime $expires
     */
    public function setExpires($expires);

    /**
     * @return bool
     */
    public function isEmbedded();

    /**
     * @param bool $embedded
     */
    public function setEmbedded($embedded);
}