<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
declare (strict_types=1);

namespace Rc\Domain\Model;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Utility\TokenGenerator;

/**
 * @copyright Copyright (c) 2018 CHECK24 Zahlungssysteme GmbH
 * @ODM\Document
 * @ODM\UniqueIndex(keys={"secret"="asc"})
 * @ODM\UniqueIndex(keys={"entityId"="asc"})
 */
class ObjectShare extends AbstractModel
{
    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $secret;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $entityId;

    /**
     * @return string
     */
    public function getSecret(): string
    {
        return $this->secret;
    }

    /**
     * @param string $secret
     */
    public function setSecret(string $secret): void
    {
        $this->secret = $secret;
    }

    /**
     * @return int
     */
    public function getEntityId(): int
    {
        return $this->entityId;
    }

    /**
     * @param int $entityId
     */
    public function setEntityId(int $entityId): void
    {
        $this->entityId = $entityId;
    }

    /**
     * @param AbstractModel
     */
    public function setEntity(AbstractModel $entity): void
    {
        $this->entityId = $entity->getId();
        $this->secret = TokenGenerator::generateForModel($entity);
    }

}
