<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Portal;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\Media\MediaAwareTrait;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research\Exposition;

/**
 * @ODM\Document
 */
class Issue extends AbstractModel
{

    use MediaAwareTrait;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $number;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $title;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $description;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", inversedBy="issues")
     */
    protected $portal;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $published = false;

    /**
     * @var ArrayCollection|Exposition[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition", mappedBy="portalIssue")
     */
    protected $expositions;

    /**
     * @return string
     */
    public function getNumber()
    {
        return $this->number;
    }

    /**
     * @param string $number
     */
    public function setNumber($number)
    {
        $this->number = $number;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return Portal
     */
    public function getPortal()
    {
        return $this->portal;
    }

    /**
     * @param Portal $portal
     */
    public function setPortal(Portal $portal)
    {
        $this->portal = $portal;
    }

    /**
     * @return bool
     */
    public function isPublished()
    {
        return $this->published;
    }

    /**
     * @param bool $published
     */
    public function setPublished($published)
    {
        $this->published = $published;
    }

    /**
     * @return Exposition[]
     */
    public function getExpositions()
    {
        return $this->expositions->toArray();
    }

    /**
     * @param Exposition[] $expositions
     */
    public function setExpositions($expositions)
    {
        $this->expositions = $expositions;
    }

}
