<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition;

use InvalidArgumentException;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\ArchiveSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\AudioSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\ImageSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\PdfSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\TextSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\VideoSimpleMedia;

class SimpleMediaFactory
{

    /**
     * @var array
     */
    protected static $mediaTypeMap = [
        MediaType::ARCHIVE => ArchiveSimpleMedia::class,
        MediaType::IMAGE   => ImageSimpleMedia::class,
        MediaType::VIDEO   => VideoSimpleMedia::class,
        MediaType::AUDIO   => AudioSimpleMedia::class,
        MediaType::PDF     => PdfSimpleMedia::class,
        MediaType::HTML    => TextSimpleMedia::class,
    ];

    /**
     * @param string $mediaType
     * @return SimpleMediaInterface
     */
    static public function create($mediaType)
    {
        if (!isset(self::$mediaTypeMap[$mediaType])) {
            throw new InvalidArgumentException(
                'Expected one of "' . implode(',', array_keys(self::$mediaTypeMap)) . '", but got "' . $mediaType . '"!',
                1472374035
            );
        }

        return new self::$mediaTypeMap[$mediaType];
    }
}