<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition;

use Rc\Domain\Model\Core\Acl\AclDelegatorAwareInterface;
use Rc\Domain\Model\Media\MediaAwareInterface;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\User;

interface SimpleMediaInterface extends ModelInterface, MediaAwareInterface, AclDelegatorAwareInterface
{

    const DEFAULT_THUMB_WIDTH = 150;
    const DEFAULT_THUMB_HEIGHT = 150;

    /**
     * @return User
     */
    public function getOwner();

    /**
     * @param User $owner
     * @return $this
     */
    public function setOwner(User $owner);

    /**
     * @param User $owner
     * @return bool
     */
    public function isOwner(User $owner);

    /**
     * @return Research
     */
    public function getResearch();

    /**
     * @param Research $research
     */
    public function setResearch(Research $research);

    /**
     * @return Exposition
     */
    public function hasExposition();

    /**
     * @return bool
     */
    public function hasWork();

    /**
     * @return string
     */
    public function getName();

    /**
     * @param string $name
     * @return void
     */
    public function setName($name);

    /**
     * @deprecated
     * @return string
     */
    public function getTitle();

    /**
     * @deprecated
     * @param string $title
     */
    public function setTitle($title);

    /**
     * @return string
     */
    public function getCopyright();

    /**
     * @param string $copyright
     * @return void
     */
    public function setCopyright($copyright);

    /**
     * @deprecated
     * @return string
     */
    public function getCopyrightHolder();

    /**
     * @deprecated
     * @param string $copyright
     */
    public function setCopyrightHolder($copyright);

    /**
     * @return string
     */
    public function getDescription();

    /**
     * @param string $description
     * @return void
     */
    public function setDescription($description);

    /**
     * @return bool
     */
    public function isHidden();

    /**
     * @param bool $hidden
     */
    public function setHidden($hidden);

    /**
     * @return string|null
     */
    public function getMediaType();

    /**
     * @param string $mediaType
     * @return bool
     */
    public function hasMediaType($mediaType);

    /**
     * @return array
     */
    public function getMediaVersions();

    /**
     * @return array
     */
    public function getOptions();
}