<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Exposition\TableOfContent\Anchor;
use Rc\Domain\Model\Research\Exposition\TableOfContent\Anchor\Headline;
use Rc\Domain\Model\Research\Exposition\TableOfContent\Anchor\Point;
use Rc\Domain\Model\Research\Exposition\TableOfContent\Chapter;
use Rc\Domain\Model\Translation;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;

/**
 * @ODM\EmbeddedDocument()
 */
class TableOfContent
{

    /**
     * @var Chapter[]|ArrayCollection
     * @ODM\EmbedMany(targetDocument="\Rc\Domain\Model\Research\Exposition\TableOfContent\Chapter")
     */
    protected $chapters;

    /**
     * @param array $chapters
     */
    public function __construct(array $chapters)
    {
        $chapters = $this->createChapterList($chapters);

        $this->chapters = new ArrayCollection($chapters);
    }

    /**
     * @return Chapter[]
     */
    public function getChapters()
    {
        return $this->chapters->toArray();
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return $this->chapters->map(function (Chapter $chapter) {
            return $chapter->toArray();
        })->toArray();
    }

    /**
     * @param array $chapters
     * @return Chapter[]
     */
    protected function createChapterList(array $chapters)
    {
        $chapterList = [];

        foreach ($chapters as $key => $chapter) {
            if (empty($chapter['weaveid'])) {
                continue;
            }

            $chapterList[$key] = new Chapter(
                WeaveRepository::getById((int)$chapter['weaveid']),
                new Translation((string)$chapter['title']),
                $this->createAnchor($chapter)
            );
        }

        ksort($chapterList);

        return $chapterList;
    }

    /**
     * @param array $chapter
     * @return Anchor|Headline|Point
     */
    protected function createAnchor(array $chapter)
    {
        if (isset($chapter['id'])) {
            return new Headline($chapter['id'], $chapter['level']);
        }

        return new Point((int)$chapter['coordx'], (int)$chapter['coordy']);
    }

}