<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\AbstractDilpsModel;
use Rc\Domain\Model\Core\Acl\AclAwareInterface;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;

/**
 * @ODM\Document()
 * @ODM\DiscriminatorField("_class")
 * @ODM\Index(keys={"exposition.$id"="asc"})
 */
abstract class Weave extends AbstractDilpsModel
{

    const DEFAULT_TITLE = 'default page';

    /**
     * @var Exposition
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research\Exposition", inversedBy="weaves", cascade={"persist"}, storeAs="dbRef")
     */
    protected $exposition;

    public function __construct()
    {
        parent::__construct();
    }

    /**
     * @return AclAwareInterface
     */
    public function getAclEntity()
    {
        return $this->exposition;
    }

    /**
     * @return Exposition
     */
    public function getExposition()
    {
        return $this->exposition;
    }

    /**
     * @param Exposition $exposition
     * @return $this
     */
    public function setExposition(Exposition $exposition)
    {
        if ($this->exposition !== $exposition) {
            $this->exposition = $exposition;
            $exposition->addWeave($this);
        }
        return $this;
    }

    /**
     * @deprecated
     * @return Exposition
     */
    public function getResearch()
    {
        return $this->getExposition();
    }

    /**
     * @deprecated
     * @param Exposition $exposition
     */
    public function setResearch($exposition)
    {
        $this->setExposition($exposition);
    }

    /**
     * @return array
     */
    public function getDefaultAspect()
    {
        return ['x' => 0, 'y' => 0];
    }

    /**
     * @return int
     */
    public function getDefaultAspectX()
    {
        return 0;
    }

    /**
     * @return int
     */
    public function getDefaultAspectY()
    {
        return 0;
    }

    /**
     * @return ToolInterface[]
     */
    public function getItems()
    {
        return [];
    }

    /**
     * @return boolean
     */
    public function hasItems()
    {
        return false;
    }

    /**
     * @return void
     */
    public function removeExternalDependencies()
    {
        //
    }
}