<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave\Generic;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\ImageSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;

trait ImageAwareTrait
{

    /**
     * @var ArrayCollection|ImageSimpleMedia[]
     * @ODM\ReferenceMany(strategy="set", storeAs="dbRef")
     */
    protected $imageList;

    /**
     * @param SimpleMediaInterface $parent
     * @param ImageSimpleMedia|null $image
     */
    public function setImage(SimpleMediaInterface $parent, ImageSimpleMedia $image = null)
    {
        if (empty($image)) {
            $this->imageList->offsetUnset($parent->getId());
        } else {
            $this->imageList->offsetSet($parent->getId(), $image);
        }
    }

    /**
     * @param SimpleMediaInterface $parent
     * @return bool
     */
    public function hasImage(SimpleMediaInterface $parent)
    {
        if (!$this->imageList->offsetExists($parent->getId())) {
            return false;
        }

        try {
            $this->imageList->offsetGet($parent->getId())->getId();
            return true;
        } catch (DocumentNotFoundException $exception) {
            return false;
        }
    }

    /**
     * @param SimpleMediaInterface $parent
     * @return ImageSimpleMedia|null
     */
    public function getImage(SimpleMediaInterface $parent)
    {
        if (!$this->hasImage($parent)) {
            return null;
        }

        return $this->imageList->offsetGet($parent->getId());
    }

    /**
     * @param SimpleMediaInterface $parent
     * @return null
     */
    public function getImageId(SimpleMediaInterface $parent)
    {
        $image = $this->getImage($parent);
        if (!isset($image)) {
            return null;
        }

        return $image->getId();
    }

    /**
     * @return ImageSimpleMedia[]
     */
    public function getImages()
    {
        return $this->imageList->toArray();
    }

    /**
     * @deprecated
     */
    public function getImageRevisions()
    {
        throw new \LogicException(__METHOD__);
    }

    /**
     * @deprecated
     */
    public function useRevisionForImages()
    {
        throw new \LogicException(__METHOD__);
    }

    /**
     * @deprecated
     */
    public function resetRevisionForImages()
    {
        throw new \LogicException(__METHOD__);
    }

}