<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave\Tool\Context;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Model\User;

/**
 * @ODM\EmbeddedDocument
 */
abstract class Context
{

    /**
     * @var array
     * @ODM\Field(type="collection")
     */
    protected $roles = [];

    /**
     * @param array $roles
     */
    public function __construct(array $roles)
    {
        $this->roles = $roles;
    }

    /**
     * @return array
     */
    public function getRoles()
    {
        return $this->roles;
    }

    /**
     * @param ToolInterface $tool
     * @param User $user
     * @return bool
     */
    abstract public function isVisibleTo(ToolInterface $tool, User $user = null);

    /**
     * @param ToolInterface $tool
     * @param User $user
     * @return bool
     */
    abstract public function isEditableBy(ToolInterface $tool, User $user = null);

    /**
     * @param ToolInterface $tool
     * @param User|null $user
     * @return bool
     */
    abstract public function isMetaEditableBy(ToolInterface $tool, User $user = null);

    /**
     * @return bool
     */
    abstract public function isActive();

    /**
     * @return array
     */
    protected function getCurrentContext()
    {
        return ContextRegistry::getContext();
    }
}