<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\User;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Core\FormValuesTrait;
use Rc\Domain\Model\User\Setting\RepositoryInterface;

/**
 * @ODM\EmbeddedDocument
 */
class Settings
{
    use FormValuesTrait;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $language = 'en';

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $announcements = false;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $digest = false;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $uploadNotification = true;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $messageNotification = false;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $hideNavigationTooltip = false;

    /**
     * @var RepositoryInterface[]|ArrayCollection
     * @ODM\EmbedMany()
     */
    protected $repositories;

    public function __construct()
    {
        $this->repositories = new ArrayCollection();
    }

    /**
     * @return string
     */
    public function getLanguage()
    {
        return $this->language;
    }

    /**
     * @param string $language
     * @return void
     */
    public function setLanguage($language)
    {
        $this->language = $language;
    }

    /**
     * @return boolean
     */
    public function getAnnouncements()
    {
        return $this->announcements;
    }

    /**
     * @param boolean $announcements
     * @return void
     */
    public function setAnnouncements($announcements)
    {
        $this->announcements = $announcements;
    }

    /**
     * @return boolean
     */
    public function getDigest()
    {
        return $this->digest;
    }

    /**
     * @param boolean $digest
     * @return void
     */
    public function setDigest($digest)
    {
        $this->digest = $digest;
    }

    /**
     * @return boolean
     */
    public function getUploadNotification()
    {
        return $this->uploadNotification;
    }

    /**
     * @param boolean $uploadNotification
     * @return void
     */
    public function setUploadNotification($uploadNotification)
    {
        $this->uploadNotification = $uploadNotification;
    }

    /**
     * @return boolean
     */
    public function getMessageNotification()
    {
        return $this->messageNotification;
    }

    /**
     * @param boolean $messageNotification
     * @return void
     */
    public function setMessageNotification($messageNotification)
    {
        $this->messageNotification = $messageNotification;
    }

    /**
     * @return boolean
     */
    public function getHideNavigationTooltip()
    {
        return $this->hideNavigationTooltip;
    }

    /**
     * @param boolean $hideNavigationTooltip
     * @return void
     */
    public function setHideNavigationTooltip($hideNavigationTooltip)
    {
        $this->hideNavigationTooltip = $hideNavigationTooltip;
    }

    /**
     * @param string $class
     * @return bool
     */
    public function hasRepository($class)
    {
        return $this->getRepository($class) !== null;
    }

    /**
     * @param string $class
     * @return RepositoryInterface|null
     */
    public function getRepository($class)
    {
        return $this->repositories->filter(function (RepositoryInterface $repository) use ($class) {
            return is_a($repository, $class);
        })->first() ?: null;
    }

    /**
     * @param RepositoryInterface $repository
     */
    public function setRepository(RepositoryInterface $repository)
    {
        $this->removeRepository(get_class($repository));

        $this->repositories->add($repository);
    }

    /**
     * @param string $repository
     */
    public function removeRepository($repository)
    {
        foreach ($this->repositories as $key => $value) {
            if ($repository == get_class($value)) {
                $this->repositories->remove($key);
            }
        }
    }

}