<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Application;

use Rc\Domain\Model\Application\ApplicationForm;
use Rc\Domain\Model\Portal;
use Rc\Domain\Repository\MongoRepository;

/**
 * @method static ApplicationForm|null findById($id)
 * @method static ApplicationForm|null findOne(array $query = [], array $options = [])
 * @method static ApplicationForm getById($id)
 * @method static ApplicationForm[] findByIds(array $id, array $options = [])
 * @method static ApplicationForm[] findAll(array $options = [])
 */
class ApplicationFormRepository extends MongoRepository
{

    /**
     * @param Portal $portal
     * @param string $genre
     * @param array $options
     * @return ApplicationForm[]
     */
    public static function findByPortalAndGenre(Portal $portal, $genre, array $options = [])
    {
        return self::find([
            'genre'       => (string)$genre,
            'portal.uuid' => (string)$portal->getUuid(),
        ], $options);
    }

    /**
     * @param string $field
     * @return bool
     */
    protected static function isMultiLanguageField($field)
    {
        return false;
    }
}