<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Repository\Generic;

use Doctrine\MongoDB\Query\Builder;

class SearchResult extends \ArrayIterator
{

    /**
     * @var Builder
     */
    protected $builder;

    /**
     * @param Builder $builder
     */
    public function __construct(Builder $builder)
    {
        parent::__construct($builder->getQuery()->execute()->toArray(false));
        $this->builder = $builder;
    }

    /**
     * @return array
     */
    public function getResult()
    {
        return $this->getArrayCopy();
    }

    /**
     * @return int
     */
    public function getCurrentPage()
    {
        if (!$this->hasOption('skip') || !$this->hasOption('limit')) {
            return 0;
        }
        return $this->getOption('skip') / $this->getOption('limit');
    }

    /**
     * @return bool
     */
    public function isFirstPage()
    {
        return $this->getCurrentPage() == 0;
    }

    /**
     * @return bool
     */
    public function isLastPage()
    {
        return $this->getCurrentPage() == $this->getTotalPageCount() - 1;
    }

    /**
     * @return int
     */
    public function getTotalPageCount()
    {
        if (!$this->hasOption('limit')) {
            return 1;
        }
        return ceil($this->getTotalCount() / $this->getOption('limit'));
    }

    /**
     * @return int
     */
    public function getTotalCount()
    {
        return $this->builder->getQuery()->count();
    }

    /**
     * @param string $name
     * @return boolean
     */
    protected function hasOption($name)
    {
        return isset($this->builder->debug()[$name]);
    }

    /**
     * @param string $name
     * @return mixed|null
     */
    protected function getOption($name)
    {
        if (!$this->hasOption($name)) {
            return null;
        }
        return $this->builder->debug($name);
    }
}