<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Rc\Domain\Model\Media;
use Rc\Domain\Model\ModelInterface;

/**
 * @method static Media|null findById($id)
 * @method static Media getById($id)
 * @method static Media[] findByIds(array $id, array $options = [])
 * @method static Media[] findAll(array $options = [])
 * @method static Media merge(ModelInterface $object)
 */
class MediaRepository extends MongoRepository
{
    const CHECKSUM_DEFAULT_PROFILE_IMAGE = '0fcc4953eba2997533700ec46227e470';

    /**
     * @param string $checkSum
     * @param array $options
     * @return \Rc\Domain\Model\Media|null
     */
    public static function findByCheckSum($checkSum, array $options = [])
    {
        return self::findOne([
            '$or' => [
                ['master.checkSum' => (string)$checkSum],
                ['splashScreen.checkSum' => (string)$checkSum],
                ['derivativeList.checkSum' => (string)$checkSum],
            ],
        ], $options);
    }

    /**
     * @param string $checkSum
     * @param array $options
     * @return \Rc\Domain\Model\Media|null
     */
    public static function findByMasterCheckSum($checkSum, array $options = [])
    {
        return self::findOne([
            'master.checkSum' => (string)$checkSum,
        ], $options);
    }

    /**
     * @param string $fileName
     * @param array $options
     * @return Media|null
     */
    public static function findByMasterFile($fileName, array $options = [])
    {
        return self::findByMasterCheckSum(md5_file($fileName), $options);
    }

    /**
     * @param string $contentType
     * @param array $options
     * @return Media[]
     */
    public static function findByMasterContentType($contentType, array $options = [])
    {
        return self::find([
            'master.contentType' => (string)$contentType,
        ], $options);
    }

}