<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Portal;

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Repository\MongoRepository;

/**
 * @method static Issue|null findById($id)
 * @method static Issue getById($id)
 * @method static Issue[] findByIds(array $id, array $options = [])
 * @method static Issue[] findAll(array $options = [])
 */
class IssueRepository extends MongoRepository
{

    /**
     * @param Portal $portal
     * @param array $options
     * @return Issue[]
     */
    public static function findByPortal(Portal $portal, array $options = [])
    {
        return self::find([
            'portal.uuid' => $portal->getUuid(),
        ], $options);
    }

    /**
     * @param Portal $portal
     * @param string $number
     * @param array $options
     * @return Issue
     */
    public static function findOneByPortalAndNumber(Portal $portal, $number, array $options = [])
    {
        return self::findOne([
            'portal.uuid' => $portal->getUuid(),
            'number'      => (string)$number,
        ], $options);
    }
}