<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Repository\Research;

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Repository\Generic\SearchResult;
use Rc\Domain\Repository\IdMappingRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\SingleCollectionInheritanceInterface;

/**
 * @method static Degree|null findById($id)
 * @method static Degree getById($id)
 * @method static Degree[] findByIds(array $id, array $options = [])
 * @method static Degree[] findAll(array $options = [])
 * @method static Degree[] findByPortalForReviewing(Portal $portal, array $options = [])
 * @method static Degree[] findByPortalIdForReviewing($portalId, array $options = [])
 */
class DegreeRepository extends ResearchRepository implements SingleCollectionInheritanceInterface
{

    /**
     * @param array $arguments
     * @param array $options
     * @return Degree[]|SearchResult
     */
    public static function search(array $arguments, array $options = [])
    {
        $queryBuilder = self::createQueryBuilder([], $options);

        // title
        if (isset($arguments['title'])) {
            $queryBuilder->addAnd(self::buildSearchRegexForTranslationList('title', $arguments['title']));
        }

        // portal
        if (isset($arguments['portal'])) {
            $portalUuid = IdMappingRepository::getByTypeAndReferenceId(Portal::class, $arguments['portal'])->getReferenceUuid();
            $queryBuilder->addAnd(['$or' => [
                ['portal.uuid' => $portalUuid],
                ['partOfPortal.uuid' => $portalUuid],
                ['additionalPortals.uuid' => $portalUuid],
            ]]);
        }

        // level
        if (isset($arguments['level'])) {
            $queryBuilder->addAnd(['level' => (string)$arguments['level']]);
        }

        // keywords
        if (isset($arguments['keyword'])) {
            $queryBuilder->addAnd(['keywords' => ['$in' => [(string)$arguments['keyword']]]]);
        }

        // fulltext
        if (isset($arguments['fulltext'])) {
            $queryBuilder->text((string)$arguments['fulltext']);
            $queryBuilder->sortMeta('score', 'textScore');
        }

        // accessible condition
        if (!empty($arguments['accessible'])) {
            $queryBuilder->addAnd(self::getIsAccessibleCondition($arguments['accessible'][0], $arguments['accessible'][1]));
        } else {
            $queryBuilder->addAnd(self::getIsAccessibleCondition());
        }

        return new SearchResult($queryBuilder);
    }
}