<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Research\Exposition;

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Repository\MongoRepository;
use Rc\Domain\Repository\Research\Exposition\Weave\ToolRepository;

/**
 * @method static Weave|null findById($id)
 * @method static Weave getById($id)
 * @method static Weave[] findByIds(array $id, array $options = [])
 * @method static Weave[] findAll(array $options = [])
 */
class WeaveRepository extends MongoRepository
{
    /**
     * @param SimpleMediaInterface $simpleMedia
     * @param array $options
     * @return Weave[]
     */
    public static function findBySimpleMedia(SimpleMediaInterface $simpleMedia, array $options = [])
    {
        $weaves = [];
        foreach (ToolRepository::findBySimpleMedia($simpleMedia) as $tool) {
            $weaves[] = $tool->getWeave()->getId();
        }
        return self::findByIds($weaves, $options);
    }

    /**
     * @param Exposition $exposition
     * @param string $genre
     * @param array $options
     * @return Weave[]
     */
    public static function findByExpositionAndTool(Exposition $exposition, $genre, array $options = [])
    {
        $weaveIds = [];
        foreach ($exposition->getWeaves() as $weave) {
            $weaveIds[] = new \MongoId($weave->getUuid());
        }

        $result = self::getClient()
            ->getDocumentCollection(Tool::class)
            ->aggregate([
                [
                    '$match' => [
                        'weave.$id' => ['$in' => $weaveIds],
                        '_class'    => \ToolFactory::getObjectName($genre),
                    ],
                ],
                [
                    '$group' => [
                        '_id' => '$weave',
                    ],
                ],
            ], ['cursor' => true])
            ->toArray();

        return self::find([
            'uuid' => [
                '$in' => array_map(function ($weave) {
                    return (string)$weave['_id']['$id'];
                }, $result),
            ],
        ], $options);
    }
}