<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Research;

use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Snapshot;
use Rc\Domain\Repository\MongoRepository;

/**
 * @method static Snapshot|null findById($id)
 * @method static Snapshot|null findOne(array $query = [], array $options = [])
 * @method static Snapshot getById($id)
 * @method static Snapshot[] findByIds(array $id, array $options = [])
 * @method static Snapshot[] findAll(array $options = [])
 */
class SnapshotRepository extends MongoRepository
{

    /**
     * @var array
     */
    protected static $defaultOrder = ['created' => self::SORT_DESC];

    /**
     * @param Research $research
     * @param array    $options
     * @return Snapshot[]
     * @throws \Rc\Domain\Repository\Exception
     */
    public static function findByResearch(Research $research, array $options = [])
    {
        return self::find([
            'research.uuid' => (string)$research->getUuid(),
            'name'          => ['$exists' => true],
        ], $options);
    }

}
