<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Metrics;

use Liuggio\StatsdClient\Sender\SocketSender;
use Liuggio\StatsdClient\StatsdClient;

class Metrics
{

    /**
     * @var StatsdClient
     */
    protected static $client;

    /**
     * @param string $key
     * @param int $value
     * @param array $tags
     */
    public static function increment($key, $value = 1, array $tags = [])
    {
        self::send('c', $key, $value, $tags);
    }

    /**
     * @param string $key
     * @param int $value
     * @param array $tags
     */
    public static function gauge($key, $value, array $tags = [])
    {
        self::send('g', $key, $value, $tags);
    }

    /**
     * @param string $key
     * @param int $value
     * @param array $tags
     */
    public static function timing($key, $value, array $tags = [])
    {
        self::send('ms', $key, $value, $tags);
    }

    /**
     * @param string $key
     * @param int $value
     * @param array $tags
     */
    public static function set($key, $value, array $tags = [])
    {
        self::send('s', $key, $value, $tags);
    }

    /**
     * @param string $metric
     * @param string $key
     * @param int|float $value
     * @param array $tags
     */
    protected static function send($metric, $key, $value, array $tags)
    {
        $tags = ['environment' => app_env()] + $tags;

        self::client()->send(self::key($key, $tags) . ':' . $value . '|' . $metric);
    }

    /**
     * @return StatsdClient
     */
    protected static function client()
    {
        if (!isset(self::$client)) {
            self::$client = new StatsdClient(new SocketSender('localhost', '8125'), true, false);
        }

        return self::$client;
    }

    /**
     * @param string $key
     * @param array $tags
     * @return string
     */
    protected static function key($key, array $tags)
    {
        foreach ($tags as $tag => $tagValue) {
            $key .= ',' . static::escape($tag) . '=' . static::escape($tagValue);
        }

        return $key;
    }

    /**
     * @param string|mixed $input
     * @return string
     */
    private static function escape($input)
    {
        return str_replace([':', '=', ','], ' ', $input);
    }
}