<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Privacy;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\User;

/**
 * @ODM\Document
 */
class AuditLog extends AbstractModel implements ModelInterface
{
    /**
     * @var int
     */
    protected $id = false;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User")
     */
    protected $user;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $action;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $email;

    /**
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $payload;

    /**
     * @param string $action
     * @param User $user
     * @param array|null $payload
     */
    public function __construct($action, User $user = null, array $payload = null)
    {
        parent::__construct();

        $this->action = $action;
        $this->payload = $payload;

        if (isset($user)) {
            $this->user = $user;
            $this->email = $user->getEmail();
        }
    }

    /**
     * Pushes a value for a specific payload key.
     *
     * @param string $key
     * @param mixed $value
     */
    public function pushPayload($key, $value)
    {
        if (!isset($this->payload[$key])) {
            $this->payload[$key] = [];
        }

        $this->payload[$key][] = $value;
    }
}