<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Privacy;

use Exception;
use Identity;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Privacy\AuditLogRepository;

class AuditLogFactory
{

    /**
     * @param string $action
     * @param User|null $user
     * @param array $payload
     * @return AuditLog
     * @throws Exception
     */
    public static function create($action, User $user = null, array $payload = [])
    {
        $payload += [
            'remote_ip'  => $_SERVER['REMOTE_ADDR'],
            'user_agent' => $_SERVER['HTTP_USER_AGENT'],
        ];

        if (Identity::isUserSwitched()) {
            $payload['original_user_id'] = Identity::getSwitchedUser()->getId();
        }

        return new AuditLog($action, $user, $payload);
    }

    /**
     * @param string $action
     * @param User|null $user
     * @param array $payload
     * @return AuditLog
     * @throws Exception
     */
    public static function createAndPersist($action, User $user = null, array $payload = [])
    {
        $log = self::create($action, $user, $payload);

        AuditLogRepository::persist($log);

        return $log;
    }
}