<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Serializer\Handler;

use JMS\Serializer\DeserializationContext;
use JMS\Serializer\GraphNavigator;
use JMS\Serializer\Handler\HandlerRegistry;
use JMS\Serializer\VisitorInterface;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;

class Weave extends Embed
{
    /**
     * @var HandlerRegistry
     */
    protected $handlerRegistry;

    /**
     * @param HandlerRegistry $handlerRegistry
     */
    public function __construct(HandlerRegistry $handlerRegistry)
    {
        $this->handlerRegistry = $handlerRegistry;
    }

    /**
     * Return format:
     *      array(
     *          array(
     *              'direction' => GraphNavigator::DIRECTION_SERIALIZATION,
     *              'format' => 'json',
     *              'type' => 'DateTime',
     *              'method' => 'serializeDateTimeToJson',
     *          ),
     *      )
     * The direction and method keys can be omitted.
     *
     * @return array
     */
    public static function getSubscribingMethods()
    {
        $methods = [];
        $formats = ['json', 'xml', 'yml'];

        foreach ($formats as $format) {
            $methods[] = [
                'direction' => GraphNavigator::DIRECTION_SERIALIZATION,
                'type'      => \Rc\Serializer\Type\Weave::class,
                'format'    => $format,
                'method'    => 'serialize',
            ];

            $methods[] = [
                'direction' => GraphNavigator::DIRECTION_DESERIALIZATION,
                'type'      => \Rc\Serializer\Type\Weave::class,
                'format'    => $format,
                'method'    => 'deserialize',
            ];
        }

        return $methods;
    }

    /**
     * @param VisitorInterface $visitor
     * @param array $data
     * @param array $type
     * @param DeserializationContext $context
     * @return mixed|object
     */
    public function deserialize(VisitorInterface $visitor, array $data, array $type, DeserializationContext $context)
    {
        $weave = parent::deserialize($visitor, $data, $type, $context);

        if ($weave instanceof TextWeave) {
            $weave->setHtml($this->replaceResourceLinks($weave->getHtml()));
            $weave->setMarkdown($this->replaceResourceLinks($weave->getMarkdown()));
        }

        return $weave;
    }

    /**
     * @param string $content
     * @return string
     */
    protected function replaceResourceLinks($content)
    {
        if (!isset($content)) {
            return null;
        }

        $expositionId = $this->idHandler()->originalExpositionId();
        if (!preg_match_all('~' . sprintf(preg_quote('/text-editor/simple-media-resource?research=%s&simple-media=%s'), $expositionId, '([\d]+)') . '~', $content, $matches)) {
            return $content;
        }

        foreach ($matches[0] as $key => $path) {
            $content = str_replace($path, '/text-editor/simple-media-resource?research=' . $this->map($expositionId) . '&simple-media=' . $this->map($matches[1][$key]), $content);
        }

        return $content;
    }

    /**
     * @return Id
     */
    protected function idHandler()
    {
        return $this->handlerRegistry->getHandler(GraphNavigator::DIRECTION_DESERIALIZATION, \Rc\Serializer\Type\Id::class, 'json')[0];
    }

    /**
     * @param int $id
     * @return int
     */
    protected function map($id)
    {
        return $this->idHandler()->map($id);
    }
}
