<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Service\Ingest;

use Identity;
use Log;
use Rc\Domain\Model\Media\Resource\Type\RoleType;
use Rc\Domain\Model\Media\Resource\VideoResource;

class VideoIngestService extends IngestService
{

    /**
     * @param \Rc\Domain\Model\Media\Resource\VideoResource $master
     * @return \Rc\Domain\Model\Media\Resource
     */
    public static function processSplashScreen($master)
    {
        $target = tempnam(temp_path(), 'ingest_') . '.png';
        $startSecond = min(config('ingest.ffmpeg_convert_startsec'), $master->getDuration());
        $command = str_replace(
            ['SOURCE', 'STARTSEC', 'TARGET'],
            [escapeshellarg($master->getUri()), $startSecond, escapeshellarg($target)],
            config('ingest.ffmpeg_convert')
        );

        exec($command);
        if (!file_exists($target)) {
            return null;
        }

        return ImageIngestService::processResource($target, new RoleType(RoleType::SPLASH_SCREEN));
    }

    /**
     * @param \Rc\Domain\Model\Media\Resource $master
     * @return \Rc\Domain\Model\Media\Resource[]
     */
    public static function processDerivativeList($master)
    {
        Log::infoUpload(
            Identity::getUserId(),
            $master->getContentType(),
            null,
            $master->getCheckSum(),
            $master->getFileSize()
        );

        if (!config('app.allow.transcoding')) {
            Log::debugMedia('transcoding disabled!');
            return [];
        }

        Log::debugMedia(Identity::getUserId(), $master->getContentType(), 'unknown', $master->getCheckSum());

        $inputDirectory = config('services.zencoder.directories.input');
        copy($master->getUri(), $inputDirectory . '/' . $master->getCheckSum() . '.' . $master->getFileExtension());
        return [];
    }

    /**
     * @return VideoResource
     */
    public static function getResourcePrototype()
    {
        return new VideoResource();
    }

    /**
     * @param VideoResource $resource
     */
    protected static function enrichResource($resource)
    {
        $command = str_replace('SOURCE', $resource->getUri(), config('ingest.ffmpeg'));
        exec($command, $result);

        foreach ($result as $row) {
            $row = trim($row);
            if (preg_match('/^Input #0, (.*), from .*:$/', $row, $matches)) {
                $matches[1]; // container
            } elseif (preg_match('/^Duration: (.*):(.*):(.*)\.(.*).*, start: (.*), bitrate: (.*)$/', $row, $matches)) {
                $resource->setDuration(
                    intval($matches[1]) * 3600 +
                    intval($matches[2]) * 60 +
                    intval($matches[3])
                );
                $matches[6]; // bitrate
            } elseif (preg_match('/^Stream #0\.(.*): Video: (.*), (.*), (\d+)x(\d+)(.*)$/', $row, $matches)) {
                $matches[2]; // vcodec
                $matches[3]; // colorspace
                $resource->setWidth(intval($matches[4]));
                $resource->setHeight(intval($matches[5]));
            } elseif (preg_match('/^Stream #0\.(.*): Audio: ([^,]*), ([^,]*), ([^,]*)(.*)$/', $row, $matches)) {
                $matches[2]; // acodec
                $matches[3]; // asamplerate
                $matches[4]; // achannels
            }
        }
    }

}