<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Utility;

class Strings
{

    /**
     * @param string $input
     * @return string
     */
    public static function toLowerCase($input)
    {
        if (!isset($input)) {
            return $input;
        }

        return mb_convert_case($input, MB_CASE_LOWER, 'UTF-8');
    }

    /**
     * @param string $input
     * @return string
     */
    public static function toUpperCase($input)
    {
        if (!isset($input)) {
            return $input;
        }

        return mb_convert_case($input, MB_CASE_UPPER, 'UTF-8');
    }

    /**
     * @param string $input
     * @param string $delimiter
     * @return string
     */
    public static function toSnakeCase($input, $delimiter = '_')
    {
        if (!isset($input)) {
            return $input;
        }

        return ltrim(self::toLowerCase(preg_replace('/[A-Z]/', $delimiter . '$0', $input)), $delimiter);
    }

    /**
     * @param string $input
     * @param string $delimiter
     * @param bool $capitalizeFirstCharacter
     * @return string
     */
    public static function toCamelCase($input, $delimiter = '_', $capitalizeFirstCharacter = false)
    {
        if (!isset($input)) {
            return $input;
        }

        $output = ucwords(str_replace($delimiter, ' ', $input));
        if (!$capitalizeFirstCharacter) {
            $output = lcfirst($output);
        }

        return str_replace(' ', '', $output);
    }

    /**
     * @param string $input
     * @param string $replace
     * @return string
     */
    public static function toAlphaNumeric($input, $replace = '_')
    {
        return preg_replace('~[^A-Z0-9]~i', $replace, $input);
    }

}
